@@ -3,35 +3,32 @@
 *                     U n d o a b l e   C o m m a n d s                         *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
-* This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Lesser General Public                    *
-* License as published by the Free Software Foundation; either                  *
-* version 2.1 of the License, or (at your option) any later version.            *
+* This program is free software; you can redistribute it and/or modify          *
+* it under the terms of the GNU General Public License as published by          *
+* the Free Software Foundation; either version 2 of the License, or             *
+* (at your option) any later version.                                           *
 *                                                                               *
-* This library is distributed in the hope that it will be useful,               *
+* This program is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Lesser General Public License for more details.                               *
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
+* GNU General Public License for more details.                                  *
 *                                                                               *
-* You should have received a copy of the GNU Lesser General Public              *
-* License along with this library; if not, write to the Free Software           *
+* You should have received a copy of the GNU General Public License             *
+* along with this program; if not, write to the Free Software                   *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: Commands.h,v 1.5.4.1 2002/04/30 13:13:07 fox Exp $                           *
+* $Id: Commands.h,v 1.12 2003/10/03 20:58:44 fox Exp $                          *
 ********************************************************************************/
 #ifndef COMMANDS_H
 #define COMMANDS_H
 
 
-/*******************************************************************************/
-
-class FXText;
-
 
 // Undo record for text fragment
 class FXTextCommand : public FXCommand {
+  FXDECLARE_ABSTRACT(FXTextCommand)
 protected:
   FXText *text;     // Text widget
   FXchar *buffer;   // Character buffer
@@ -40,13 +37,14 @@ protected:
   FXint   ins;      // Inserted characters
 public:
   FXTextCommand(FXText* txt,FXint p,FXint ni,FXint nd):text(txt),buffer(NULL),pos(p),del(nd),ins(ni){}
-  virtual FXuint size() const { return del; }
+  virtual FXuint size() const;
   virtual ~FXTextCommand(){FXFREE(&buffer);}
   };
 
 
 // Insert command
 class FXTextInsert : public FXTextCommand {
+  FXDECLARE_ABSTRACT(FXTextInsert)
 public:
   FXTextInsert(FXText* txt,FXint p,FXint ni);
   virtual FXString undoName() const { return "Undo insert"; }
@@ -58,6 +56,7 @@ public:
 
 // Delete command
 class FXTextDelete : public FXTextCommand {
+  FXDECLARE_ABSTRACT(FXTextDelete)
 public:
   FXTextDelete(FXText* txt,FXint p,FXint nd);
   virtual FXString undoName() const { return "Undo delete"; }
@@ -67,5 +66,16 @@ public:
   };
 
 
+// Replace command
+class FXTextReplace : public FXTextCommand {
+  FXDECLARE_ABSTRACT(FXTextReplace)
+public:
+  FXTextReplace(FXText* txt,FXint p,FXint ni,FXint nd);
+  virtual FXString undoName() const { return "Undo replace"; }
+  virtual FXString redoName() const { return "Redo replace"; }
+  virtual void undo();
+  virtual void redo();
+  };
+
 #endif
 
