@@ -5,50 +5,57 @@
 *********************************************************************************
 * Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
-* This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Lesser General Public                    *
-* License as published by the Free Software Foundation; either                  *
-* version 2.1 of the License, or (at your option) any later version.            *
+* This program is free software; you can redistribute it and/or modify          *
+* it under the terms of the GNU General Public License as published by          *
+* the Free Software Foundation; either version 2 of the License, or             *
+* (at your option) any later version.                                           *
 *                                                                               *
-* This library is distributed in the hope that it will be useful,               *
+* This program is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Lesser General Public License for more details.                               *
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
+* GNU General Public License for more details.                                  *
 *                                                                               *
-* You should have received a copy of the GNU Lesser General Public              *
-* License along with this library; if not, write to the Free Software           *
+* You should have received a copy of the GNU General Public License             *
+* along with this program; if not, write to the Free Software                   *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: HelpWindow.cpp,v 1.3 2001/11/21 06:21:31 jeroen Exp $                    *
+* $Id: HelpWindow.cpp,v 1.7 2002/08/29 20:37:26 fox Exp $                       *
 ********************************************************************************/
 #include "fx.h"
-#include "HelpWindow.h"
 #include "icons.h"
 #include "help.h"
 #include <stdio.h>
 #include <stdlib.h>
+#include "Preferences.h"
+#include "Commands.h"
+#include "Hilite.h"
+#include "TextWindow.h"
+#include "Adie.h"
+#include "HelpWindow.h"
 
 
 /*******************************************************************************/
 
-  
+
 FXIMPLEMENT(HelpWindow,FXDialogBox,NULL,0)
 
 
 // Construct help dialog box
-HelpWindow::HelpWindow(FXWindow *owner):
-  FXDialogBox(owner,"Help on Adie",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0, 6,6,6,6, 4,4){
-  
+HelpWindow::HelpWindow(Adie *a):
+  FXDialogBox(a,"Help on Adie",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0, 6,6,6,6, 4,4){
+
   // Bottom part
   FXHorizontalFrame *closebox=new FXHorizontalFrame(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH);
-  FXButton *button=new FXButton(closebox,"&Close",NULL,this,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 20,20,5,5);
+
+  // Destroy window when closed
+  FXButton *button=new FXButton(closebox,"&Close",NULL,this,FXDialogBox::ID_CLOSE,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 20,20,5,5);
 
   // Editor part
   FXHorizontalFrame *editbox=new FXHorizontalFrame(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, 0,0,0,0);
   helptext=new FXText(editbox,NULL,0,TEXT_READONLY|TEXT_WORDWRAP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-  helptext->setVisRows(50);
-  helptext->setVisCols(90);
-  
+  helptext->setVisibleRows(50);
+  helptext->setVisibleColumns(90);
+
   // Fill with help
   helptext->setText(help,strlen(help));
   helptext->setTabColumns(35);
