@@ -3,23 +3,23 @@
 *                        P r e f e r e n c e s   D i a l o g                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
-* This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Lesser General Public                    *
-* License as published by the Free Software Foundation; either                  *
-* version 2.1 of the License, or (at your option) any later version.            *
+* This program is free software; you can redistribute it and/or modify          *
+* it under the terms of the GNU General Public License as published by          *
+* the Free Software Foundation; either version 2 of the License, or             *
+* (at your option) any later version.                                           *
 *                                                                               *
-* This library is distributed in the hope that it will be useful,               *
+* This program is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Lesser General Public License for more details.                               *
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
+* GNU General Public License for more details.                                  *
 *                                                                               *
-* You should have received a copy of the GNU Lesser General Public              *
-* License along with this library; if not, write to the Free Software           *
+* You should have received a copy of the GNU General Public License             *
+* along with this program; if not, write to the Free Software                   *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: Preferences.cpp,v 1.47.4.1 2003/06/20 19:02:07 fox Exp $                     *
+* $Id: Preferences.cpp,v 1.55 2003/09/12 00:55:07 fox Exp $                     *
 ********************************************************************************/
 #include "fx.h"
 #include "FXRex.h"
@@ -29,6 +29,7 @@
 #include <stdlib.h>
 #include "Preferences.h"
 #include "Hilite.h"
+#include "TextWindow.h"
 #include "Adie.h"
 #include "icons.h"
 
@@ -333,8 +334,8 @@ Preferences::Preferences(TextWindow *owner):
   new FXLabel(textboxes,"Regular expression:",NULL,JUSTIFY_LEFT|JUSTIFY_BOTTOM|LAYOUT_FILL_X);
   FXVerticalFrame *patternframe=new FXVerticalFrame(textboxes,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0, 0,0,0,0);
   patterntext=new FXText(patternframe,this,ID_SYNTAX_PATTERN,TEXT_WORDWRAP|HSCROLLER_NEVER|VSCROLLER_NEVER|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-  patterntext->setVisRows(1);
-  patterntext->setVisCols(40);
+  patterntext->setVisibleRows(1);
+  patterntext->setVisibleColumns(40);
 
 
   //// Syntax settings button
@@ -349,9 +350,9 @@ Preferences::Preferences(TextWindow *owner):
 
   // Fill styles
   stylecombo->appendItem("None");
-  for(i=0; i<owner->getNumStyles(); i++){
-    stylelist->appendItem(owner->getStyleName(i));
-    stylecombo->appendItem(owner->getStyleName(i));
+  for(i=0; i<getApp()->getNumStyles(); i++){
+    stylelist->appendItem(getApp()->getStyleName(i));
+    stylecombo->appendItem(getApp()->getStyleName(i));
     }
 
   // Fill languages
@@ -373,7 +374,7 @@ Preferences::Preferences(TextWindow *owner):
 long Preferences::onCmdStyleName(FXObject*,FXSelector,void*){
   FXint curstyle=stylelist->getCurrentItem();
   FXString name=stylename->getText();
-  getOwner()->setStyleName(curstyle,name);
+  getApp()->setStyleName(curstyle,name);
   stylelist->setItemText(curstyle,name);
   stylecombo->setItemText(curstyle+1,name);
   return 1;
@@ -384,12 +385,12 @@ long Preferences::onCmdStyleName(FXObject*,FXSelector,void*){
 long Preferences::onUpdStyleName(FXObject* sender,FXSelector,void*){
   FXint curstyle=stylelist->getCurrentItem();
   if(0<=curstyle){
-    FXString name=getOwner()->getStyleName(curstyle);
-    sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&name);
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
+    FXString name=getApp()->getStyleName(curstyle);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&name);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
     }
   else{
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
     }
   return 1;
   }
@@ -398,12 +399,12 @@ long Preferences::onUpdStyleName(FXObject* sender,FXSelector,void*){
 // Change style flags
 long Preferences::onCmdStyleFlags(FXObject*,FXSelector sel,void*){
   FXint curstyle=stylelist->getCurrentItem();
-  FXHiliteStyle style=getOwner()->getStyleColors(curstyle);
-  switch(SELID(sel)){
+  FXHiliteStyle style=getApp()->getStyleColors(curstyle);
+  switch(FXSELID(sel)){
     case ID_STYLE_UNDERLINE: style.style^=FXText::STYLE_UNDERLINE; break;
     case ID_STYLE_STRIKEOUT: style.style^=FXText::STYLE_STRIKEOUT; break;
     }
-  getOwner()->setStyleColors(curstyle,style);
+  getApp()->setStyleColors(curstyle,style);
   return 1;
   }
 
@@ -412,17 +413,17 @@ long Preferences::onCmdStyleFlags(FXObject*,FXSelector sel,void*){
 long Preferences::onUpdStyleFlags(FXObject* sender,FXSelector sel,void*){
   FXint curstyle=stylelist->getCurrentItem();
   if(0<=curstyle){
-    FXHiliteStyle style=getOwner()->getStyleColors(curstyle);
+    FXHiliteStyle style=getApp()->getStyleColors(curstyle);
     FXuint bit=0;
-    switch(SELID(sel)){
+    switch(FXSELID(sel)){
       case ID_STYLE_UNDERLINE: bit=style.style&FXText::STYLE_UNDERLINE; break;
       case ID_STYLE_STRIKEOUT: bit=style.style&FXText::STYLE_STRIKEOUT; break;
       }
-    sender->handle(this,bit?MKUINT(ID_CHECK,SEL_COMMAND):MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
+    sender->handle(this,bit?FXSEL(SEL_COMMAND,ID_CHECK):FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
     }
   else{
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
     }
   return 1;
   }
@@ -431,8 +432,8 @@ long Preferences::onUpdStyleFlags(FXObject* sender,FXSelector sel,void*){
 // Change style color
 long Preferences::onCmdStyleColor(FXObject*,FXSelector sel,void* ptr){
   FXint curstyle=stylelist->getCurrentItem();
-  FXHiliteStyle style=getOwner()->getStyleColors(curstyle);
-  switch(SELID(sel)){
+  FXHiliteStyle style=getApp()->getStyleColors(curstyle);
+  switch(FXSELID(sel)){
     case ID_STYLE_NORMAL_FG: style.normalForeColor=(FXColor)(FXuval)ptr; break;
     case ID_STYLE_NORMAL_BG: style.normalBackColor=(FXColor)(FXuval)ptr; break;
     case ID_STYLE_SELECT_FG: style.selectForeColor=(FXColor)(FXuval)ptr; break;
@@ -441,7 +442,7 @@ long Preferences::onCmdStyleColor(FXObject*,FXSelector sel,void* ptr){
     case ID_STYLE_HILITE_BG: style.hiliteBackColor=(FXColor)(FXuval)ptr; break;
     case ID_STYLE_ACTIVE_BG: style.activeBackColor=(FXColor)(FXuval)ptr; break;
     }
-  getOwner()->setStyleColors(curstyle,style);
+  getApp()->setStyleColors(curstyle,style);
   return 1;
   }
 
@@ -450,9 +451,9 @@ long Preferences::onCmdStyleColor(FXObject*,FXSelector sel,void* ptr){
 long Preferences::onUpdStyleColor(FXObject* sender,FXSelector sel,void*){
   FXint curstyle=stylelist->getCurrentItem();
   if(0<=curstyle){
-    FXHiliteStyle style=getOwner()->getStyleColors(curstyle);
+    FXHiliteStyle style=getApp()->getStyleColors(curstyle);
     FXColor color=0;
-    switch(SELID(sel)){
+    switch(FXSELID(sel)){
       case ID_STYLE_NORMAL_FG: color=style.normalForeColor; break;
       case ID_STYLE_NORMAL_BG: color=style.normalBackColor; break;
       case ID_STYLE_SELECT_FG: color=style.selectForeColor; break;
@@ -461,11 +462,11 @@ long Preferences::onUpdStyleColor(FXObject* sender,FXSelector sel,void*){
       case ID_STYLE_HILITE_BG: color=style.hiliteBackColor; break;
       case ID_STYLE_ACTIVE_BG: color=style.activeBackColor; break;
       }
-    sender->handle(this,MKUINT(ID_SETINTVALUE,SEL_COMMAND),(void*)&color);
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_SETINTVALUE),(void*)&color);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
     }
   else{
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
     }
   return 1;
   }
@@ -483,7 +484,7 @@ long Preferences::onCmdStyleNew(FXObject*,FXSelector,void*){
   style.hiliteBackColor=0;
   style.activeBackColor=0;
   style.style=0;
-  getOwner()->appendStyle("New",style);
+  getApp()->appendStyle("New",style);
   stylelist->appendItem("New");
   stylecombo->appendItem("New");
   stylelist->setCurrentItem(stylelist->getNumItems()-1);
@@ -493,8 +494,8 @@ long Preferences::onCmdStyleNew(FXObject*,FXSelector,void*){
 
 // Update new style
 long Preferences::onUpdStyleNew(FXObject* sender,FXSelector,void*){
-  FXint nstyles=getOwner()->getNumStyles();
-  sender->handle(this,(nstyles<MAXSTYLES)?MKUINT(ID_ENABLE,SEL_COMMAND):MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+  FXint nstyles=getApp()->getNumStyles();
+  sender->handle(this,(nstyles<MAXSTYLES)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
   return 1;
   }
 
@@ -502,7 +503,7 @@ long Preferences::onUpdStyleNew(FXObject* sender,FXSelector,void*){
 // Delete style
 long Preferences::onCmdStyleDelete(FXObject*,FXSelector,void*){
   FXint curstyle=stylelist->getCurrentItem();
-  getOwner()->deleteStyle(curstyle);
+  getApp()->deleteStyle(curstyle);
   stylelist->removeItem(curstyle);
   stylecombo->removeItem(curstyle);
   return 1;
@@ -512,7 +513,7 @@ long Preferences::onCmdStyleDelete(FXObject*,FXSelector,void*){
 // Update delete style
 long Preferences::onUpdStyleDelete(FXObject* sender,FXSelector,void*){
   FXint curstyle=stylelist->getCurrentItem();
-  sender->handle(this,(0<=curstyle)?MKUINT(ID_ENABLE,SEL_COMMAND):MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+  sender->handle(this,(0<=curstyle)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
   return 1;
   }
 
@@ -596,9 +597,9 @@ long Preferences::onCmdLangDelete(FXObject*,FXSelector,void*){
 // Is a language selected
 long Preferences::onUpdLangSelected(FXObject* sender,FXSelector,void*){
   if(0<=langlist->getCurrentItem())
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
   else
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
   return 1;
   }
 
@@ -686,9 +687,9 @@ long Preferences::onCmdSyntaxNew(FXObject*,FXSelector,void*){
 // Something about pattern changed
 long Preferences::onUpdSyntaxNew(FXObject* sender,FXSelector,void*){
   if(0<=langbox->getCurrentItem())
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
   else
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
   return 1;
   }
 
@@ -735,9 +736,9 @@ long Preferences::onCmdSyntaxChanged(FXObject*,FXSelector,void*){
 // Something about pattern changed
 long Preferences::onUpdSyntaxChanged(FXObject* sender,FXSelector,void*){
   if(0<=patternlist->getCurrentItem() && 0<=langbox->getCurrentItem())
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
   else
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
   return 1;
   }
 
