@@ -3,23 +3,23 @@
 *                  F O X   D e s k t o p   C a l c u l a t o r                  *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2001 by Jeroen van der Zijp.   All Rights Reserved.             *
+* Copyright (C) 2001,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
-* This library is free software; you can redistribute it and/or                 *
-* modify it under the terms of the GNU Lesser General Public                    *
-* License as published by the Free Software Foundation; either                  *
-* version 2.1 of the License, or (at your option) any later version.            *
+* This program is free software; you can redistribute it and/or modify          *
+* it under the terms of the GNU General Public License as published by          *
+* the Free Software Foundation; either version 2 of the License, or             *
+* (at your option) any later version.                                           *
 *                                                                               *
-* This library is distributed in the hope that it will be useful,               *
+* This program is distributed in the hope that it will be useful,               *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                *
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
-* Lesser General Public License for more details.                               *
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 *
+* GNU General Public License for more details.                                  *
 *                                                                               *
-* You should have received a copy of the GNU Lesser General Public              *
-* License along with this library; if not, write to the Free Software           *
+* You should have received a copy of the GNU General Public License             *
+* along with this program; if not, write to the Free Software                   *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: Calculator.h,v 1.18 2001/07/19 22:17:04 jeroen Exp $                     *
+* $Id: Calculator.h,v 1.23 2003/09/10 03:54:37 fox Exp $                        *
 ********************************************************************************/
 #ifndef CALCULATOR_H
 #define CALCULATOR_H
@@ -28,10 +28,6 @@
 /*******************************************************************************/
 
 
-class FXTextField;
-class FXButton;
-class FXFont;
-
 
 // Mini application object
 class Calculator : public FXMainWindow {
@@ -93,7 +89,6 @@ private:
   void lparen();
   void rparen();
 public:
-  long onClose(FXObject*,FXSelector,void*);
   long onCmdAngle(FXObject*,FXSelector,void*);
   long onUpdAngle(FXObject*,FXSelector,void*);
   long onCmdBase(FXObject*,FXSelector,void*);
@@ -253,7 +248,6 @@ public:
     ID_QUESTION,
     ID_BEEP,
     ID_FONT,
-    ID_QUIT,
     ID_BASE,
     ID_BIN=ID_BASE+NUM_BIN,
     ID_OCT=ID_BASE+NUM_OCT,
@@ -325,6 +319,9 @@ public:
   /// Construct calculator dialog
   Calculator(FXApp* a);
 
+  // Close the window and save registry
+  virtual FXbool close(FXbool notify=FALSE);
+  
   /// Create
   virtual void create();
 
