@@ -3,7 +3,7 @@
 *                        G L  C o n t e x t   C l a s s                         *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2000,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,13 +19,14 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGLContext.h,v 1.8 2002/01/18 22:42:53 jeroen Exp $                     *
+* $Id: FXGLContext.h,v 1.11 2003/03/25 13:09:23 fox Exp $                        *
 ********************************************************************************/
 #ifndef FXGLCONTEXT_H
 #define FXGLCONTEXT_H
 
 //////////////////////////////  UNDER DEVELOPMENT  //////////////////////////////
 
+namespace FX {
 
 class FXApp;
 class FXDrawable;
@@ -105,6 +106,7 @@ public:
   virtual ~FXGLContext();
   };
 
+}
 
 #endif
 
