@@ -3,7 +3,7 @@
 *                        M e n u   P a n e   W i d g e t                        *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,11 +19,16 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXMenuPane.h,v 1.6 2001/03/23 07:02:42 jeroen Exp $                      *
+* $Id: FXMenuPane.h,v 1.11 2003/03/25 13:09:23 fox Exp $                         *
 ********************************************************************************/
 #ifndef FXMENUPANE_H
 #define FXMENUPANE_H
 
+#ifndef FXPOPUP_H
+#include "FXPopup.h"
+#endif
+
+namespace FX {
 
 /// Popup menu pane
 class FXAPI FXMenuPane : public FXPopup {
@@ -42,5 +47,6 @@ public:
   virtual FXbool contains(FXint parentx,FXint parenty) const;
   };
 
+}
 
 #endif
