@@ -3,7 +3,7 @@
 *                      M e n u   T i t l e   W i d g e t                        *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXMenuTitle.h,v 1.13 2002/01/18 22:42:54 jeroen Exp $                    *
+* $Id: FXMenuTitle.h,v 1.17 2003/06/06 21:07:48 fox Exp $                       *
 ********************************************************************************/
 #ifndef FXMENUTITLE_H
 #define FXMENUTITLE_H
@@ -28,6 +28,8 @@
 #include "FXMenuCaption.h"
 #endif
 
+namespace FX {
+
 
 class FXPopup;
 
@@ -39,7 +41,7 @@ class FXPopup;
 class FXAPI FXMenuTitle : public FXMenuCaption {
   FXDECLARE(FXMenuTitle)
 protected:
-  FXPopup *pane;
+  FXPopup *pane;        // Pane to pop up
 protected:
   FXMenuTitle(){}
 private:
@@ -88,7 +90,7 @@ public:
   virtual void killFocus();
 
   /// Set popup menu to pop up
-  void setMenu(FXPopup *menu){ pane = menu; }
+  void setMenu(FXPopup *menu){ pane=menu; }
 
   /// Return popup menu
   FXPopup* getMenu() const { return pane; }
@@ -106,6 +108,6 @@ public:
   virtual ~FXMenuTitle();
   };
 
-
+}
 
 #endif
