@@ -3,7 +3,7 @@
 *                 V e r t i c a l   C o n t a i n e r   W i d g e t             *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXVerticalFrame.h,v 1.9 2002/01/18 22:42:55 jeroen Exp $                 *
+* $Id: FXVerticalFrame.h,v 1.12 2003/03/25 13:09:23 fox Exp $                   *
 ********************************************************************************/
 #ifndef FXVERTICALFRAME_H
 #define FXVERTICALFRAME_H
@@ -28,6 +28,7 @@
 #include "FXPacker.h"
 #endif
 
+namespace FX {
 
 
 /**
@@ -39,7 +40,6 @@ class FXAPI FXVerticalFrame : public FXPacker {
   FXDECLARE(FXVerticalFrame)
 protected:
   FXVerticalFrame(){}
-  virtual void layout();
 private:
   FXVerticalFrame(const FXVerticalFrame&);
   FXVerticalFrame& operator=(const FXVerticalFrame&);
@@ -48,6 +48,9 @@ public:
   /// Construct a vertical frame layout manager
   FXVerticalFrame(FXComposite *p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_SPACING,FXint pr=DEFAULT_SPACING,FXint pt=DEFAULT_SPACING,FXint pb=DEFAULT_SPACING,FXint hs=DEFAULT_SPACING,FXint vs=DEFAULT_SPACING);
 
+  /// Perform layout
+  virtual void layout();
+
   /// Return default width
   virtual FXint getDefaultWidth();
 
@@ -55,5 +58,6 @@ public:
   virtual FXint getDefaultHeight();
   };
 
+}
 
 #endif
