@@ -3,7 +3,7 @@
 *                             B i t m a p    O b j e c t                        *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXBitmap.cpp,v 1.35 2002/01/18 22:42:58 jeroen Exp $                     *
+* $Id: FXBitmap.cpp,v 1.48 2003/10/23 04:19:44 fox Exp $                        *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -31,6 +31,7 @@
 #include "FXRectangle.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXVisual.h"
 #include "FXBitmap.h"
@@ -61,9 +62,12 @@
 
 #define DISPLAY(app) ((Display*)((app)->display))
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 
 // Object implementation
 FXIMPLEMENT(FXBitmap,FXDrawable,NULL,0)
@@ -83,44 +87,56 @@ FXBitmap::FXBitmap(FXApp* a,const void *pix,FXuint opts,FXint w,FXint h):FXDrawa
   if(pix) opts&=~BITMAP_OWNED;
   data=(FXuchar*)pix;
   options=opts;
+  /// FIXME
   }
 
 
 // Create bitmap
 void FXBitmap::create(){
   if(!xid){
-    if(getApp()->initialized){
+    if(getApp()->isInitialized()){
       FXTRACE((100,"%s::create %p\n",getClassName(),this));
+
 #ifndef WIN32
 
       // Initialize visual
       visual->create();
 
       // Make pixmap
-      xid=XCreatePixmap(DISPLAY(getApp()),XDefaultRootWindow(DISPLAY(getApp())),width,height,1);
+      xid=XCreatePixmap(DISPLAY(getApp()),XDefaultRootWindow(DISPLAY(getApp())),FXMAX(width,1),FXMAX(height,1),1);
       if(!xid){ fxerror("%s::create: unable to create bitmap.\n",getClassName()); }
+
 #else
 
       // Initialize visual
       visual->create();
 
       // Create uninitialized shape bitmap
-      xid=CreateBitmap(width,height,1,1,NULL);
+      xid=CreateBitmap(FXMAX(width,1),FXMAX(height,1),1,1,NULL);
       if(!xid){ fxerror("%s::create: unable to create bitmap.\n",getClassName()); }
+
 #endif
+
       // Render pixels
       render();
 
-      // Zap data
-      if(!(options&BITMAP_KEEP) && (options&BITMAP_OWNED)){
-        options&=~BITMAP_OWNED;
-        FXFREE(&data);
-        }
+      // If we're not keeping the pixel buffer, release it
+      if(!(options&BITMAP_KEEP)) release();
       }
     }
   }
 
 
+// Release the client-side buffer, free it if it was owned.
+void FXBitmap::release(){
+  if(options&BITMAP_OWNED){
+    options&=~BITMAP_OWNED;
+    FXFREE(&data);
+    }
+  data=NULL;
+  }
+
+
 // Detach bitmap
 void FXBitmap::detach(){
   visual->detach();
@@ -134,7 +150,7 @@ void FXBitmap::detach(){
 // Destroy bitmap
 void FXBitmap::destroy(){
   if(xid){
-    if(getApp()->initialized){
+    if(getApp()->isInitialized()){
       FXTRACE((100,"%s::destroy %p\n",getClassName(),this));
 #ifndef WIN32
 
@@ -151,16 +167,12 @@ void FXBitmap::destroy(){
   }
 
 
+#ifndef WIN32
+
+
 // Render into pixmap
 void FXBitmap::render(){
   if(xid){
-
-    FXTRACE((100,"%s::render bitmap %p\n",getClassName(),this));
-
-    // Do it for X-Windows
-#ifndef WIN32
-
-    // XImage xim;
     register XImage *xim=NULL;
     register Visual *vis;
     register int size;
@@ -169,16 +181,15 @@ void FXBitmap::render(){
     XGCValues values;
     GC gc;
 
-    // Check for legal size
-    if(width<1 || height<1){ fxerror("%s::render: illegal bitmap size.\n",getClassName()); }
+    FXTRACE((100,"%s::render bitmap %p\n",getClassName(),this));
 
-    // Make GC
-    values.foreground=0xffffffff;
-    values.background=0;
-    gc=XCreateGC(DISPLAY(getApp()),xid,GCForeground|GCBackground,&values);
+    // Fill with pixels if there is data
+    if(data && 0<width && 0<height){
 
-    // Just leave if black if no data
-    if(data){
+      // Make GC
+      values.foreground=0xffffffff;
+      values.background=0;
+      gc=XCreateGC(DISPLAY(getApp()),xid,GCForeground|GCBackground,&values);
 
       // Get Visual
       vis=(Visual*)visual->visual;
@@ -207,37 +218,34 @@ void FXBitmap::render(){
       XPutImage(DISPLAY(getApp()),xid,gc,xim,0,0,0,0,width,height);
       FXFREE(&xim->data);
       XDestroyImage(xim);
+      XFreeGC(DISPLAY(getApp()),gc);
       }
+    }
+  }
 
-    // No data, fill with black
-    else{
-      XFillRectangle(DISPLAY(getApp()),xid,gc,0,0,width,height);
-      }
-
-    // We're done
-    XFreeGC(DISPLAY(getApp()),gc);
 
 #else
 
+
+struct BITMAPINFO2 {
+  BITMAPINFOHEADER bmiHeader;
+  RGBQUAD          bmiColors[2];
+  };
+
+
+// Render into pixmap
+void FXBitmap::render(){
+  if(xid){
     register FXuchar *p,*q,bits;
     register FXint nb,i,j,bytes_per_line;
     FXuchar *widedata;
 
-    // Check for legal size
-    if(width<1 || height<1){ fxerror("%s::render: illegal bitmap size.\n",getClassName()); }
+    FXTRACE((100,"%s::render bitmap %p\n",getClassName(),this));
 
-    HDC hdcmem=::CreateCompatibleDC(NULL);
+    // Fill with pixels if there is data
+    if(data && 0<width && 0<height){
 
-    // Fill with data
-    if(data){
-
-      // Now make the device dependent bitmap
-      struct BITMAPINFO2 {
-        BITMAPINFOHEADER bmiHeader;
-        RGBQUAD          bmiColors[2];
-        };
-
-      // Set up the bitmap info
+      // Set up the bitmap info, with fixed black/white palette
       BITMAPINFO2 bmi;
       bmi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
       bmi.bmiHeader.biWidth=width;
@@ -259,6 +267,7 @@ void FXBitmap::render(){
       bmi.bmiColors[1].rgbRed=255;
       bmi.bmiColors[1].rgbReserved=0;
 
+      // Fill temp array
       nb=(width+7)>>3;
       bytes_per_line=((width+31)&~31)>>3;
       FXCALLOC(&widedata,FXuchar,height*bytes_per_line);
@@ -272,23 +281,21 @@ void FXBitmap::render(){
         q+=nb;
         p-=bytes_per_line;
         }
+
+      // Get memory device context
+      HDC hdcmem=::CreateCompatibleDC(NULL);
       if(!SetDIBits(hdcmem,(HBITMAP)xid,0,height,widedata,(BITMAPINFO*)&bmi,DIB_RGB_COLORS)){
         fxerror("%s::render: unable to render pixels\n",getClassName());
         }
-      FXFREE(&widedata);
       GdiFlush();
+      FXFREE(&widedata);
+      ::DeleteDC(hdcmem);
       }
-
-    // No data, fill with black
-    else{
-      SelectObject(hdcmem,(HBITMAP)xid);
-      BitBlt(hdcmem,0,0,width,height,hdcmem,0,0,BLACKNESS);
-      }
-    ::DeleteDC(hdcmem);
-#endif
     }
   }
 
+#endif
+
 
 // Get pixel at x,y
 FXbool FXBitmap::getPixel(FXint x,FXint y) const {
@@ -308,9 +315,7 @@ void FXBitmap::setPixel(FXint x,FXint y,FXbool color){
 void FXBitmap::resize(FXint w,FXint h){
   if(w<1) w=1;
   if(h<1) h=1;
-  width=w;
-  height=h;
-  FXTRACE((100,"%s::resize %p\n",getClassName(),this));
+  FXTRACE((100,"%s::resize(%d,%d) %p\n",getClassName(),w,h,this));
   if(xid){
 
 #ifndef WIN32
@@ -319,7 +324,7 @@ void FXBitmap::resize(FXint w,FXint h){
     XFreePixmap(DISPLAY(getApp()),xid);
 
     // Make new pixmap
-    xid=XCreatePixmap(DISPLAY(getApp()),XDefaultRootWindow(DISPLAY(getApp())),width,height,1);
+    xid=XCreatePixmap(DISPLAY(getApp()),XDefaultRootWindow(DISPLAY(getApp())),w,h,1);
     if(!xid){ fxerror("%s::resize: unable to resize bitmap.\n",getClassName()); }
 
 #else
@@ -328,11 +333,27 @@ void FXBitmap::resize(FXint w,FXint h){
     DeleteObject(xid);
 
     // Create a bitmap compatible with current display
-    xid=CreateBitmap(width,height,1,1,NULL);
+    xid=CreateBitmap(w,h,1,1,NULL);
     if(!xid){ fxerror("%s::resize: unable to resize bitmap.\n",getClassName()); }
 
 #endif
     }
+  /// FIXME below
+/*
+  // Resize data array
+  if(data){
+    if(!(options&IMAGE_OWNED)){       // Need to own array
+      FXMALLOC(&data,FXColor,w*h);
+      options|=IMAGE_OWNED;
+      }
+    else if(w*h!=width*height){
+      FXRESIZE(&data,FXColor,w*h);
+      }
+    }
+*/
+  // Remember new size
+  width=w;
+  height=h;
   }
 
 
@@ -355,19 +376,21 @@ int FXBitmap::ReleaseDC(FXID hdc) const {
 
 
 // Save pixel data only
-void FXBitmap::savePixels(FXStream& store) const {
+FXbool FXBitmap::savePixels(FXStream& store) const {
   FXuint size=height*((width+7)>>3);
   store.save(data,size);
+  return TRUE;
   }
 
 
 // Load pixel data only
-void FXBitmap::loadPixels(FXStream& store){
+FXbool FXBitmap::loadPixels(FXStream& store){
   FXuint size=height*((width+7)>>3);
-  if(options&BITMAP_OWNED) FXFREE(&data);
-  FXMALLOC(&data,FXuchar,size);
+  if(options&BITMAP_OWNED){ FXFREE(&data); }
+  if(!FXMALLOC(&data,FXuchar,size)) return FALSE;
   store.load(data,size);
   options|=BITMAP_OWNED;
+  return TRUE;
   }
 
 
@@ -396,6 +419,7 @@ FXBitmap::~FXBitmap(){
   FXTRACE((100,"FXBitmap::~FXBitmap %p\n",this));
   destroy();
   if(options&BITMAP_OWNED){FXFREE(&data);}
-  data=(FXuchar*)-1;
+  data=(FXuchar*)-1L;
   }
 
+}
