@@ -3,7 +3,7 @@
 *                           C o l o r   D i a l o g                             *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXColorDialog.cpp,v 1.15 2002/01/18 22:42:58 jeroen Exp $                *
+* $Id: FXColorDialog.cpp,v 1.21 2003/10/23 04:19:44 fox Exp $                   *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -32,6 +32,7 @@
 #include "FXRectangle.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXDrawable.h"
@@ -54,9 +55,12 @@
     on a color well.
 */
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 // Map
 FXDEFMAP(FXColorDialog) FXColorDialogMap[]={
   FXMAPFUNC(SEL_CHANGED,FXColorDialog::ID_COLORSELECTOR,FXColorDialog::onChgColor),
@@ -94,14 +98,14 @@ FXColor FXColorDialog::getRGBA() const {
 
 // Forward ColorSelector color change to target [a color well]
 long FXColorDialog::onChgColor(FXObject*,FXSelector,void* ptr){
-  if(target) target->handle(this,MKUINT(message,SEL_CHANGED),ptr);
+  if(target) target->handle(this,FXSEL(SEL_CHANGED,message),ptr);
   return 1;
   }
 
 
 // Forward ColorSelector color command to target [a color well]
 long FXColorDialog::onCmdColor(FXObject*,FXSelector,void* ptr){
-  if(target) target->handle(this,MKUINT(message,SEL_COMMAND),ptr);
+  if(target) target->handle(this,FXSEL(SEL_COMMAND,message),ptr);
   return 1;
   }
 
@@ -134,7 +138,7 @@ void FXColorDialog::load(FXStream& store){
 
 // Cleanup
 FXColorDialog::~FXColorDialog(){
-  colorbox=(FXColorSelector*)-1;
+  colorbox=(FXColorSelector*)-1L;
   }
 
-
+}
