@@ -3,7 +3,7 @@
 *                         C u r s o r - O b j e c t                             *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXCursor.cpp,v 1.22.4.1 2003/06/20 19:02:07 fox Exp $                     *
+* $Id: FXCursor.cpp,v 1.37 2003/10/23 04:19:44 fox Exp $                        *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -31,23 +31,65 @@
 #include "FXRectangle.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXVisual.h"
 #include "FXCursor.h"
 
+/*
+#include <X11/Xcursor/Xcursor.h>
+
+#define O 0
+#define R 0x80ff0000
+#define G 0x8000ff00
+#define B 0x800000ff
+
+const XcursorPixel cursorpixels[]={
+  R,R,R,R,R,R,R,R,R,R,R,R,R,R,R,R,
+  R,R,R,R,R,R,R,R,R,R,R,R,R,R,R,R,
+  R,R,R,R,R,R,R,R,R,R,R,R,R,R,R,R,
+  R,R,R,R,R,R,R,R,R,R,R,R,R,R,R,R,
+  R,R,R,R,R,R,R,R,R,R,R,R,R,R,R,R,
+  G,G,G,G,G,G,G,G,G,G,G,G,G,G,G,G,
+  G,G,G,G,G,G,G,G,G,G,G,G,G,G,G,G,
+  G,G,G,G,G,G,G,G,G,G,G,G,G,G,G,G,
+  G,G,G,G,G,G,G,G,G,G,G,G,G,G,G,G,
+  G,G,G,G,G,G,G,G,G,G,G,G,G,G,G,G,
+  B,B,B,B,B,B,B,B,B,B,B,B,B,B,B,B,
+  B,B,B,B,B,B,B,B,B,B,B,B,B,B,B,B,
+  B,B,B,B,B,B,B,B,B,B,B,B,B,B,B,B,
+  B,B,B,B,B,B,B,B,B,B,B,B,B,B,B,B,
+  B,B,B,B,B,B,B,B,B,B,B,B,B,B,B,B,
+  O,O,O,O,O,O,O,O,O,O,O,O,O,O,O,O
+  };
+
+#undef O
+#undef B
+#undef G
+#undef R
+
+
+const XcursorImage cursorimage[1]={
+  {XCURSOR_IMAGE_VERSION,16,16,16,0,0,0,(XcursorPixel*)cursorpixels}
+  };
+*/
+
 /*
   To do:
-  - Add subclass to read in .CUR file (similar to FXGIFIcon reading
-    e.g. a GIF icon) from an FXStream.
   - Cursor size should be less than or equal to 32x32; this is necessary for Windows!
   - Need standard glyph for "invisible" cursor.
+  - Keep hotx and hoty INSIDE the cursor glyph!!
 */
 
 #define DISPLAY(app) ((Display*)((app)->display))
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
+
 // Object implementation
 FXIMPLEMENT(FXCursor,FXId,NULL,0)
 
@@ -58,8 +100,8 @@ FXCursor::FXCursor(){
   mask=NULL;
   width=0;
   height=0;
-  hotx=-1;
-  hoty=-1;
+  hotx=0;
+  hoty=0;
   glyph=0;
   owned=FALSE;
   }
@@ -72,8 +114,8 @@ FXCursor::FXCursor(FXApp* a,FXStockCursor curid):FXId(a){
   mask=NULL;
   width=0;
   height=0;
-  hotx=-1;
-  hoty=-1;
+  hotx=0;
+  hoty=0;
   glyph=(FXuchar)curid;   // Got to do this, we can't serialize enums (don't know how big they are!)
   owned=FALSE;
   }
@@ -84,10 +126,10 @@ FXCursor::FXCursor(FXApp* a,const void* src,const void* msk,FXint w,FXint h,FXin
   FXTRACE((100,"FXCursor::FXCursor %p\n",this));
   source=(FXuchar*)src;
   mask=(FXuchar*)msk;
-  width=w;
-  height=h;
-  hotx=hx;
-  hoty=hy;
+  width=FXMAX(w,1);
+  height=FXMAX(h,1);
+  hotx=FXCLAMP(0,hx,width-1);
+  hoty=FXCLAMP(0,hy,height-1);
   glyph=0;
   owned=FALSE;
   }
@@ -96,13 +138,13 @@ FXCursor::FXCursor(FXApp* a,const void* src,const void* msk,FXint w,FXint h,FXin
 // Create cursor
 void FXCursor::create(){
   if(!xid){
-    if(getApp()->initialized){
+    if(getApp()->isInitialized()){
       FXTRACE((100,"%s::create %p\n",getClassName(),this));
 
 #ifndef WIN32
 
       // Stock cursor glyphs
-      const FXuint stock[]={XC_top_left_arrow,XC_arrow,XC_xterm,XC_watch,XC_crosshair,XC_sb_h_double_arrow,XC_sb_v_double_arrow,XC_fleur};
+      const FXuint stock[]={XC_left_ptr,XC_right_ptr,XC_xterm,XC_watch,XC_crosshair,XC_sb_h_double_arrow,XC_sb_v_double_arrow,XC_fleur};
       Pixmap srcpix,mskpix;
       XColor color[2];
 
@@ -134,6 +176,7 @@ void FXCursor::create(){
       else{
         FXASSERT(glyph<=ARRAYNUMBER(stock));
         xid=XCreateFontCursor(DISPLAY(getApp()),stock[glyph-1]);
+//xid=XcursorImageLoadCursor(DISPLAY(getApp()),cursorimage);
         }
 
       // Were we successful?
@@ -176,12 +219,12 @@ void FXCursor::create(){
           }
 
         // Always 32x32
-        xid=CreateCursor((HINSTANCE)FXApp::hInstance,hotx,hoty,32,32,tmpand,tmpxor);
+        xid=CreateCursor((HINSTANCE)(getApp()->display),hotx,hoty,32,32,tmpand,tmpxor);
         }
 
       // Building stock cursor
       else{
-        FXASSERT(glyph-1<ARRAYNUMBER(stock));
+        FXASSERT(glyph<=ARRAYNUMBER(stock));
         xid=LoadCursor(NULL,stock[glyph-1]);
         }
 
@@ -189,6 +232,7 @@ void FXCursor::create(){
       if(!xid){ fxerror("%s::create: unable to create cursor.\n",getClassName()); }
 
 #endif
+
       }
     }
   }
@@ -206,8 +250,9 @@ void FXCursor::detach(){
 // Destroy cursor
 void FXCursor::destroy(){
   if(xid){
-    if(getApp()->initialized){
+    if(getApp()->isInitialized()){
       FXTRACE((100,"%s::destroy %p\n",getClassName(),this));
+
 #ifndef WIN32
 
       // Delete cursor
@@ -216,7 +261,9 @@ void FXCursor::destroy(){
 
       // Delete cursor
       DestroyCursor((HCURSOR)xid);
+
 #endif
+
       }
     xid=0;
     }
@@ -224,32 +271,36 @@ void FXCursor::destroy(){
 
 
 // Save pixel data only
-void FXCursor::savePixels(FXStream& store) const {
+FXbool FXCursor::savePixels(FXStream& store) const {
+  FXuint size;
   store << width << height;
   store << hotx << hoty;
   store << glyph;
   if(glyph==0){
-    FXuint size=height*((width+7)>>3);
+    size=height*((width+7)>>3);
     store.save(source,size);
     store.save(mask,size);
     }
+  return TRUE;
   }
 
 
 // Load pixel data only
-void FXCursor::loadPixels(FXStream& store){
+FXbool FXCursor::loadPixels(FXStream& store){
+  FXuint size;
   store >> width >> height;
   store >> hotx >> hoty;
   store >> glyph;
   if(glyph==0){
-    if(owned){FXFREE(&source);FXFREE(&mask);}
-    FXuint size=height*((width+7)>>3);
-    FXMALLOC(&source,FXuchar,size);
-    FXMALLOC(&mask,FXuchar,size);
+    if(owned){ FXFREE(&source); FXFREE(&mask); }
+    size=height*((width+7)>>3);
+    if(!FXMALLOC(&source,FXuchar,size)) return FALSE;
+    if(!FXMALLOC(&mask,FXuchar,size)) return FALSE;
     store.load(source,size);
     store.load(mask,size);
     owned=TRUE;
     }
+  return TRUE;
   }
 
 
@@ -272,8 +323,8 @@ FXCursor::~FXCursor(){
   FXTRACE((100,"FXCursor::~FXCursor %p\n",this));
   destroy();
   if(owned){FXFREE(&source);FXFREE(&mask);}
-  source=(FXuchar*)-1;
-  mask=(FXuchar*)-1;
+  source=(FXuchar*)-1L;
+  mask=(FXuchar*)-1L;
   }
 
-
+}
