@@ -3,9 +3,7 @@
 *           D e v i c e   C o n t e x t   F o r   P r i n t i n g               *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
-*********************************************************************************
-* Contributed by celer@ipro.lug.usf.edu                                         *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -21,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDCPrint.cpp,v 1.26.4.2 2004/03/03 19:02:17 fox Exp $                    *
+* $Id: FXDCPrint.cpp,v 1.38 2003/10/23 04:19:44 fox Exp $                       *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -35,6 +33,7 @@
 #include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXFont.h"
@@ -55,6 +54,7 @@
 
 /*
   Notes:
+  - Contributed by celer@ipro.lug.usf.edu.
   - Coordinate system starts in upper left corner, same as screen [which is
     different from the way PostScript normally does things].
   - Implement the many missing functions.
@@ -70,17 +70,21 @@
     psdc.endPrint();
   - Perhaps feed into FXStream instead of FILE* this might be
     cool to drag and drop [E]PS into apps...
+  - Do we still need the enum's in FXMediaSize if mediasize
+    indexes into the registry database's paper size list?
 */
 
-
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 
 // Construct
 FXDCPrint::FXDCPrint(FXApp* a):FXDC(a){
   font=getApp()->getNormalFont();
-  psout=NULL;
+  psout=NULL;   // FIXME use ctx for this
   mediawidth=0.0;
   mediaheight=0.0;
   mediabb.xmin=0.0;
@@ -136,12 +140,11 @@ void FXDCPrint::bbox(FXfloat x,FXfloat y){
 void FXDCPrint::tfm(FXfloat& xo,FXfloat& yo,FXfloat xi,FXfloat yi){
   if(flags&PRINT_LANDSCAPE){
     xo=yi;
-    yo=xi;
-//    yo=mediaheight-xi;
+    yo=(FXfloat)(mediaheight-xi);
     }
   else{
     xo=xi;
-    yo=mediaheight-yi;
+    yo=(FXfloat)(mediaheight-yi);
     }
   }
 
@@ -218,16 +221,6 @@ FXbool FXDCPrint::beginPrint(FXPrinter& job){
     outf("%%%%BoundingBox: %d %d %d %d\n",(int)docbb.xmin,(int)docbb.ymin,(int)docbb.xmax,(int)docbb.ymax);
     }
 
-  // Portrait/landscape
-  if(flags&PRINT_LANDSCAPE){
-    outf("%%%%Orientation: Landscape\n");
-    }
-  else{
-    // I am not sure this should be here, so I leave it commented out
-    // (portrait seems to be default anyway)
-    //outf("%%%%Orientation: Portrait\n");
-    }
-
   // Calculate number of pages
   numpages=0;
   if(flags&PRINT_PAGES_ODD){
@@ -461,13 +454,11 @@ FXbool FXDCPrint::beginPage(FXuint page){
   outf("%%%%EndPageSetup\n");
   outf("gsave\n");
 
-/*
   // Maybe in landscape?
   if(flags&PRINT_LANDSCAPE){
     outf("%g %g translate\n",mediawidth,0.0);
     outf("90 rotate\n");
     }
-*/
 
   return TRUE;
   }
@@ -659,6 +650,17 @@ void FXDCPrint::fillRectangles(const FXRectangle* rectangles,FXuint nrectangles)
   }
 
 
+// Fill chord
+void FXDCPrint::fillChord(FXint,FXint,FXint,FXint,FXint,FXint){
+  }
+
+
+// Fill chords
+void FXDCPrint::fillChords(const FXArc*,FXuint){
+  }
+
+
+
 // Fill arc
 void FXDCPrint::fillArc(FXint,FXint,FXint,FXint,FXint,FXint){
   }
@@ -750,7 +752,7 @@ void FXDCPrint::drawImage(const FXImage *img,FXint dx,FXint dy){
   if(opts&IMAGE_OWNED){
     FXint    width  = img->getWidth();
     FXint    height = img->getHeight();
-    FXuchar *buffer = img->getData();
+    FXuchar *buffer = (FXuchar*)img->getData();
 
     outf("/picstr %d string def\n",width*3);
     outf("%d %d translate\n",dx,height-dy);
@@ -761,11 +763,8 @@ void FXDCPrint::drawImage(const FXImage *img,FXint dx,FXint dy){
     outf("false %d\n",3);
     outf("colorimage\n");
 
-    int bpp=3;
-    if(opts&IMAGE_ALPHA) bpp=4;
-
-    int end=width*height*bpp;
-    for(int i=0; i<end ; i+=bpp){
+    int end=width*height;
+    for(int i=0; i<end ; i+=4){
       outhex(buffer[i]);
       outhex(buffer[i+1]);
       outhex(buffer[i+2]);
@@ -776,7 +775,7 @@ void FXDCPrint::drawImage(const FXImage *img,FXint dx,FXint dy){
 
 
 // Draw bitmap
-void FXDCPrint::drawBitmap(const FXBitmap* bitmap,FXint dx,FXint dy){
+void FXDCPrint::drawBitmap(const FXBitmap*,FXint,FXint){
   }
 
 
@@ -933,7 +932,7 @@ void FXDCPrint::clearClipMask(){
 
 
 // Set font to draw text with
-void FXDCPrint::setTextFont(FXFont *fnt){
+void FXDCPrint::setFont(FXFont *fnt){
   font=fnt;
   }
 
@@ -1033,3 +1032,6 @@ FXint FXSetup::DoNTPrint(FXPrinter& printInfo){
   }
 
 */
+
+}
+
