@@ -3,7 +3,7 @@
 *                              D a t a   T a r g e t                            *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDataTarget.cpp,v 1.20 2002/01/18 22:42:59 jeroen Exp $                 *
+* $Id: FXDataTarget.cpp,v 1.28 2003/10/23 04:19:44 fox Exp $                    *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -32,6 +32,7 @@
 #include "FXRectangle.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXWindow.h"
 #include "FXDataTarget.h"
@@ -47,11 +48,15 @@
   - When the sender of onCmdValue does not understand the ID_GETXXXXVALUE message,
     the data target keeps the same value as before.
   - Catch SEL_CHANGED when we have expunged this from FXTextField.
+  - DT_VOID, i.e. unconnected FXDataTarget maybe it should grey out corresponding
+    widgets.
 */
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
 
 // Map
 FXDEFMAP(FXDataTarget) FXDataTargetMap[]={
@@ -74,44 +79,44 @@ long FXDataTarget::onCmdValue(FXObject* sender,FXSelector sel,void*){
   switch(type){
     case DT_CHAR:
       i=*((FXchar*)data);
-      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),(void*)&i);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_GETINTVALUE),(void*)&i);
       *((FXchar*)data)=i;
       break;
     case DT_UCHAR:
       i=*((FXuchar*)data);
-      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),(void*)&i);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_GETINTVALUE),(void*)&i);
       *((FXuchar*)data)=i;
       break;
     case DT_SHORT:
       i=*((FXshort*)data);
-      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),(void*)&i);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_GETINTVALUE),(void*)&i);
       *((FXshort*)data)=i;
       break;
     case DT_USHORT:
       i=*((FXushort*)data);
-      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),(void*)&i);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_GETINTVALUE),(void*)&i);
       *((FXushort*)data)=i;
       break;
     case DT_INT:
-      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),data);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_GETINTVALUE),data);
       break;
     case DT_UINT:
-      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),data);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_GETINTVALUE),data);
       break;
     case DT_FLOAT:
       d=*((FXfloat*)data);
-      sender->handle(this,MKUINT(FXWindow::ID_GETREALVALUE,SEL_COMMAND),(void*)&d);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_GETREALVALUE),(void*)&d);
       *((FXfloat*)data)=(FXfloat)d;
       break;
     case DT_DOUBLE:
-      sender->handle(this,MKUINT(FXWindow::ID_GETREALVALUE,SEL_COMMAND),data);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_GETREALVALUE),data);
       break;
     case DT_STRING:
-      sender->handle(this,MKUINT(FXWindow::ID_GETSTRINGVALUE,SEL_COMMAND),data);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_GETSTRINGVALUE),data);
       break;
     }
   if(target){
-    target->handle(this,MKUINT(message,SELTYPE(sel)),data);
+    target->handle(this,FXSEL(FXSELTYPE(sel),message),data);
     }
   return 1;
   }
@@ -124,35 +129,35 @@ long FXDataTarget::onUpdValue(FXObject* sender,FXSelector,void*){
   switch(type){
     case DT_CHAR:
       i=*((FXchar*)data);
-      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),(void*)&i);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETINTVALUE),(void*)&i);
       break;
     case DT_UCHAR:
       i=*((FXuchar*)data);
-      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),(void*)&i);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETINTVALUE),(void*)&i);
       break;
     case DT_SHORT:
       i=*((FXshort*)data);
-      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),(void*)&i);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETINTVALUE),(void*)&i);
       break;
     case DT_USHORT:
       i=*((FXushort*)data);
-      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),(void*)&i);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETINTVALUE),(void*)&i);
       break;
     case DT_INT:
-      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),data);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETINTVALUE),data);
       break;
     case DT_UINT:
-      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),data);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETINTVALUE),data);
       break;
     case DT_FLOAT:
       d=*((FXfloat*)data);
-      sender->handle(this,MKUINT(FXWindow::ID_SETREALVALUE,SEL_COMMAND),(void*)&d);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETREALVALUE),(void*)&d);
       break;
     case DT_DOUBLE:
-      sender->handle(this,MKUINT(FXWindow::ID_SETREALVALUE,SEL_COMMAND),data);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETREALVALUE),data);
       break;
     case DT_STRING:
-      sender->handle(this,MKUINT(FXWindow::ID_SETSTRINGVALUE,SEL_COMMAND),data);
+      sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_SETSTRINGVALUE),data);
       break;
     }
   return 1;
@@ -161,7 +166,7 @@ long FXDataTarget::onUpdValue(FXObject* sender,FXSelector,void*){
 
 // Value set from message id
 long FXDataTarget::onCmdOption(FXObject*,FXSelector sel,void*){
-  FXint num=((FXint)SELID(sel))-ID_OPTION;
+  FXint num=((FXint)FXSELID(sel))-ID_OPTION;
   switch(type){
     case DT_CHAR:
       *((FXchar*)data)=num;
@@ -189,7 +194,7 @@ long FXDataTarget::onCmdOption(FXObject*,FXSelector sel,void*){
       break;
     }
   if(target){
-    target->handle(this,MKUINT(message,SELTYPE(sel)),data);
+    target->handle(this,FXSEL(FXSELTYPE(sel),message),data);
     }
   return 1;
   }
@@ -197,7 +202,7 @@ long FXDataTarget::onCmdOption(FXObject*,FXSelector sel,void*){
 
 // Check widget whose message id matches
 long FXDataTarget::onUpdOption(FXObject* sender,FXSelector sel,void*){
-  FXint num=((FXint)SELID(sel))-ID_OPTION;
+  FXint num=((FXint)FXSELID(sel))-ID_OPTION;
   FXint i=0;
   switch(type){
     case DT_CHAR:
@@ -226,11 +231,20 @@ long FXDataTarget::onUpdOption(FXObject* sender,FXSelector sel,void*){
       break;
     }
   if(i==num){
-    sender->handle(this,MKUINT(FXWindow::ID_CHECK,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_CHECK),NULL);
     }
   else{
-    sender->handle(this,MKUINT(FXWindow::ID_UNCHECK,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_UNCHECK),NULL);
     }
   return 1;
   }
 
+
+/// Destroy
+FXDataTarget::~FXDataTarget(){
+  target=(FXObject*)-1L;
+  data=(void*)-1L;
+  }
+
+}
+
