@@ -3,7 +3,7 @@
 *                     D i a l o g   B o x    O b j e c t                        *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDialogBox.cpp,v 1.19 2002/01/18 22:42:59 jeroen Exp $                  *
+* $Id: FXDialogBox.cpp,v 1.28 2003/10/23 04:19:44 fox Exp $                     *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -32,8 +32,8 @@
 #include "FXRectangle.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
-#include "FXCursor.h"
 #include "FXDialogBox.h"
 
 /*
@@ -49,14 +49,17 @@
     has the focus will be the default button; default-ness moves between buttons.
 */
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
 
 // Map
 FXDEFMAP(FXDialogBox) FXDialogBoxMap[]={
   FXMAPFUNC(SEL_KEYPRESS,0,FXDialogBox::onKeyPress),
   FXMAPFUNC(SEL_KEYRELEASE,0,FXDialogBox::onKeyRelease),
-  FXMAPFUNC(SEL_CLOSE,0,FXDialogBox::onClose),
+  FXMAPFUNC(SEL_CLOSE,0,FXDialogBox::onCmdCancel),
   FXMAPFUNC(SEL_COMMAND,FXDialogBox::ID_CANCEL,FXDialogBox::onCmdCancel),
   FXMAPFUNC(SEL_COMMAND,FXDialogBox::ID_ACCEPT,FXDialogBox::onCmdAccept),
   };
@@ -78,14 +81,6 @@ FXDialogBox::FXDialogBox(FXWindow* owner,const FXString& name,FXuint opts,FXint
   }
 
 
-// Close window & cancel out of dialog
-long FXDialogBox::onClose(FXObject*,FXSelector,void*){
-  if(target && target->handle(this,MKUINT(message,SEL_CLOSE),NULL)) return 1;
-  handle(this,MKUINT(FXDialogBox::ID_CANCEL,SEL_COMMAND),NULL);
-  return 1;
-  }
-
-
 // Close dialog with an accept
 long FXDialogBox::onCmdAccept(FXObject*,FXSelector,void*){
   getApp()->stopModal(this,TRUE);
@@ -106,7 +101,7 @@ long FXDialogBox::onCmdCancel(FXObject*,FXSelector,void*){
 long FXDialogBox::onKeyPress(FXObject* sender,FXSelector sel,void* ptr){
   if(FXTopWindow::onKeyPress(sender,sel,ptr)) return 1;
   if(((FXEvent*)ptr)->code==KEY_Escape){
-    handle(this,MKUINT(ID_CANCEL,SEL_COMMAND),NULL);
+    handle(this,FXSEL(SEL_COMMAND,ID_CANCEL),NULL);
     return 1;
     }
   return 0;
@@ -130,3 +125,5 @@ FXuint FXDialogBox::execute(FXuint placement){
   show(placement);
   return getApp()->runModalFor(this);
   }
+
+}
