@@ -3,7 +3,7 @@
 *              D i r e c t o r y   S e l e c t i o n   W i d g e t              *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2000,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDirSelector.cpp,v 1.18 2002/02/05 03:59:27 fox Exp $                *
+* $Id: FXDirSelector.cpp,v 1.26 2003/10/23 04:19:44 fox Exp $                   *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -34,6 +34,7 @@
 #include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXFont.h"
 #include "FXGIFIcon.h"
@@ -47,7 +48,7 @@
 #include "FXPopup.h"
 #include "FXTopWindow.h"
 #include "FXDialogBox.h"
-#include "FXScrollbar.h"
+#include "FXScrollBar.h"
 #include "FXTextField.h"
 #include "FXScrollArea.h"
 #include "FXTreeList.h"
@@ -57,7 +58,6 @@
 #include "FXDirList.h"
 #include "FXList.h"
 #include "FXListBox.h"
-#include "FXDriveBox.h"
 #include "FXDirSelector.h"
 
 
@@ -71,13 +71,19 @@
     directories.
 */
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
+
 // Map
 FXDEFMAP(FXDirSelector) FXDirSelectorMap[]={
   FXMAPFUNC(SEL_COMMAND,FXDirSelector::ID_DIRNAME,FXDirSelector::onCmdName),
   FXMAPFUNC(SEL_OPENED,FXDirSelector::ID_DIRLIST,FXDirSelector::onCmdOpened),
-  FXMAPFUNC(SEL_COMMAND,FXDirSelector::ID_DRIVEBOX,FXDirSelector::onCmdDriveChanged),
+  FXMAPFUNC(SEL_COMMAND,FXDirSelector::ID_HOME,FXDirSelector::onCmdHome),
+  FXMAPFUNC(SEL_COMMAND,FXDirSelector::ID_WORK,FXDirSelector::onCmdWork),
+  FXMAPFUNC(SEL_COMMAND,FXDirSelector::ID_DIRECTORY_UP,FXDirSelector::onCmdDirectoryUp),
   };
 
 
@@ -89,23 +95,21 @@ FXIMPLEMENT(FXDirSelector,FXPacker,FXDirSelectorMap,ARRAYNUMBER(FXDirSelectorMap
 FXDirSelector::FXDirSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h):
   FXPacker(p,opts,x,y,w,h){
   FXString currentdirectory=FXFile::getCurrentDirectory();
+  FXAccelTable *table=getShell()->getAccelTable();
   target=tgt;
   message=sel;
+  FXHorizontalFrame *buttons=new FXHorizontalFrame(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH);
+  accept=new FXButton(buttons,"&Accept",NULL,NULL,0,LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0,20,20);
+  cancel=new FXButton(buttons,"&Cancel",NULL,NULL,0,LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0,20,20);
   new FXLabel(this,"&Directory name:",NULL,JUSTIFY_LEFT|LAYOUT_SIDE_TOP|LAYOUT_FILL_X);
-  FXHorizontalFrame *contents=new FXHorizontalFrame(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
-  FXVerticalFrame *leftside=new FXVerticalFrame(contents,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0);
-  dirname=new FXTextField(leftside,25,this,ID_DIRNAME,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK);
-  FXVerticalFrame *buttons=new FXVerticalFrame(contents,PACK_UNIFORM_WIDTH,0,0,0,0, 0,0,0,0);
-  accept=new FXButton(buttons,"&Accept",NULL,NULL,0,FRAME_RAISED|FRAME_THICK,0,0,0,0,20,20);
-  cancel=new FXButton(buttons,"&Cancel",NULL,NULL,0,FRAME_RAISED|FRAME_THICK,0,0,0,0,20,20);
-#ifndef WIN32
-  drivebox=NULL;
-#else
-  drivebox=new FXDriveBox(leftside,5,this,ID_DRIVEBOX,FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_BOTTOM|LAYOUT_BOTTOM|LAYOUT_FILL_X,0,0,0,0);
-  drivebox->setDrive(currentdirectory);
-#endif
-  FXHorizontalFrame *frame=new FXHorizontalFrame(leftside,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0,0,0,0,0);
+  dirname=new FXTextField(this,25,this,ID_DIRNAME,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK);
+  FXHorizontalFrame *frame=new FXHorizontalFrame(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0,0,0,0,0);
   dirbox=new FXDirList(frame,0,this,ID_DIRLIST,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|TREELIST_BROWSESELECT);
+  if(table){
+    table->addAccel(MKUINT(KEY_BackSpace,0),this,FXSEL(SEL_COMMAND,ID_DIRECTORY_UP));
+    table->addAccel(MKUINT(KEY_h,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_HOME));
+    table->addAccel(MKUINT(KEY_w,CONTROLMASK),this,FXSEL(SEL_COMMAND,ID_WORK));
+    }
   dirbox->setDirectory(currentdirectory);
   dirname->setText(currentdirectory);
   dirbox->setFocus();
@@ -114,7 +118,6 @@ FXDirSelector::FXDirSelector(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint
 
 // Set directory
 void FXDirSelector::setDirectory(const FXString& path){
-  if(drivebox) drivebox->setDrive(path);        // FXDriveBox will ignore if the second char is not ':'
   dirname->setText(path);
   dirbox->setDirectory(path);
   }
@@ -140,7 +143,6 @@ FXuint FXDirSelector::getDirBoxStyle() const {
 
 // Typed in new directory name, open path in the tree
 long FXDirSelector::onCmdName(FXObject*,FXSelector,void*){
-  if(drivebox) drivebox->setDrive(dirname->getText());        // May have typed a new drive letter
   dirbox->setDirectory(dirname->getText());
   return 1;
   }
@@ -154,16 +156,23 @@ long FXDirSelector::onCmdOpened(FXObject*,FXSelector,void* ptr){
   }
 
 
-// Changed the drive, making it the current one
-long FXDirSelector::onCmdDriveChanged(FXObject*,FXSelector,void* ptr){
-  if(drivebox){
-    FXString path=FXString((char*)ptr);
-#ifdef WIN32
-    path.append(PATHSEPSTRING);        // Add the backslash to out drive letter
-#endif
-    dirname->setText(path);
-    dirbox->setDirectory(path);
-    }
+// Back to home directory
+long FXDirSelector::onCmdHome(FXObject*,FXSelector,void*){
+  setDirectory(FXFile::getHomeDirectory());
+  return 1;
+  }
+
+
+// Back to current working directory
+long FXDirSelector::onCmdWork(FXObject*,FXSelector,void*){
+  setDirectory(FXFile::getCurrentDirectory());
+  return 1;
+  }
+
+
+// User clicked up directory button
+long FXDirSelector::onCmdDirectoryUp(FXObject*,FXSelector,void*){
+  setDirectory(FXFile::upLevel(getDirectory()));
   return 1;
   }
 
@@ -172,7 +181,6 @@ long FXDirSelector::onCmdDriveChanged(FXObject*,FXSelector,void* ptr){
 void FXDirSelector::save(FXStream& store) const {
   FXPacker::save(store);
   store << dirbox;
-  store << drivebox;
   store << dirname;
   store << accept;
   store << cancel;
@@ -183,7 +191,6 @@ void FXDirSelector::save(FXStream& store) const {
 void FXDirSelector::load(FXStream& store){
   FXPacker::load(store);
   store >> dirbox;
-  store >> drivebox;
   store >> dirname;
   store >> accept;
   store >> cancel;
@@ -192,9 +199,10 @@ void FXDirSelector::load(FXStream& store){
 
 // Clean up
 FXDirSelector::~FXDirSelector(){
-  dirbox=(FXDirList*)-1;
-  drivebox=(FXDriveBox*)-1;
-  dirname=(FXTextField*)-1;
-  accept=(FXButton*)-1;
-  cancel=(FXButton*)-1;
+  dirbox=(FXDirList*)-1L;
+  dirname=(FXTextField*)-1L;
+  accept=(FXButton*)-1L;
+  cancel=(FXButton*)-1L;
   }
+
+}
