@@ -3,7 +3,7 @@
 *                          E x c e p t i o n  T y p e s                         *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2000,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,12 +19,15 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXException.cpp,v 1.6 2002/01/18 22:42:59 jeroen Exp $                   *
+* $Id: FXException.cpp,v 1.9 2003/04/05 14:22:36 fox Exp $                      *
 ********************************************************************************/
 #include "fxver.h"
 #include "fxdefs.h"
 #include "FXException.h"
 
+using namespace FX;
+
+namespace FX {
 
 // Generic unspecified exception
 const FXchar FXException::exceptionName[]="unknown";
@@ -48,3 +51,5 @@ const FXchar FXResourceException::exceptionName[]="resource exhausted";
 
 // Ran out of memory or such
 const FXchar FXMemoryException::exceptionName[]="out of memory";
+
+}
