@@ -3,7 +3,7 @@
 *                    F i l e   S e l e c t i o n   D i a l o g                  *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXFileDialog.cpp,v 1.23.4.3 2003/03/27 20:34:43 fox Exp $                 *
+* $Id: FXFileDialog.cpp,v 1.34 2003/10/23 04:19:44 fox Exp $                    *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -33,6 +33,7 @@
 #include "FXFile.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXDrawable.h"
@@ -52,19 +53,34 @@
 
 
 /*
-  To do:
+  Notes:
+  - Wraps the FXFileSelector file selection mega widget.
 */
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 // Object implementation
 FXIMPLEMENT(FXFileDialog,FXDialogBox,NULL,0)
 
 
-// File Open Dialog
+// Construct file fialog box
 FXFileDialog::FXFileDialog(FXWindow* owner,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h):
-  FXDialogBox(owner,name,opts|DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,x,y,w,h,0,0,0,0,4,4){
+  FXDialogBox(owner,name,opts|DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_CLOSE,x,y,w,h,0,0,0,0,4,4){
+  filebox=new FXFileSelector(this,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);
+  filebox->acceptButton()->setTarget(this);
+  filebox->acceptButton()->setSelector(FXDialogBox::ID_ACCEPT);
+  filebox->cancelButton()->setTarget(this);
+  filebox->cancelButton()->setSelector(FXDialogBox::ID_CANCEL);
+  }
+
+
+// Construct free-floating file dialog box
+FXFileDialog::FXFileDialog(FXApp* a,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h):
+  FXDialogBox(a,name,opts|DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_CLOSE,x,y,w,h,0,0,0,0,4,4){
   filebox=new FXFileSelector(this,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);
   filebox->acceptButton()->setTarget(this);
   filebox->acceptButton()->setSelector(FXDialogBox::ID_ACCEPT);
@@ -148,12 +164,6 @@ void FXFileDialog::setPatternText(FXint patno,const FXString& text){
   }
 
 
-// Set list of patterns (DEPRECATED)
-void FXFileDialog::setPatternList(const FXchar **ptrns){
-  filebox->setPatternList(ptrns);
-  }
-
-
 // Change space for item
 void FXFileDialog::setItemSpace(FXint s){
   filebox->setItemSpace(s);
@@ -231,7 +241,7 @@ void FXFileDialog::load(FXStream& store){
 
 // Cleanup
 FXFileDialog::~FXFileDialog(){
-  filebox=(FXFileSelector*)-1;
+  filebox=(FXFileSelector*)-1L;
   }
 
 
@@ -292,3 +302,5 @@ FXString FXFileDialog::getOpenDirectory(FXWindow* owner,const FXString& caption,
   return FXString::null;
   }
 
+}
+
