@@ -3,7 +3,7 @@
 *                               F o n t   O b j e c t                           *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXFont.cpp,v 1.52 2002/02/12 00:58:29 fox Exp $                          *
+* $Id: FXFont.cpp,v 1.65 2003/10/27 23:59:21 fox Exp $                          *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -32,6 +32,7 @@
 #include "FXRectangle.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXFont.h"
@@ -186,12 +187,16 @@
     o If no exact match, leave as number, e.g. weight=550.
     o leave hints as number.
     o need only specify family, or more if you want.
+
+  - Does it make sense to match font if no family name known?
 */
 
 
 // X11
 #ifndef WIN32
 
+// Maximum XLFD name length
+#define MAX_XLFD          400
 
 // XLFD Fields
 #define XLFD_FOUNDRY      0
@@ -246,9 +251,18 @@
 
 #endif
 
+using namespace FX;
+
+
+namespace FX {
+
 
 /*******************************************************************************/
 
+// Helper functions X11
+
+#ifndef WIN32
+
 
 // Convert text to font weight
 static FXuint weightfromtext(const FXchar* text){
@@ -352,13 +366,6 @@ static FXbool ismultibyte(const FXchar* text){
   }
 
 
-/*******************************************************************************/
-
-// Helper functions X11
-
-#ifndef WIN32
-
-
 // Get list of font names matching pattern
 static char **listfontnames(Display *dpy,const char* pattern,int& numfnames){
   int maxfnames=1024;
@@ -402,10 +409,10 @@ static int parsefontname(char *fields[],char* fname){
 
 // Try find matching font
 char* FXFont::findmatch(char *fontname,const char* family) const {
-  FXchar pattern[300],fname[300],*field[14],**fnames;
+  FXchar pattern[MAX_XLFD],fname[MAX_XLFD],*field[14],**fnames;
   FXint bestf,bestdweight,bestdsize,bestvalue,bestscalable,bestxres,bestyres;
   FXint screenres,xres,yres;
-  FXint dweight,w,scalable,polymorphic,dsize;
+  FXint dweight,w,scalable,dsize;
   FXint numfnames,f,value;
   FXuint sw,sz,sl,pitch,en;
 
@@ -422,8 +429,7 @@ char* FXFont::findmatch(char *fontname,const char* family) const {
   bestscalable=0;
   bestxres=75;
   bestyres=75;
-  fname[299]=0;
-
+  fname[MAX_XLFD-1]=0;
 
   // Perhaps override screen resolution via registry
   screenres=getApp()->reg().readUnsignedEntry("SETTINGS","screenres",100);
@@ -436,13 +442,12 @@ char* FXFont::findmatch(char *fontname,const char* family) const {
 
   // Loop over all fonts to find the best match
   for(f=0; f<numfnames; f++){
-    strncpy(fname,fnames[f],299);
+    strncpy(fname,fnames[f],MAX_XLFD-1);
     if(parsefontname(field,fname)){
 
       // This font's match value
       value=0;
       scalable=0;
-      polymorphic=0;
       dsize=1000000;
       dweight=1000;
 
@@ -495,7 +500,6 @@ char* FXFont::findmatch(char *fontname,const char* family) const {
       // Polymorphic
       if(EQUAL1(field[XLFD_WEIGHT],'0') || EQUAL1(field[XLFD_SETWIDTH],'0') || EQUAL1(field[XLFD_SLANT],'0') || EQUAL1(field[XLFD_ADDSTYLE],'0')){
         value+=POLY_FACTOR;
-        polymorphic=1;
         }
       else{
         if(!(hints&FONTHINT_POLYMORPHIC)) value+=POLY_FACTOR;
@@ -570,7 +574,7 @@ char* FXFont::findmatch(char *fontname,const char* family) const {
           }
         }
 
-      FXTRACE((160,"%4d: match=%-3x dw=%-3d ds=%3d sc=%d py=%d xres=%-3d yres=%-3d xlfd=%s\n",f,value,dweight,dsize,scalable,polymorphic,xres,yres,fnames[f]));
+      FXTRACE((160,"%4d: match=%-3x dw=%-3d ds=%3d sc=%d xres=%-3d yres=%-3d xlfd=%s\n",f,value,dweight,dsize,scalable,xres,yres,fnames[f]));
 
       // How close is the match?
       if((value>bestvalue) || ((value==bestvalue) && (dsize<bestdsize)) || ((value==bestvalue) && (dsize==bestdsize) && (dweight<bestdweight))){
@@ -586,10 +590,10 @@ char* FXFont::findmatch(char *fontname,const char* family) const {
     }
   if(0<=bestf){
     if(!bestscalable){
-      strncpy(fontname,fnames[bestf],299);
+      strncpy(fontname,fnames[bestf],MAX_XLFD-1);
       }
     else{
-      strncpy(fname,fnames[bestf],299);
+      strncpy(fname,fnames[bestf],MAX_XLFD-1);
       parsefontname(field,fname);
 
       // Build XLFD, correcting for possible difference between font resolution and screen resolution
@@ -599,7 +603,7 @@ char* FXFont::findmatch(char *fontname,const char* family) const {
       //sprintf(fontname,"-%s-%s-%s-%s-%s-%s-*-[0 64 ~64 0]-%d-%d-%s-*-%s-%s",field[XLFD_FOUNDRY],field[XLFD_FAMILY],field[XLFD_WEIGHT],field[XLFD_SLANT],field[XLFD_SETWIDTH],field[XLFD_ADDSTYLE],screenxres,screenyres,field[XLFD_SPACING],field[XLFD_REGISTRY],field[XLFD_ENCODING]);
       }
     FXTRACE((150,"Best Font:\n"));
-    FXTRACE((150,"%4d: match=%3x dw=%-3d ds=%-3d sc=%d py=%d xres=%-3d yres=%-3d xlfd=%s\n",bestf,bestvalue,bestdweight,bestdsize,bestscalable,polymorphic,bestxres,bestyres,fontname));
+    FXTRACE((150,"%4d: match=%3x dw=%-3d ds=%-3d sc=%d xres=%-3d yres=%-3d xlfd=%s\n",bestf,bestvalue,bestdweight,bestdsize,bestscalable,bestxres,bestyres,fontname));
     XFreeFontNames(fnames);
     return fontname;
     }
@@ -759,19 +763,23 @@ static BYTE FXFontEncoding2CharSet(FXuint encoding){
 // Character set encoding
 static FXuint CharSet2FXFontEncoding(BYTE lfCharSet){
   switch(lfCharSet){
+    case ANSI_CHARSET: return FONTENCODING_USASCII;
+    case ARABIC_CHARSET: return FONTENCODING_ARABIC;
+    case BALTIC_CHARSET: return FONTENCODING_BALTIC;
+    case CHINESEBIG5_CHARSET: return FONTENCODING_DEFAULT;
     case DEFAULT_CHARSET: return FONTENCODING_DEFAULT;
+    case EASTEUROPE_CHARSET: return FONTENCODING_EASTEUROPE;
+    case GB2312_CHARSET: return FONTENCODING_DEFAULT;
+    case GREEK_CHARSET: return FONTENCODING_GREEK;
+    case HANGUL_CHARSET: return FONTENCODING_DEFAULT;
+    case HEBREW_CHARSET: return FONTENCODING_HEBREW;
+    case MAC_CHARSET: return FONTENCODING_DEFAULT;
     case OEM_CHARSET: return FONTENCODING_DEFAULT;
     case SYMBOL_CHARSET: return FONTENCODING_DEFAULT;
-    case MAC_CHARSET: return FONTENCODING_DEFAULT;
-    case ANSI_CHARSET: return FONTENCODING_USASCII;
-    case TURKISH_CHARSET: return FONTENCODING_TURKISH;
-    case BALTIC_CHARSET: return FONTENCODING_BALTIC;
     case RUSSIAN_CHARSET: return FONTENCODING_CYRILLIC;
-    case ARABIC_CHARSET: return FONTENCODING_ARABIC;
-    case GREEK_CHARSET: return FONTENCODING_GREEK;
-    case HEBREW_CHARSET: return FONTENCODING_HEBREW;
+    case SHIFTJIS_CHARSET: return FONTENCODING_DEFAULT;
     case THAI_CHARSET: return FONTENCODING_THAI;
-    case EASTEUROPE_CHARSET: return FONTENCODING_EASTEUROPE;
+    case TURKISH_CHARSET: return FONTENCODING_TURKISH;
     }
   return FONTENCODING_DEFAULT;
   }
@@ -789,16 +797,24 @@ struct FXFontStore {
 
 // Callback function for EnumFontFamiliesEx()
 static int CALLBACK EnumFontFamExProc(const LOGFONT *lf,const TEXTMETRIC *lptm,DWORD FontType,LPARAM lParam){
-  FXASSERT(lf!=0);
-  FXASSERT(lptm!=0);
-  FXFontStore *pFontStore=(FXFontStore*)lParam;
-  FXASSERT(pFontStore!=0);
+  register FXFontStore *pFontStore=(FXFontStore*)lParam;
+  FXASSERT(lf);
+  FXASSERT(lptm);
+  FXASSERT(pFontStore);
 
   // Get pitch
   FXuint flags=FONTPITCH_DEFAULT;
   if(lf->lfPitchAndFamily&FIXED_PITCH) flags|=FONTPITCH_FIXED;
   if(lf->lfPitchAndFamily&VARIABLE_PITCH) flags|=FONTPITCH_VARIABLE;
 
+  // Get hints
+  if(lf->lfPitchAndFamily&FF_DONTCARE) flags|=FONTHINT_DONTCARE;
+  if(lf->lfPitchAndFamily&FF_MODERN) flags|=FONTHINT_MODERN;
+  if(lf->lfPitchAndFamily&FF_ROMAN) flags|=FONTHINT_ROMAN;
+  if(lf->lfPitchAndFamily&FF_SCRIPT) flags|=FONTHINT_SCRIPT;
+  if(lf->lfPitchAndFamily&FF_DECORATIVE) flags|=FONTHINT_DECORATIVE;
+  if(lf->lfPitchAndFamily&FF_SWISS) flags|=FONTHINT_SWISS;
+
   // Skip if no match
   FXuint h=pFontStore->desc.flags;
   if((h&FONTPITCH_FIXED) && !(flags&FONTPITCH_FIXED)) return 1;
@@ -806,10 +822,11 @@ static int CALLBACK EnumFontFamExProc(const LOGFONT *lf,const TEXTMETRIC *lptm,D
 
   // Get weight (also guess from the name)
   FXuint weight=lf->lfWeight;
-  if(strstr(lf->lfFaceName," Bold")!=NULL) weight=FONTWEIGHT_BOLD;
-  if(strstr(lf->lfFaceName," Demi")!=NULL) weight=FONTWEIGHT_DEMIBOLD;
-  if(strstr(lf->lfFaceName," Light")!=NULL) weight=FONTWEIGHT_LIGHT;
-  if(strstr(lf->lfFaceName," Medium")!=NULL) weight=FONTWEIGHT_MEDIUM;
+  if(strstr(lf->lfFaceName,"Bold")!=NULL) weight=FONTWEIGHT_BOLD;
+  if(strstr(lf->lfFaceName,"Black")!=NULL) weight=FONTWEIGHT_BLACK;
+  if(strstr(lf->lfFaceName,"Demi")!=NULL) weight=FONTWEIGHT_DEMIBOLD;
+  if(strstr(lf->lfFaceName,"Light")!=NULL) weight=FONTWEIGHT_LIGHT;
+  if(strstr(lf->lfFaceName,"Medium")!=NULL) weight=FONTWEIGHT_MEDIUM;
 
   // Skip if weight doesn't match
   FXuint wt=pFontStore->desc.weight;
@@ -818,8 +835,8 @@ static int CALLBACK EnumFontFamExProc(const LOGFONT *lf,const TEXTMETRIC *lptm,D
   // Get slant
   FXuint slant=FONTSLANT_REGULAR;
   if(lf->lfItalic==TRUE) slant=FONTSLANT_ITALIC;
-  if(strstr(lf->lfFaceName," Italic")!=NULL) slant=FONTSLANT_ITALIC;
-  if(strstr(lf->lfFaceName," Roman")!=NULL) slant=FONTSLANT_REGULAR;
+  if(strstr(lf->lfFaceName,"Italic")!=NULL) slant=FONTSLANT_ITALIC;
+  if(strstr(lf->lfFaceName,"Roman")!=NULL) slant=FONTSLANT_REGULAR;
 
   // Skip if no match
   FXuint sl=pFontStore->desc.slant;
@@ -827,8 +844,9 @@ static int CALLBACK EnumFontFamExProc(const LOGFONT *lf,const TEXTMETRIC *lptm,D
 
   // Get set width (also guess from the name)
   FXuint setwidth=FONTSETWIDTH_DONTCARE;
-  if(strstr(lf->lfFaceName," Cond")!=NULL) setwidth=FONTSETWIDTH_CONDENSED;
-  if(strstr(lf->lfFaceName," Ext Cond")!=NULL) setwidth=FONTSETWIDTH_EXTRACONDENSED;
+  if(strstr(lf->lfFaceName,"Cond")!=NULL) setwidth=FONTSETWIDTH_CONDENSED;
+  if(strstr(lf->lfFaceName,"Narrow")!=NULL) setwidth=FONTSETWIDTH_NARROW;
+  if(strstr(lf->lfFaceName,"Ext Cond")!=NULL) setwidth=FONTSETWIDTH_EXTRACONDENSED;
 
   // Skip if no match
   FXuint sw=pFontStore->desc.setwidth;
@@ -869,7 +887,12 @@ static int CALLBACK EnumFontFamExProc(const LOGFONT *lf,const TEXTMETRIC *lptm,D
   FXuint numfonts=pFontStore->numfonts;
 
   strncpy(fonts[numfonts].face,lf->lfFaceName,sizeof(fonts[0].face));
-  fonts[numfonts].size=fxheight_to_pointsize(pFontStore->hdc,lf->lfHeight); /// This may be incorrect
+  if(lf->lfHeight<0){
+    fonts[numfonts].size=-MulDiv(lf->lfHeight,720,GetDeviceCaps(pFontStore->hdc,LOGPIXELSY));
+    }
+  else{
+    fonts[numfonts].size=MulDiv(lf->lfHeight,720,GetDeviceCaps(pFontStore->hdc,LOGPIXELSY));
+    }
   fonts[numfonts].weight=weight;
   fonts[numfonts].slant=slant;
   fonts[numfonts].encoding=encoding;
@@ -896,7 +919,7 @@ FXIMPLEMENT(FXFont,FXId,NULL,0)
 
 // Deserialization
 FXFont::FXFont(){
-  font=(void*)-1;
+  font=(void*)-1L;
 #ifdef WIN32
   dc=NULL;
 #endif
@@ -984,7 +1007,7 @@ void FXFont::setFontDesc(const FXFontDesc& fontdesc){
 // Create font
 void FXFont::create(){
   if(!xid){
-    if(getApp()->initialized){
+    if(getApp()->isInitialized()){
       FXTRACE((100,"%s::create %p\n",getClassName(),this));
 #ifndef WIN32
 
@@ -1005,7 +1028,7 @@ void FXFont::create(){
         FXTRACE((150,"%s::create: face: %s size: %d weight: %d slant: %d encoding: %d hints: %04x\n",getClassName(),name.text()?name.text():"none",size,weight,slant,encoding,hints));
 
         // Try load best font
-        char fontname[400];
+        char fontname[MAX_XLFD];
         font=XLoadQueryFont(DISPLAY(getApp()),findbestfont(fontname));
         }
 
@@ -1057,7 +1080,6 @@ void FXFont::create(){
 
       // Now fill in the fields
       LOGFONT lf;
-      //lf.lfHeight=fxpointsize_to_height(dc,size);
       lf.lfHeight = -MulDiv(size, GetDeviceCaps((HDC)dc, LOGPIXELSY), 720);  // This looks correct!!
       lf.lfWidth=0;
       lf.lfEscapement=0;
@@ -1148,7 +1170,7 @@ void FXFont::detach(){
 // Destroy font
 void FXFont::destroy(){
   if(xid){
-    if(getApp()->initialized){
+    if(getApp()->isInitialized()){
       FXTRACE((100,"%s::destroy %p\n",getClassName(),this));
 #ifndef WIN32
 
@@ -1339,6 +1361,12 @@ FXint FXFont::getTextWidth(const FXchar *text,FXuint n) const {
   }
 
 
+// Text width
+FXint FXFont::getTextWidth(const FXString& text) const {
+  return getTextWidth(text.text(),text.length());
+  }
+
+
 // Text height
 FXint FXFont::getTextHeight(const FXchar *text,FXuint n) const {
   if(!text && n){ fxerror("%s::getTextHeight: NULL string argument\n",getClassName()); }
@@ -1358,6 +1386,12 @@ FXint FXFont::getTextHeight(const FXchar *text,FXuint n) const {
   }
 
 
+// Text height
+FXint FXFont::getTextHeight(const FXString& text) const {
+  return getTextHeight(text.text(),text.length());
+  }
+
+
 /*******************************************************************************/
 
 
@@ -1369,8 +1403,8 @@ static int comparefont(const void * a,const void *b){
 #endif
   register FXFontDesc *fa=(FXFontDesc*)a;
   register FXFontDesc *fb=(FXFontDesc*)b;
-  register int cmp;
-  return (cmp=strcmp(fa->face,fb->face)) ? cmp : (fa->weight!=fb->weight) ? fa->weight-fb->weight : (fa->slant!=fb->slant) ? fa->slant-fb->slant : fa->size-fb->size;
+  register FXint cmp=strcmp(fa->face,fb->face);
+  return cmp ? cmp : (fa->weight!=fb->weight) ? fa->weight-fb->weight : (fa->slant!=fb->slant) ? fa->slant-fb->slant : fa->size-fb->size;
   }
 
 
@@ -1378,10 +1412,9 @@ static int comparefont(const void * a,const void *b){
 
 // List all fonts matching hints
 FXbool FXFont::listFonts(FXFontDesc*& fonts,FXuint& numfonts,const FXString& face,FXuint wt,FXuint sl,FXuint sw,FXuint en,FXuint h){
+  FXchar pattern[MAX_XLFD],fname[MAX_XLFD],*field[14],**fnames;
   FXuint size,weight,slant,encoding,setwidth,flags;
   FXint screenres,xres,yres;
-  FXchar pattern[300],fname[300],*field[14];
-  FXchar **fnames;
   const FXchar *scal;
   const FXchar *facename;
   FXint numfnames,f,j,addit;
@@ -1389,6 +1422,9 @@ FXbool FXFont::listFonts(FXFontDesc*& fonts,FXuint& numfonts,const FXString& fac
   fonts=NULL;
   numfonts=0;
 
+  pattern[MAX_XLFD-1]='\0';
+  fname[MAX_XLFD-1]='\0';
+
   // Gotta have display open!
   if(!FXApp::instance()){ fxerror("FXFont::listFonts: no application object.\n"); }
   if(!DISPLAY(FXApp::instance())){ fxerror("FXFont::listFonts: trying to list fonts before opening display.\n"); }
@@ -1408,7 +1444,7 @@ FXbool FXFont::listFonts(FXFontDesc*& fonts,FXuint& numfonts,const FXString& fac
   if(h&FONTHINT_X11){
     facename="*";
     if(!face.empty()) facename=face.text();
-    strcpy(pattern,facename);
+    strncpy(pattern,facename,MAX_XLFD-1);
     }
 
   // Match XLFD fonts; try to limit the number by using
@@ -1431,7 +1467,7 @@ FXbool FXFont::listFonts(FXFontDesc*& fonts,FXuint& numfonts,const FXString& fac
 
   // Add all matching fonts to the list
   for(f=0; f<numfnames; f++){
-    strncpy(fname,fnames[f],299);
+    strncpy(fname,fnames[f],MAX_XLFD-1);
 
     // XLFD font name; parse out unique face names
     if(parsefontname(field,fname)){
@@ -1595,10 +1631,21 @@ FXbool FXFont::listFonts(FXFontDesc*& fonts,FXuint& numfonts,const FXString& fac
   // EnumFontFamiliesEx() only examines the lfCharSet, lfFaceName and
   // lpPitchAndFamily fields of this struct.
   LOGFONT lf;
+  lf.lfHeight=0;
+  lf.lfWidth=0;
+  lf.lfEscapement=0;
+  lf.lfOrientation=0;
+  lf.lfWeight=0;
+  lf.lfItalic=0;
+  lf.lfUnderline=0;
+  lf.lfStrikeOut=0;
   lf.lfCharSet=FXFontEncoding2CharSet(en);
+  lf.lfOutPrecision=0;
+  lf.lfClipPrecision=0;
+  lf.lfQuality=0;
+  lf.lfPitchAndFamily=0;                          // Should be MONO_FONT for Hebrew and Arabic?
   FXASSERT(face.length()<LF_FACESIZE);
   strncpy(lf.lfFaceName,face.text(),LF_FACESIZE);
-  lf.lfPitchAndFamily=0;                          // Should be MONO_FONT for Hebrew and Arabic?
 
   // Start enumerating!
   EnumFontFamiliesEx(hdc,&lf,EnumFontFamExProc,(LPARAM)&fontStore,0);
@@ -1745,13 +1792,13 @@ void FXFont::load(FXStream& store){
 //
 // // Parse font description from a string
 // FXbool FXFont::parseFontDesc(FXFontDesc& fontdesc,const FXString& string){
-//   strncpy(fontdesc.face,string.extract(0,',').text(),sizeof(fontdesc.face));
-//   fontdesc.size=(FXint)(10.0*FXDoubleVal(string.extract(1,',')));
-//   fontdesc.weight=100*findintable(weighttable,string.extract(2,','),ARRAYNUMBER(weighttable));
-//   fontdesc.slant=findintable(slanttable,string.extract(3,','),ARRAYNUMBER(slanttable));
-//   fontdesc.setwidth=10*findintable(setwidthtable,string.extract(4,','),ARRAYNUMBER(setwidthtable));
-//   fontdesc.encoding=findintable(encodingtable,string.extract(5,','),ARRAYNUMBER(encodingtable));
-//   fontdesc.flags=FXUIntVal(string.extract(6,','));
+//   strncpy(fontdesc.face,string.section(',',0).text(),sizeof(fontdesc.face));
+//   fontdesc.size=(FXint)(10.0*FXDoubleVal(string.section(',',1)));
+//   fontdesc.weight=100*findintable(weighttable,string.section(',',2),ARRAYNUMBER(weighttable));
+//   fontdesc.slant=findintable(slanttable,string.section(',',3),ARRAYNUMBER(slanttable));
+//   fontdesc.setwidth=10*findintable(setwidthtable,string.section(',',4),ARRAYNUMBER(setwidthtable));
+//   fontdesc.encoding=findintable(encodingtable,string.section(',',5),ARRAYNUMBER(encodingtable));
+//   fontdesc.flags=FXUIntVal(string.section(',',6));
 //   return TRUE;
 //   }
 //
@@ -1809,3 +1856,4 @@ FXFont::~FXFont(){
   destroy();
   }
 
+}
