@@ -3,7 +3,7 @@
 *                        F o n t   S e l e c t i o n   B o x                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1999,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXFontSelector.cpp,v 1.28.4.1 2003/06/20 19:02:07 fox Exp $               *
+* $Id: FXFontSelector.cpp,v 1.37 2003/10/23 04:19:44 fox Exp $                  *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -32,6 +32,7 @@
 #include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXFont.h"
@@ -53,7 +54,7 @@
 #include "FXMatrix.h"
 #include "FXCanvas.h"
 #include "FXShell.h"
-#include "FXScrollbar.h"
+#include "FXScrollBar.h"
 #include "FXScrollArea.h"
 #include "FXScrollWindow.h"
 #include "FXList.h"
@@ -65,8 +66,11 @@
   Notes:
 */
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
 
 // Map
 FXDEFMAP(FXFontSelector) FXFontSelectorMap[]={
@@ -674,23 +678,24 @@ void FXFontSelector::load(FXStream& store){
 // Cleanup
 FXFontSelector::~FXFontSelector(){
   delete previewfont;
-  family=(FXTextField*)-1;
-  familylist=(FXList*)-1;
-  weight=(FXTextField*)-1;
-  weightlist=(FXList*)-1;
-  style=(FXTextField*)-1;
-  stylelist=(FXList*)-1;
-  size=(FXTextField*)-1;
-  sizelist=(FXList*)-1;
-  charset=(FXComboBox*)-1;
-  setwidth=(FXComboBox*)-1;
-  pitch=(FXComboBox*)-1;
-  scalable=(FXCheckButton*)-1;
-  allfonts=(FXCheckButton*)-1;
-  preview=(FXLabel*)-1;
-  previewfont=(FXFont*)-1;
-  accept=(FXButton*)-1;
-  cancel=(FXButton*)-1;
+  family=(FXTextField*)-1L;
+  familylist=(FXList*)-1L;
+  weight=(FXTextField*)-1L;
+  weightlist=(FXList*)-1L;
+  style=(FXTextField*)-1L;
+  stylelist=(FXList*)-1L;
+  size=(FXTextField*)-1L;
+  sizelist=(FXList*)-1L;
+  charset=(FXComboBox*)-1L;
+  setwidth=(FXComboBox*)-1L;
+  pitch=(FXComboBox*)-1L;
+  scalable=(FXCheckButton*)-1L;
+  allfonts=(FXCheckButton*)-1L;
+  preview=(FXLabel*)-1L;
+  previewfont=(FXFont*)-1L;
+  accept=(FXButton*)-1L;
+  cancel=(FXButton*)-1L;
   }
 
+}
 
