@@ -3,7 +3,7 @@
 *                        G I F   C u r s o r   O b j e c t                      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2002 by Daniel Gehriger.   All Rights Reserved.            *
+* Copyright (C) 2000,2003 by Daniel Gehriger.   All Rights Reserved.            *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,18 +19,20 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGIFCursor.cpp,v 1.10 2002/01/18 22:43:00 jeroen Exp $                   *
+* $Id: FXGIFCursor.cpp,v 1.21 2003/10/23 04:19:44 fox Exp $                     *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
+#include "FXMemoryStream.h"
 #include "FXString.h"
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXGIFCursor.h"
 
@@ -50,48 +52,51 @@
   - Maybe want to have some API to turn FXBitmap into FXCursor.
 */
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
+
 // Object implementation
 FXIMPLEMENT(FXGIFCursor,FXCursor,NULL,0)
 
 
-/*-----------------------------------------------------------------*\
- *
- * Constructor
- *
-\*-----------------------------------------------------------------*/
-FXGIFCursor::FXGIFCursor(FXApp* a,const void *pix,FXint hx,FXint hy):FXCursor(a,NULL,NULL,0,0,hx,hy){
+// Constructor
+FXGIFCursor::FXGIFCursor(FXApp* a,const void *pix,FXint hx,FXint hy):FXCursor(a,NULL,NULL,0,0,0,0){
   if(pix){
-    FXuchar* pbData;
-    FXColor  clrPixel,clrTransp;
-    FXint    h,w,nWidth,nHeight,nWidthBytes,nOffset;
+    FXColor *data;
+    FXColor  color;
+    FXint    h,w,nWidth,nHeight,nWidthBytes;
     FXMemoryStream ms;
-    ms.open((FXuchar*)pix,FXStreamLoad);
-    if(fxloadGIF(ms,pbData,clrTransp,nWidth,nHeight)){
+    ms.open(FXStreamLoad,(FXuchar*)pix);
+    if(fxloadGIF(ms,data,nWidth,nHeight)){
       if(nWidth>32 || nHeight>32){ fxerror("%s::create: cursor exceeds maximum size of 32x32 pixels\n",getClassName()); }
       width=nWidth;
       height=nHeight;
-      nWidthBytes=(nWidth+7)/8;
+      hotx=FXCLAMP(0,hx,width-1);
+      hoty=FXCLAMP(0,hy,height-1);
+      nWidthBytes=(nWidth+7)>>3;
       FXCALLOC(&source,FXuchar,nWidthBytes*nHeight);
       FXCALLOC(&mask,FXuchar,nWidthBytes*nHeight);
       owned=TRUE;
-      for(h=0; h<nHeight; ++h){
-        for(w=0; w<nWidth; ++w){
-          nOffset=3*(w+nWidth*h);
-          clrPixel=FXRGB(pbData[nOffset],pbData[nOffset+1],pbData[nOffset+2]);
-          if(clrPixel==FXRGB(0,0,0)){
-            source[(w+nWidth*h)/8]|=1<<(w%8);
+      for(h=0; h<nHeight; h++){
+        for(w=0; w<nWidth; w++){
+          color=data[w+nWidth*h];
+          if(color==FXRGB(0,0,0)){
+            source[(w+nWidth*h)>>3]|=1<<(w&7);
             }
-          if(clrPixel!=clrTransp){
-            mask[(w+nWidth*h)/8]|=1<<(w%8);
+          if(FXALPHAVAL(color)){            // FIXME now have real alpha
+            mask[(w+nWidth*h)>>3]|=1<<(w&7);
             }
           }
         }
-      FXFREE(&pbData);    // Free original pixels
+      FXFREE(&data);    // Free original pixels
       }
     ms.close();
     }
   }
 
+}
+
 
