@@ -3,7 +3,7 @@
 *                            G I F   I m a g e   O b j e c t                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,18 +19,20 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGIFImage.cpp,v 1.13 2002/01/18 22:43:00 jeroen Exp $                   *
+* $Id: FXGIFImage.cpp,v 1.24 2003/10/23 04:19:44 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
+#include "FXMemoryStream.h"
 #include "FXString.h"
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXGIFImage.h"
 
@@ -41,21 +43,24 @@
   - Only free image if owned!
 */
 
+using namespace FX;
+
 
 /*******************************************************************************/
 
+namespace FX {
+
 // Object implementation
 FXIMPLEMENT(FXGIFImage,FXImage,NULL,0)
 
 
 // Initialize
 FXGIFImage::FXGIFImage(FXApp* a,const void *pix,FXuint opts,FXint w,FXint h):
-  FXImage(a,NULL,opts&~IMAGE_ALPHA,w,h){
+  FXImage(a,NULL,opts,w,h){
   if(pix){
     FXMemoryStream ms;
-    FXColor clearcolor;
-    ms.open((FXuchar*)pix,FXStreamLoad);
-    fxloadGIF(ms,data,clearcolor,width,height);
+    ms.open(FXStreamLoad,(FXuchar*)pix);
+    fxloadGIF(ms,data,width,height);
     options|=IMAGE_OWNED;
     ms.close();
     }
@@ -63,23 +68,23 @@ FXGIFImage::FXGIFImage(FXApp* a,const void *pix,FXuint opts,FXint w,FXint h):
 
 
 // Save object to stream
-void FXGIFImage::savePixels(FXStream& store) const {
-  FXColor clearcolor=FXRGB(192,192,192);
-  FXASSERT(!(options&IMAGE_ALPHA));
-  fxsaveGIF(store,data,clearcolor,width,height);
+FXbool FXGIFImage::savePixels(FXStream& store) const {
+  if(!fxsaveGIF(store,data,width,height)) return FALSE;
+  return TRUE;
   }
 
 
 // Load object from stream
-void FXGIFImage::loadPixels(FXStream& store){
-  FXColor clearcolor;
+FXbool FXGIFImage::loadPixels(FXStream& store){
   if(options&IMAGE_OWNED){FXFREE(&data);}
-  fxloadGIF(store,data,clearcolor,width,height);
-  options&=~IMAGE_ALPHA;
+  if(!fxloadGIF(store,data,width,height)) return FALSE;
   options|=IMAGE_OWNED;
+  return TRUE;
   }
 
 
 // Clean up
 FXGIFImage::~FXGIFImage(){
   }
+
+}
