@@ -3,7 +3,7 @@
 *                      O p e n G L   C o n e   O b j e c t                      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1999,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * Contributed by: Angel-Ventura Mendo Gomez <ventura@labri.u-bordeaux.fr>       *
 *********************************************************************************
@@ -21,16 +21,17 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGLCone.cpp,v 1.12 2002/01/18 22:43:00 jeroen Exp $                     *
+* $Id: FXGLCone.cpp,v 1.20 2003/10/23 04:19:44 fox Exp $                        *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
-#include "FXVec.h"
-#include "FXHVec.h"
-#include "FXQuat.h"
-#include "FXHMat.h"
+#include "FXVec2f.h"
+#include "FXVec3f.h"
+#include "FXVec4f.h"
+#include "FXQuatf.h"
+#include "FXMat4f.h"
 #include "FXRange.h"
 #include "FXString.h"
 #include "FXSize.h"
@@ -39,6 +40,7 @@
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXObjectList.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXGLViewer.h"
 #include "FXGLCone.h"
@@ -54,18 +56,21 @@
 #define FXGLCONE_STACKS_NUMBER		20
 #define FXGLCONE_LOOPS			4
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
 
 // Object implementation
 FXIMPLEMENT(FXGLCone,FXGLShape,NULL,0)
 
 
 // Create cone
-FXGLCone::FXGLCone():height(1.0),radius(1.0f){
+FXGLCone::FXGLCone():height(1.0f),radius(1.0f){
   FXTRACE((100,"FXGLCone::FXGLCone\n"));
   range[0][0]=-radius; range[0][1]=radius;
-  range[1][0]=0;       range[1][1]=height;
+  range[1][0]=0.0f;    range[1][1]=height;
   range[2][0]=-radius; range[2][1]=radius;
   }
 
@@ -75,7 +80,7 @@ FXGLCone::FXGLCone(FXfloat x,FXfloat y,FXfloat z,FXfloat h,FXfloat r):
   FXGLShape(x,y,z,SHADING_SMOOTH|STYLE_SURFACE),height(h),radius(r){
   FXTRACE((100,"FXGLCone::FXGLCone\n"));
   range[0][0]=-radius; range[0][1]=radius;
-  range[1][0]=0;       range[1][1]=height;
+  range[1][0]=0.0f;    range[1][1]=height;
   range[2][0]=-radius; range[2][1]=radius;
   }
 
@@ -85,7 +90,7 @@ FXGLCone::FXGLCone(FXfloat x,FXfloat y,FXfloat z,FXfloat h, FXfloat r,const FXMa
   FXGLShape(x,y,z,SHADING_SMOOTH|STYLE_SURFACE,mtl,mtl),height(h),radius(r){
   FXTRACE((100,"FXGLCone::FXGLCone\n"));
   range[0][0]=-radius; range[0][1]=radius;
-  range[1][0]=0;       range[1][1]=height;
+  range[1][0]=0.0f;    range[1][1]=height;
   range[2][0]=-radius; range[2][1]=radius;
   }
 
@@ -100,7 +105,7 @@ FXGLCone::FXGLCone(const FXGLCone& orig):FXGLShape(orig){
 
 // Draw
 void FXGLCone::drawshape(FXGLViewer*){
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   GLUquadricObj* quad=gluNewQuadric();
   gluQuadricDrawStyle(quad,(GLenum)GLU_FILL);
   /*
@@ -108,7 +113,7 @@ void FXGLCone::drawshape(FXGLViewer*){
     gluQuadricOrientation(quad,(GLenum)GLU_OUTSIDE);
   */
   glPushMatrix();
-  glRotatef(-90,1,0,0);
+  glRotatef(-90.0f,1.0f,0.0f,0.0f);
   gluCylinder(quad,radius,0,height,FXGLCONE_SLICES_NUMBER,FXGLCONE_STACKS_NUMBER);
   gluQuadricOrientation(quad,(GLenum)GLU_INSIDE);
   gluDisk(quad,0,radius,FXGLCONE_SLICES_NUMBER,FXGLCONE_LOOPS);
@@ -142,3 +147,5 @@ void FXGLCone::load(FXStream& store){
 FXGLCone::~FXGLCone(){
   FXTRACE((100,"FXGLCone::~FXGLCone\n"));
   }
+
+}
