@@ -3,7 +3,7 @@
 *                            V i s u a l   C l a s s                            *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1999,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGLVisual.cpp,v 1.34.4.1 2003/02/28 22:46:02 fox Exp $                   *
+* $Id: FXGLVisual.cpp,v 1.43 2003/10/23 04:19:44 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -33,6 +33,7 @@
 #include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXFont.h"
 #include "FXGLVisual.h"
@@ -86,8 +87,11 @@
 
 #define DISPLAY(app) ((Display*)((app)->display))
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
 
 // Object implementation
 FXIMPLEMENT(FXGLVisual,FXVisual,NULL,0)
@@ -245,11 +249,52 @@ void FXGLVisual::makeOpenGLPalette(){
 /*******************************************************************************/
 
 
+// Test if OpenGL is possible
+FXbool FXGLVisual::supported(FXApp* application,int& major,int& minor){
+  major=minor=0;
+  if(application->isInitialized()){
+#ifdef HAVE_GL_H
+#ifndef WIN32
+    if(!glXQueryExtension(DISPLAY(application),NULL,NULL)) return FALSE;
+    if(!glXQueryVersion(DISPLAY(application),&major,&minor)) return FALSE;
+#if GL_VERSION_1_4
+    if(minor>4) minor=4;
+#elif GL_VERSION_1_3
+    if(minor>3) minor=3;
+#elif GL_VERSION_1_2
+    if(minor>2) minor=2;
+#elif GL_VERSION_1_1
+    if(minor>1) minor=1;
+#else
+    if(minor>0) minor=0;
+#endif
+    return TRUE;
+#else
+    major=1;
+#if GL_VERSION_1_4
+    minor=4;
+#elif GL_VERSION_1_3
+    minor=3;
+#elif GL_VERSION_1_2
+    minor=2;
+#elif GL_VERSION_1_1
+    minor=1;
+#else
+    minor=0;
+#endif
+    return TRUE;
+#endif
+#endif
+    }
+  return FALSE;
+  }
+
+
 // Initialize
 void FXGLVisual::create(){
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   if(!xid){
-    if(getApp()->initialized){
+    if(getApp()->isInitialized()){
       FXTRACE((100,"%s::create %p\n",getClassName(),this));
 #ifndef WIN32
       XVisualInfo vitemplate;
@@ -630,7 +675,7 @@ void FXGLVisual::create(){
 
 // Detach visual
 void FXGLVisual::detach(){
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   if(xid){
     FXTRACE((100,"%s::detach %p\n",getClassName(),this));
     xid=0;
@@ -641,9 +686,9 @@ void FXGLVisual::detach(){
 
 // Destroy visual
 void FXGLVisual::destroy(){
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   if(xid){
-    if(getApp()->initialized){
+    if(getApp()->isInitialized()){
       FXTRACE((100,"%s::destroy %p\n",getClassName(),this));
 #ifndef WIN32
       //if(freemap){XFreeColormap(DISPLAY(getApp()),colormap);}
@@ -667,7 +712,7 @@ void FXGLVisual::destroy(){
 // Get actual red size
 FXint FXGLVisual::getActualRedSize() const {
   if(!info){ fxerror("%s::getActualRedSize: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_RED_SIZE,&s);
@@ -684,7 +729,7 @@ FXint FXGLVisual::getActualRedSize() const {
 // Get actual green size
 FXint FXGLVisual::getActualGreenSize() const {
   if(!info){ fxerror("%s::getActualGreenSize: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_GREEN_SIZE,&s);
@@ -701,7 +746,7 @@ FXint FXGLVisual::getActualGreenSize() const {
 // Get actual blue size
 FXint FXGLVisual::getActualBlueSize() const {
   if(!info){ fxerror("%s::getActualBlueSize: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_BLUE_SIZE,&s);
@@ -718,7 +763,7 @@ FXint FXGLVisual::getActualBlueSize() const {
 // Get actual alpha size
 FXint FXGLVisual::getActualAlphaSize() const {
   if(!info){ fxerror("%s::getActualAlphaSize: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_ALPHA_SIZE,&s);
@@ -736,7 +781,7 @@ FXint FXGLVisual::getActualAlphaSize() const {
 // Get actual depth size
 FXint FXGLVisual::getActualDepthSize() const {
   if(!info){ fxerror("%s::getActualDepthSize: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_DEPTH_SIZE,&s);
@@ -752,7 +797,7 @@ FXint FXGLVisual::getActualDepthSize() const {
 // Get actual stencil size
 FXint FXGLVisual::getActualStencilSize() const {
   if(!info){ fxerror("%s::getActualStencilSize: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_STENCIL_SIZE,&s);
@@ -769,7 +814,7 @@ FXint FXGLVisual::getActualStencilSize() const {
 // Get actual accum red size
 FXint FXGLVisual::getActualAccumRedSize() const {
   if(!info){ fxerror("%s::getActualAccumRedSize: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_ACCUM_RED_SIZE,&s);
@@ -786,7 +831,7 @@ FXint FXGLVisual::getActualAccumRedSize() const {
 // Get actual accum green size
 FXint FXGLVisual::getActualAccumGreenSize() const {
   if(!info){ fxerror("%s::getActualAccumGreenSize: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_ACCUM_GREEN_SIZE,&s);
@@ -803,7 +848,7 @@ FXint FXGLVisual::getActualAccumGreenSize() const {
 // Get actual accum blue size
 FXint FXGLVisual::getActualAccumBlueSize() const {
   if(!info){ fxerror("%s::getActualAccumBlueSize: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_ACCUM_BLUE_SIZE,&s);
@@ -820,7 +865,7 @@ FXint FXGLVisual::getActualAccumBlueSize() const {
 // Get actual accum alpha size
 FXint FXGLVisual::getActualAccumAlphaSize() const {
   if(!info){ fxerror("%s::getActualAccumAlphaSize: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_ACCUM_ALPHA_SIZE,&s);
@@ -837,7 +882,7 @@ FXint FXGLVisual::getActualAccumAlphaSize() const {
 // Is it double buffer
 FXbool FXGLVisual::isDoubleBuffer() const {
   if(!info){ fxerror("%s::isDoubleBuffer: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_DOUBLEBUFFER,&s);
@@ -854,7 +899,7 @@ FXbool FXGLVisual::isDoubleBuffer() const {
 // Is it stereo
 FXbool FXGLVisual::isStereo() const {
   if(!info){ fxerror("%s::isStereo: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
   FXint s;
 #ifndef WIN32
   glXGetConfig(DISPLAY(getApp()),(XVisualInfo*)info,GLX_STEREO,&s);
@@ -871,7 +916,7 @@ FXbool FXGLVisual::isStereo() const {
 // Is it hardware-accelerated?
 FXbool FXGLVisual::isAccelerated() const {
   if(!info){ fxerror("%s::isHardwareAccelerated: visual not yet initialized.\n",getClassName()); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
 #ifndef WIN32
   return TRUE;
 #else
@@ -929,7 +974,7 @@ FXGLVisual::~FXGLVisual(){
 // Create a display list of bitmaps from font glyphs in a font
 void glUseFXFont(FXFont* font,int first,int count,int list){
   if(!font || !font->id()){ fxerror("glUseFXFont: invalid font.\n"); }
-#ifdef HAVE_OPENGL
+#ifdef HAVE_GL_H
 #ifndef WIN32
   glXUseXFont((Font)font->id(),first,count,list);
 #else
@@ -941,3 +986,4 @@ void glUseFXFont(FXFont* font,int first,int count,int list){
 #endif
   }
 
+}
