@@ -3,7 +3,7 @@
 *                               H e a d e r   O b j e c t                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXHeader.cpp,v 1.31.4.3 2003/08/15 12:57:44 fox Exp $                     *
+* $Id: FXHeader.cpp,v 1.66 2003/10/23 04:19:44 fox Exp $                        *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -31,6 +31,7 @@
 #include "FXRectangle.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXDCWindow.h"
 #include "FXFont.h"
@@ -48,103 +49,147 @@
   - Should up/down arrows go sideways when vertically oriented?
   - Perhaps maintain button state in item.
   - Perhaps perform drawing of border in item also.
+  - If icon or text of item changes, need to recalc().
 */
 
 
 #define FUDGE         8
-#define ARROW_SPACING 8
 #define ICON_SPACING  4
 #define HEADER_MASK   (HEADER_BUTTON|HEADER_TRACKING|HEADER_VERTICAL)
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 // Object implementation
 FXIMPLEMENT(FXHeaderItem,FXObject,NULL,0)
 
 
 // Draw item
 void FXHeaderItem::draw(const FXHeader* header,FXDC& dc,FXint x,FXint y,FXint w,FXint h){
-  FXFont *font=header->getFont();
+  register FXint tx,ty,tw,th,ix,iy,iw,ih,s,ml,mr,mt,mb,beg,end,t,xx,yy,bb,aa,ax,ay;
+  register FXFont *font=header->getFont();
 
-  // Clip to inside of header control
+  // Get border width and padding
+  bb=header->getBorderWidth();
+  ml=header->getPadLeft()+bb;
+  mr=header->getPadRight()+bb;
+  mt=header->getPadTop()+bb;
+  mb=header->getPadBottom()+bb;
+
+  // Shrink by margins
+  x+=ml; w-=ml+mr;
+  y+=mt; h-=mt+mb;
+
+  // Initial clip rectangle
   dc.setClipRectangle(x,y,w,h);
 
-  // Account for borders
-  w=w-(header->getPadLeft()+header->getPadRight()+(header->getBorderWidth()<<1));
-  h=h-(header->getPadTop()+header->getPadBottom()+(header->getBorderWidth()<<1));
-  x+=header->getBorderWidth()+header->getPadLeft();
-  y+=header->getBorderWidth()+header->getPadTop();
+  // Text width and height
+  tw=th=iw=ih=beg=s=0;
+  do{
+    end=beg;
+    while(end<label.length() && label[end]!='\n') end++;
+    if((t=font->getTextWidth(&label[beg],end-beg))>tw) tw=t;
+    th+=font->getFontHeight();
+    beg=end+1;
+    }
+  while(end<label.length());
 
-  // Draw icon
+  // Icon size
   if(icon){
-    if(icon->getWidth()<=w){
-      dc.drawIcon(icon,x,y+(h-icon->getHeight())/2);
-      x+=icon->getWidth();
-      w-=icon->getWidth();
-      }
+    iw=icon->getWidth();
+    ih=icon->getHeight();
     }
 
-  // Draw text
-  if(!label.empty()){
-    FXint dw=font->getTextWidth("...",3);
-    FXint num=label.length();
-    FXint tw=font->getTextWidth(label.text(),num);
-    FXint th=font->getFontHeight();
-    FXint ty=y+(h-th)/2+font->getFontAscent();
-    dc.setTextFont(font);
-    if(icon){ x+=ICON_SPACING; w-=ICON_SPACING; }
-    if(tw<=w){
-      dc.setForeground(header->getTextColor());
-      dc.drawText(x,ty,label.text(),num);
-      x+=tw;
-      w-=tw;
-      }
-    else{
-      while(num>0 && (tw=font->getTextWidth(label.text(),num))>(w-dw)) num--;
-      if(num>0){
-        dc.setForeground(header->getTextColor());
-        dc.drawText(x,ty,label.text(),num);
-        dc.drawText(x+tw,ty,"...",3);
-        x+=tw+dw;
-        w-=tw+dw;
-        }
-      else{
-        tw=font->getTextWidth(label.text(),1);
-        if(tw<=w){
-          dc.setForeground(header->getTextColor());
-          dc.drawText(x,ty,label.text(),1);
-          x+=tw;
-          w-=tw;
-          }
-        }
-      }
-    }
+  // Icon-text spacing
+  if(iw && tw) s=ICON_SPACING;
 
   // Draw arrows
-  if(arrow!=MAYBE){
-    FXint aa=(font->getFontHeight()-3)|1;
-    if(icon || !label.empty()){ x+=ARROW_SPACING; w-=ARROW_SPACING; }
-    if(w>aa){
-      if(arrow==TRUE){
-        y=y+(h-aa)/2;
-        dc.setForeground(header->getHiliteColor());
-        dc.drawLine(x+aa/2,y,x+aa-1,y+aa);
-        dc.drawLine(x,y+aa,x+aa,y+aa);
-        dc.setForeground(header->getShadowColor());
-        dc.drawLine(x+aa/2,y,x,y+aa);
-        }
-      else{
-        y=y+(h-aa)/2;
-        dc.setForeground(header->getHiliteColor());
-        dc.drawLine(x+aa/2,y+aa,x+aa-1,y);
-        dc.setForeground(header->getShadowColor());
-        dc.drawLine(x+aa/2,y+aa,x,y);
-        dc.drawLine(x,y,x+aa,y);
-        }
+  if(state&(ARROW_UP|ARROW_DOWN)){
+    aa=(font->getFontHeight()-5)|1;
+    ay=y+(h-aa)/2;
+    ax=x+w-aa-2;
+    if(state&ARROW_UP){
+      dc.setForeground(header->getHiliteColor());
+      dc.drawLine(ax+aa/2,ay,ax+aa-1,ay+aa);
+      dc.drawLine(ax,ay+aa,ax+aa,ay+aa);
+      dc.setForeground(header->getShadowColor());
+      dc.drawLine(ax+aa/2,ay,ax,ay+aa);
       }
+    else{
+      dc.setForeground(header->getHiliteColor());
+      dc.drawLine(ax+aa/2,ay+aa,ax+aa-1,ay);
+      dc.setForeground(header->getShadowColor());
+      dc.drawLine(ax+aa/2,ay+aa,ax,ay);
+      dc.drawLine(ax,ay,ax+aa,ay);
+      }
+    w-=aa+4;
+    dc.setClipRectangle(x,y,w,h);
     }
 
+  // Fix x coordinate
+  if(state&LEFT){
+    if(state&BEFORE){ ix=x; tx=ix+iw+s; }
+    else if(state&AFTER){ tx=x; ix=tx+tw+s; }
+    else{ ix=x; tx=x; }
+    }
+  else if(state&RIGHT){
+    if(state&BEFORE){ tx=x+w-tw; ix=tx-iw-s; }
+    else if(state&AFTER){ ix=x+w-iw; tx=ix-tw-s; }
+    else{ ix=x+w-iw; tx=x+w-tw; }
+    }
+  else{
+    if(state&BEFORE){ ix=x+(w-tw-iw-s)/2; tx=ix+iw+s; }
+    else if(state&AFTER){ tx=x+(w-tw-iw-s)/2; ix=tx+tw+s; }
+    else{ ix=x+(w-iw)/2; tx=x+(w-tw)/2; }
+    }
+
+  // Fix y coordinate
+  if(state&TOP){
+    if(state&ABOVE){ iy=y; ty=iy+ih; }
+    else if(state&BELOW){ ty=y; iy=ty+th; }
+    else{ iy=y; ty=y; }
+    }
+  else if(state&BOTTOM){
+    if(state&ABOVE){ ty=y+h-th; iy=ty-ih; }
+    else if(state&BELOW){ iy=y+h-ih; ty=iy-th; }
+    else{ iy=y+h-ih; ty=y+h-th; }
+    }
+  else{
+    if(state&ABOVE){ iy=y+(h-th-ih)/2; ty=iy+ih; }
+    else if(state&BELOW){ ty=y+(h-th-ih)/2; iy=ty+th; }
+    else{ iy=y+(h-ih)/2; ty=y+(h-th)/2; }
+    }
+
+  // Offset a bit when pressed
+//  if(state&PRESSED){ tx++; ty++; ix++; iy++; }
+
+  // Paint icon
+  if(icon){
+    dc.drawIcon(icon,ix,iy);
+    }
+
+  // Text color
+  dc.setForeground(header->getTextColor());
+
+  // Draw text
+  dc.setFont(font);
+  yy=ty+font->getFontAscent();
+  beg=0;
+  do{
+    end=beg;
+    while(end<label.length() && label[end]!='\n') end++;
+    if(state&LEFT) xx=tx;
+    else if(state&RIGHT) xx=tx+tw-font->getTextWidth(&label[beg],end-beg);
+    else xx=tx+(tw-font->getTextWidth(&label[beg],end-beg))/2;
+    dc.drawText(xx,yy,&label[beg],end-beg);
+    yy+=font->getFontHeight();
+    beg=end+1;
+    }
+  while(end<label.length());
+
   // Restore original clip path
   dc.clearClipRectangle();
   }
@@ -162,14 +207,52 @@ void FXHeaderItem::destroy(){ /*if(icon) icon->destroy();*/ }
 void FXHeaderItem::detach(){ if(icon) icon->detach(); }
 
 
+// Change sort direction
+void FXHeaderItem::setArrowDir(FXbool dir){
+  state&=~(ARROW_UP|ARROW_DOWN);
+  if(dir==TRUE) state|=ARROW_UP; else if(dir==FALSE) state|=ARROW_DOWN;
+  }
+
+
+// Change sort direction
+FXbool FXHeaderItem::getArrowDir() const {
+  return (state&ARROW_UP) ? TRUE : (state&ARROW_DOWN) ? FALSE : MAYBE;
+  }
+
+
+// Change justify mode
+void FXHeaderItem::setJustify(FXuint justify){
+  state=(state&~(RIGHT|LEFT|TOP|BOTTOM)) | (justify&(RIGHT|LEFT|TOP|BOTTOM));
+  }
+
+// Change icon positioning
+void FXHeaderItem::setIconPosition(FXuint mode){
+  state=(state&~(BEFORE|AFTER|ABOVE|BELOW)) | (mode&(BEFORE|AFTER|ABOVE|BELOW));
+  }
+
+
 // Get width of item
 FXint FXHeaderItem::getWidth(const FXHeader* header) const {
-  register FXint w;
+  register FXFont *font=header->getFont();
+  register FXint beg,end,tw,iw,s,t,ml,mr,w;
   if(header->getHeaderStyle()&HEADER_VERTICAL){
-    w=0;
-    if(icon) w=icon->getWidth();
-    if(!label.empty()){ w+=header->getFont()->getTextWidth(label.text(),label.length()); if(icon) w+=ICON_SPACING; }
-    return w+header->getPadLeft()+header->getPadRight()+(header->getBorderWidth()<<1);
+    ml=header->getPadLeft()+header->getBorderWidth();
+    mr=header->getPadRight()+header->getBorderWidth();
+    tw=iw=beg=s=0;
+    if(icon) iw=icon->getWidth();
+    do{
+      end=beg;
+      while(end<label.length() && label[end]!='\n') end++;
+      if((t=font->getTextWidth(&label[beg],end-beg))>tw) tw=t;
+      beg=end+1;
+      }
+    while(end<label.length());
+    if(iw && tw) s=4;
+    if(state&(BEFORE|AFTER))
+      w=iw+tw+s;
+    else
+      w=FXMAX(iw,tw);
+    return ml+mr+w;
     }
   return size;
   }
@@ -177,12 +260,25 @@ FXint FXHeaderItem::getWidth(const FXHeader* header) const {
 
 // Get height of item
 FXint FXHeaderItem::getHeight(const FXHeader* header) const {
-  register FXint th,ih;
+  register FXFont *font=header->getFont();
+  register FXint beg,end,th,ih,h,mt,mb;
   if(!(header->getHeaderStyle()&HEADER_VERTICAL)){
-    th=ih=0;
-    if(!label.empty()) th=header->getFont()->getFontHeight();
+    mt=header->getPadTop()+header->getBorderWidth();
+    mb=header->getPadBottom()+header->getBorderWidth();
+    th=ih=beg=0;
     if(icon) ih=icon->getHeight();
-    return FXMAX(th,ih)+header->getPadTop()+header->getPadBottom()+(header->getBorderWidth()<<1);
+    do{
+      end=beg;
+      while(end<label.length() && label[end]!='\n') end++;
+      th+=font->getFontHeight();
+      beg=end+1;
+      }
+    while(end<label.length());
+    if(state&(ABOVE|BELOW))
+      h=ih+th;
+    else
+      h=FXMAX(ih,th);
+    return h+mt+mb;
     }
   return size;
   }
@@ -194,7 +290,8 @@ void FXHeaderItem::save(FXStream& store) const {
   store << label;
   store << icon;
   store << size;
-  store << arrow;
+  store << pos;
+  store << state;
   }
 
 
@@ -204,7 +301,8 @@ void FXHeaderItem::load(FXStream& store){
   store >> label;
   store >> icon;
   store >> size;
-  store >> arrow;
+  store >> pos;
+  store >> state;
   }
 
 
@@ -234,12 +332,11 @@ FXHeader::FXHeader(){
   nitems=0;
   textColor=0;
   state=FALSE;
-  font=(FXFont*)-1;
-  timer=NULL;
+  font=(FXFont*)-1L;
   active=-1;
   activepos=0;
   activesize=0;
-  off=0;
+  activeoffset=0;
   }
 
 
@@ -253,12 +350,11 @@ FXHeader::FXHeader(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXint
   nitems=0;
   textColor=getApp()->getForeColor();
   font=getApp()->getNormalFont();
-  timer=NULL;
   state=FALSE;
   active=-1;
   activepos=0;
   activesize=0;
-  off=0;
+  activeoffset=0;
   }
 
 
@@ -267,7 +363,9 @@ void FXHeader::create(){
   register FXint i;
   FXFrame::create();
   font->create();
-  for(i=0; i<nitems; i++){if(items[i]->icon){items[i]->icon->create();}}
+  for(i=0; i<nitems; i++){
+    if(items[i]->getIcon()){ items[i]->getIcon()->create(); }
+    }
   }
 
 
@@ -276,7 +374,9 @@ void FXHeader::detach(){
   register FXint i;
   FXFrame::detach();
   font->detach();
-  for(i=0; i<nitems; i++){if(items[i]->icon){items[i]->icon->detach();}}
+  for(i=0; i<nitems; i++){
+    if(items[i]->getIcon()){ items[i]->getIcon()->detach(); }
+    }
   }
 
 
@@ -313,26 +413,27 @@ FXHeaderItem *FXHeader::createItem(const FXString& text,FXIcon* icon,FXint size,
 
 
 // Retrieve item
-FXHeaderItem *FXHeader::retrieveItem(FXint index) const {
-  if(index<0 || nitems<=index){ fxerror("%s::retrieveItem: index out of range.\n",getClassName()); }
+FXHeaderItem *FXHeader::getItem(FXint index) const {
+  if(index<0 || nitems<=index){ fxerror("%s::getItem: index out of range.\n",getClassName()); }
   return items[index];
   }
 
 
 // Replace item with another
-FXint FXHeader::replaceItem(FXint index,FXHeaderItem* item,FXbool notify){
+FXint FXHeader::setItem(FXint index,FXHeaderItem* item,FXbool notify){
 
   // Must have item
-  if(!item){ fxerror("%s::replaceItem: item is NULL.\n",getClassName()); }
+  if(!item){ fxerror("%s::setItem: item is NULL.\n",getClassName()); }
 
   // Must be in range
-  if(index<0 || nitems<=index){ fxerror("%s::replaceItem: index out of range.\n",getClassName()); }
+  if(index<0 || nitems<=index){ fxerror("%s::setItem: index out of range.\n",getClassName()); }
 
   // Notify item will be replaced
-  if(notify && target){target->handle(this,MKUINT(message,SEL_REPLACED),(void*)(FXival)index);}
+  if(notify && target){target->handle(this,FXSEL(SEL_REPLACED,message),(void*)(FXival)index);}
 
   // Copy the size over
   item->setSize(items[index]->getSize());
+  item->setPos(items[index]->getPos());
 
   // Delete old
   delete items[index];
@@ -347,13 +448,14 @@ FXint FXHeader::replaceItem(FXint index,FXHeaderItem* item,FXbool notify){
 
 
 // Replace item with another
-FXint FXHeader::replaceItem(FXint index,const FXString& text,FXIcon *icon,FXint size,void* ptr,FXbool notify){
-  return replaceItem(index,createItem(text,icon,FXMAX(size,0),ptr),notify);
+FXint FXHeader::setItem(FXint index,const FXString& text,FXIcon *icon,FXint size,void* ptr,FXbool notify){
+  return setItem(index,createItem(text,icon,FXMAX(size,0),ptr),notify);
   }
 
 
 // Insert item
 FXint FXHeader::insertItem(FXint index,FXHeaderItem* item,FXbool notify){
+  register FXint i,d;
 
   // Must have item
   if(!item){ fxerror("%s::insertItem: item is NULL.\n",getClassName()); }
@@ -361,6 +463,12 @@ FXint FXHeader::insertItem(FXint index,FXHeaderItem* item,FXbool notify){
   // Must be in range
   if(index<0 || nitems<index){ fxerror("%s::insertItem: index out of range.\n",getClassName()); }
 
+  // New item position
+  item->setPos((0<index)?items[index-1]->getPos()+items[index-1]->getSize():0);
+
+  // Move over remaining items
+  for(i=index,d=item->getSize(); i<nitems; i++) items[i]->setPos(items[i]->getPos()+d);
+
   // Add item to list
   FXRESIZE(&items,FXHeaderItem*,nitems+1);
   memmove(&items[index+1],&items[index],sizeof(FXHeaderItem*)*(nitems-index));
@@ -368,7 +476,7 @@ FXint FXHeader::insertItem(FXint index,FXHeaderItem* item,FXbool notify){
   nitems++;
 
   // Notify item has been inserted
-  if(notify && target){target->handle(this,MKUINT(message,SEL_INSERTED),(void*)(FXival)index);}
+  if(notify && target){target->handle(this,FXSEL(SEL_INSERTED,message),(void*)(FXival)index);}
 
   // Redo layout
   recalc();
@@ -408,17 +516,22 @@ FXint FXHeader::prependItem(const FXString& text,FXIcon *icon,FXint size,void* p
 
 // Remove node from list
 void FXHeader::removeItem(FXint index,FXbool notify){
+  register FXint i,d;
 
   // Must be in range
   if(index<0 || nitems<=index){ fxerror("%s::removeItem: index out of range.\n",getClassName()); }
 
   // Notify item will be deleted
-  if(notify && target){target->handle(this,MKUINT(message,SEL_DELETED),(void*)(FXival)index);}
+  if(notify && target){target->handle(this,FXSEL(SEL_DELETED,message),(void*)(FXival)index);}
+
+  // Adjust remaining columns
+  for(i=index+1,d=items[index]->getSize(); i<nitems; i++) items[i]->setPos(items[i]->getPos()-d);
 
   // Remove item from list
   nitems--;
   delete items[index];
   memmove(&items[index],&items[index+1],sizeof(FXHeaderItem*)*(nitems-index));
+  FXRESIZE(&items,FXHeaderItem*,nitems);
 
   // Redo layout
   recalc();
@@ -430,7 +543,7 @@ void FXHeader::clearItems(FXbool notify){
 
   // Delete items
   for(FXint index=nitems-1; 0<=index; index--){
-    if(notify && target){target->handle(this,MKUINT(message,SEL_DELETED),(void*)(FXival)index);}
+    if(notify && target){target->handle(this,FXSEL(SEL_DELETED,message),(void*)(FXival)index);}
     delete items[index];
     }
 
@@ -443,33 +556,12 @@ void FXHeader::clearItems(FXbool notify){
   }
 
 
-// Get item at offset
-FXint FXHeader::getItemAt(FXint offset) const {
-  register FXint x,y,i,w,h;
-  if(options&HEADER_VERTICAL){
-    for(i=0,y=0; i<nitems; i++){
-      h=items[i]->getHeight(this);
-      if(y<=offset && offset<y+h) return i;
-      y+=h;
-      }
-    }
-  else{
-    for(i=0,x=0; i<nitems; i++){
-      w=items[i]->getWidth(this);
-      if(x<=offset && offset<x+w) return i;
-      x+=w;
-      }
-    }
-  return -1;
-  }
-
-
 // Change item's text
 void FXHeader::setItemText(FXint index,const FXString& text){
   if(index<0 || nitems<=index){ fxerror("%s::setItemText: index out of range.\n",getClassName()); }
   if(items[index]->getText()!=text){
     items[index]->setText(text);
-    update();
+    recalc();
     }
   }
 
@@ -486,7 +578,7 @@ void FXHeader::setItemIcon(FXint index,FXIcon* icon){
   if(index<0 || nitems<=index){ fxerror("%s::setItemIcon: index out of range.\n",getClassName()); }
   if(items[index]->getIcon()!=icon){
     items[index]->setIcon(icon);
-    update();
+    recalc();
     }
   }
 
@@ -500,10 +592,13 @@ FXIcon* FXHeader::getItemIcon(FXint index) const {
 
 // Change item's size
 void FXHeader::setItemSize(FXint index,FXint size){
+  register FXint i,d;
   if(index<0 || nitems<=index){ fxerror("%s::setItemSize: index out of range.\n",getClassName()); }
   if(size<0) size=0;
-  if(items[index]->getSize()!=size){
+  d=size-items[index]->getSize();
+  if(d!=0){
     items[index]->setSize(size);
+    for(i=index+1; i<nitems; i++) items[i]->setPos(items[i]->getPos()+d);
     recalc();
     }
   }
@@ -518,15 +613,27 @@ FXint FXHeader::getItemSize(FXint index) const {
 
 // Get item's offset
 FXint FXHeader::getItemOffset(FXint index) const {
-  register FXint off,i;
   if(index<0 || nitems<=index){ fxerror("%s::getItemOffset: index out of range.\n",getClassName()); }
-  if(options&HEADER_VERTICAL){
-    for(i=0,off=0; i<index; i++){off+=items[i]->getHeight(this);}
+  return items[index]->getPos();
+  }
+
+
+// Get index of item at offset
+FXint FXHeader::getItemAt(FXint offset) const {
+  register FXint h=nitems-1,l=0,m;
+  if(l<=h){
+    if(offset<items[l]->getPos()) return -1;
+    if(offset>items[h]->getPos()+items[h]->getSize()) return nitems;
+    do{
+      m=(h+l)>>1;
+      if(offset<items[m]->getPos()) h=m-1;
+      else if(items[m]->getPos()+items[m]->getSize()<=offset) l=m+1;
+      else break;
+      }
+    while(h>=l);
+    return m;
     }
-  else{
-    for(i=0,off=0; i<index; i++){off+=items[i]->getWidth(this);}
-    }
-  return off;
+  return -1;
   }
 
 
@@ -561,6 +668,40 @@ FXbool FXHeader::getArrowDir(FXint index) const {
   }
 
 
+// Change item justification
+void FXHeader::setItemJustify(FXint index,FXuint justify){
+  if(index<0 || nitems<=index){ fxerror("%s::setItemJustify: index out of range.\n",getClassName()); }
+  if(items[index]->getJustify()!=justify){
+    items[index]->setJustify(justify);
+    update();
+    }
+  }
+
+
+// Return item justification
+FXuint FXHeader::getItemJustify(FXint index) const {
+  if(index<0 || nitems<=index){ fxerror("%s::getItemJustify: index out of range.\n",getClassName()); }
+  return items[index]->getJustify();
+  }
+
+
+// Change relative position of icon and text of item
+void FXHeader::setItemIconPosition(FXint index,FXuint mode){
+  if(index<0 || nitems<=index){ fxerror("%s::setItemIconPosition: index out of range.\n",getClassName()); }
+  if(items[index]->getIconPosition()!=mode){
+    items[index]->setIconPosition(mode);
+    update();
+    }
+  }
+
+
+// Return relative icon and text position
+FXuint FXHeader::getItemIconPosition(FXint index) const {
+  if(index<0 || nitems<=index){ fxerror("%s::getItemIconPosition: index out of range.\n",getClassName()); }
+  return items[index]->getIconPosition();
+  }
+
+
 // Do layout
 void FXHeader::layout(){
 
@@ -576,14 +717,44 @@ void FXHeader::layout(){
 long FXHeader::onPaint(FXObject*,FXSelector,void* ptr){
   FXEvent *ev=(FXEvent*)ptr;
   FXDCWindow dc(this,ev);
-  FXint x,y,w,h,i;
+  register FXint x,y,w,h,i,ilo,ihi;
+
+  // Paint background
   dc.setForeground(backColor);
   dc.fillRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
-  if(options&HEADER_VERTICAL){
-    for(i=0,y=0; i<nitems; i++){
-      h=items[i]->getHeight(this);
-      if(ev->rect.y<y+h && y<ev->rect.y+ev->rect.h){
-        items[i]->draw(this,dc,0,y,width,h);
+
+  // Got items
+  if(nitems){
+    if(options&HEADER_VERTICAL){
+
+      // Determine affected items
+      ilo=getItemAt(ev->rect.y);
+      ihi=getItemAt(ev->rect.y+ev->rect.h);
+
+      // Fragment below first item
+      if(ilo<0){
+        y=items[0]->getPos();
+        if(0<y){
+          if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,0,0,width,y);
+          else drawRaisedRectangle(dc,0,0,width,y);
+          }
+        ilo=0;
+        }
+
+      // Fragment above last item
+      if(ihi>=nitems){
+        y=items[nitems-1]->getPos()+items[nitems-1]->getSize();
+        if(y<height){
+          if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,0,y,width,height-y);
+          else drawRaisedRectangle(dc,0,y,width,height-y);
+          }
+        ihi=nitems-1;
+        }
+
+      // Draw only affected items
+      for(i=ilo; i<=ihi; i++){
+        y=items[i]->getPos();
+        h=items[i]->getSize();
         if(i==active && state){
           if(options&FRAME_THICK) drawDoubleSunkenRectangle(dc,0,y,width,h);
           else drawSunkenRectangle(dc,0,y,width,h);
@@ -592,19 +763,39 @@ long FXHeader::onPaint(FXObject*,FXSelector,void* ptr){
           if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,0,y,width,h);
           else drawRaisedRectangle(dc,0,y,width,h);
           }
+        items[i]->draw(this,dc,0,y,width,h);
         }
-      y+=h;
       }
-    if(y<height){
-      if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,0,y,width,height-y);
-      else drawRaisedRectangle(dc,0,y,width,height-y);
-      }
-    }
-  else{
-    for(i=0,x=0; i<nitems; i++){
-      w=items[i]->getWidth(this);
-      if(ev->rect.x<x+w && x<ev->rect.x+ev->rect.w){
-        items[i]->draw(this,dc,x,0,w,height);
+    else{
+
+      // Determine affected items
+      ilo=getItemAt(ev->rect.x);
+      ihi=getItemAt(ev->rect.x+ev->rect.w);
+
+      // Fragment below first item
+      if(ilo<0){
+        x=items[0]->getPos();
+        if(0<x){
+          if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,0,0,x,height);
+          else drawRaisedRectangle(dc,0,0,x,height);
+          }
+        ilo=0;
+        }
+
+      // Fragment above last item
+      if(ihi>=nitems){
+        x=items[nitems-1]->getPos()+items[nitems-1]->getSize();
+        if(x<width){
+          if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,x,0,width-x,height);
+          else drawRaisedRectangle(dc,x,0,width-x,height);
+          }
+        ihi=nitems-1;
+        }
+
+      // Draw only the affected items
+      for(i=ilo; i<=ihi; i++){
+        x=items[i]->getPos();
+        w=items[i]->getSize();
         if(i==active && state){
           if(options&FRAME_THICK) drawDoubleSunkenRectangle(dc,x,0,w,height);
           else drawSunkenRectangle(dc,x,0,w,height);
@@ -613,12 +804,8 @@ long FXHeader::onPaint(FXObject*,FXSelector,void* ptr){
           if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,x,0,w,height);
           else drawRaisedRectangle(dc,x,0,w,height);
           }
+        items[i]->draw(this,dc,x,0,w,height);
         }
-      x+=w;
-      }
-    if(x<width){
-      if(options&FRAME_THICK) drawDoubleRaisedRectangle(dc,x,0,width-x,height);
-      else drawRaisedRectangle(dc,x,0,width-x,height);
       }
     }
   return 1;
@@ -631,9 +818,9 @@ long FXHeader::onQueryTip(FXObject* sender,FXSelector,void*){
   if(flags&FLAG_TIP){
     getCursorPosition(cx,cy,btns);
     index=getItemAt((options&HEADER_VERTICAL)?cy:cx);
-    if(0<=index){
+    if(0<=index && index<nitems){
       FXString string=items[index]->getText();
-      sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&string);
+      sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&string);
       return 1;
       }
     }
@@ -645,7 +832,7 @@ long FXHeader::onQueryTip(FXObject* sender,FXSelector,void*){
 long FXHeader::onQueryHelp(FXObject* sender,FXSelector,void*){
   if(!help.empty() && (flags&FLAG_HELP)){
     FXTRACE((250,"%s::onQueryHelp %p\n",getClassName(),this));
-    sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&help);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&help);
     return 1;
     }
   return 0;
@@ -655,7 +842,6 @@ long FXHeader::onQueryHelp(FXObject* sender,FXSelector,void*){
 // We timed out, i.e. the user didn't move for a while
 long FXHeader::onTipTimer(FXObject*,FXSelector,void*){
   FXTRACE((250,"%s::onTipTimer %p\n",getClassName(),this));
-  timer=NULL;
   flags|=FLAG_TIP;
   return 1;
   }
@@ -665,110 +851,70 @@ long FXHeader::onTipTimer(FXObject*,FXSelector,void*){
 // Button being pressed
 long FXHeader::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   FXEvent* ev=(FXEvent*)ptr;
-  FXint i,x,y,w,h;
   flags&=~FLAG_TIP;
-  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+  handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr);
   if(isEnabled()){
     grab();
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
-    if(options&HEADER_VERTICAL){
-      for(i=0,y=0; i<nitems; i++){
-        h=items[i]->getHeight(this);
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONPRESS,message),ptr)) return 1;
 
-        // Hit a header button?
-        if((options&HEADER_BUTTON) && y+FUDGE<=ev->win_y && ev->win_y<y+h-FUDGE){
-          active=i;
-          activepos=y;
-          activesize=h;
+    // Vertically oriented
+    if(options&HEADER_VERTICAL){
+      active=getItemAt(ev->win_y);
+      if(0<=active){
+        if(active<nitems && items[active]->getPos()+items[active]->getSize()-FUDGE<ev->win_y){
+          activepos=items[active]->getPos();
+          activesize=items[active]->getSize();
+          activeoffset=ev->win_y-activepos-activesize;
+          setDragCursor(getApp()->getDefaultCursor(DEF_VSPLIT_CURSOR));
+          flags|=FLAG_PRESSED|FLAG_TRYDRAG;
+          }
+        else if(0<active && ev->win_y<items[active-1]->getPos()+items[active-1]->getSize()+FUDGE){
+          active--;
+          activepos=items[active]->getPos();
+          activesize=items[active]->getSize();
+          activeoffset=ev->win_y-activepos-activesize;
+          setDragCursor(getApp()->getDefaultCursor(DEF_VSPLIT_CURSOR));
+          flags|=FLAG_PRESSED|FLAG_TRYDRAG;
+          }
+        else if(active<nitems){
+          activepos=items[active]->getPos();
+          activesize=items[active]->getSize();
           state=TRUE;
           update(0,activepos,width,activesize);
-          flags&=~FLAG_UPDATE;
-          flags|=FLAG_PRESSED;
-          break;
-          }
-
-        // Upper end of a button
-        if(y+h-FUDGE<=ev->win_y && ev->win_y<y+h){
-          setDragCursor(getApp()->getDefaultCursor(DEF_VSPLIT_CURSOR));
-          active=i;
-          activepos=y;
-          activesize=h;
-          flags|=FLAG_DODRAG;
-          flags&=~FLAG_UPDATE;
-          flags|=FLAG_PRESSED;
-          break;
-          }
-
-        // Lower end of last button at this place
-        if(y+h<=ev->win_y && ev->win_y<y+h+FUDGE){
-          setDragCursor(getApp()->getDefaultCursor(DEF_VSPLIT_CURSOR));
-          active=i;
-          activepos=y;
-          activesize=h;
-          flags|=FLAG_DODRAG;
-          flags&=~FLAG_UPDATE;
           flags|=FLAG_PRESSED;
           }
-
-        // Hit nothing, next item
-        y+=h;
-        }
-
-      // Dragging
-      if(flags&FLAG_DODRAG){
-        off=ev->win_y-activepos-activesize;
-        if(!(options&HEADER_TRACKING)) drawSplit(activepos+activesize);
         }
       }
-    else{
-      for(i=0,x=0; i<nitems; i++){
-        w=items[i]->getWidth(this);
 
-        // Hit a header button?
-        if((options&HEADER_BUTTON) && x+FUDGE<=ev->win_x && ev->win_x<x+w-FUDGE){
-          active=i;
-          activepos=x;
-          activesize=w;
+    // Horizontally oriented
+    else{
+      active=getItemAt(ev->win_x);
+      if(0<=active){
+        if(active<nitems && items[active]->getPos()+items[active]->getSize()-FUDGE<ev->win_x){
+          activepos=items[active]->getPos();
+          activesize=items[active]->getSize();
+          activeoffset=ev->win_x-activepos-activesize;
+          setDragCursor(getApp()->getDefaultCursor(DEF_HSPLIT_CURSOR));
+          flags|=FLAG_PRESSED|FLAG_TRYDRAG;
+          }
+        else if(0<active && ev->win_x<items[active-1]->getPos()+items[active-1]->getSize()+FUDGE){
+          active--;
+          activepos=items[active]->getPos();
+          activesize=items[active]->getSize();
+          activeoffset=ev->win_x-activepos-activesize;
+          setDragCursor(getApp()->getDefaultCursor(DEF_HSPLIT_CURSOR));
+          flags|=FLAG_PRESSED|FLAG_TRYDRAG;
+          }
+        else if(active<nitems){
+          activepos=items[active]->getPos();
+          activesize=items[active]->getSize();
           state=TRUE;
           update(activepos,0,activesize,height);
-          flags&=~FLAG_UPDATE;
-          flags|=FLAG_PRESSED;
-          break;
-          }
-
-        // Upper end of a button
-        if(x+w-FUDGE<=ev->win_x && ev->win_x<x+w){
-          setDragCursor(getApp()->getDefaultCursor(DEF_HSPLIT_CURSOR));
-          active=i;
-          activepos=x;
-          activesize=w;
-          flags|=FLAG_DODRAG;
-          flags&=~FLAG_UPDATE;
-          flags|=FLAG_PRESSED;
-          break;
-          }
-
-        // Lower end of last button at this place
-        if(x+w<=ev->win_x && ev->win_x<x+w+FUDGE){
-          setDragCursor(getApp()->getDefaultCursor(DEF_HSPLIT_CURSOR));
-          active=i;
-          activepos=x;
-          activesize=w;
-          flags|=FLAG_DODRAG;
-          flags&=~FLAG_UPDATE;
           flags|=FLAG_PRESSED;
           }
-
-        // Hit nothing, next item
-        x+=w;
-        }
-
-      // Dragging
-      if(flags&FLAG_DODRAG){
-        off=ev->win_x-activepos-activesize;
-        if(!(options&HEADER_TRACKING)) drawSplit(activepos+activesize);
         }
       }
+    flags&=~FLAG_UPDATE;
     return 1;
     }
   return 0;
@@ -777,27 +923,33 @@ long FXHeader::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
 
 // Button being released
 long FXHeader::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
-  flags&=~FLAG_PRESSED;
-  flags|=FLAG_UPDATE;
+  FXEvent* ev=(FXEvent*)ptr;
+  FXuint flg=flags;
   if(isEnabled()){
     ungrab();
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
-    if(flags&FLAG_DODRAG){
-      setDragCursor(getApp()->getDefaultCursor(DEF_ARROW_CURSOR));
+    flags|=FLAG_UPDATE;
+    flags&=~(FLAG_PRESSED|FLAG_DODRAG|FLAG_TRYDRAG);
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONRELEASE,message),ptr)) return 1;
+    setDragCursor(getApp()->getDefaultCursor(DEF_ARROW_CURSOR));
+    if(!ev->moved && (flg&FLAG_TRYDRAG)){
+      if(target) target->handle(this,FXSEL(SEL_CLICKED,message),(void*)(FXival)active);
+      }
+    else if(flg&FLAG_DODRAG){
       if(!(options&HEADER_TRACKING)){
         drawSplit(activepos+activesize);
         setItemSize(active,activesize);
-        if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)(FXival)active);
+        if(target) target->handle(this,FXSEL(SEL_CHANGED,message),(void*)(FXival)active);
         }
-      flags&=~FLAG_DODRAG;
       }
-    else if(state){
-      state=FALSE;
-      if(options&HEADER_VERTICAL)
-        update(0,activepos,width,activesize);
-      else
-        update(activepos,0,activesize,height);
-      if(target) target->handle(this,MKUINT(message,SEL_COMMAND),(void*)(FXival)active);
+    else{
+      if(state){
+        state=FALSE;
+        if(options&HEADER_VERTICAL)
+          update(0,activepos,width,activesize);
+        else
+          update(activepos,0,activesize,height);
+        if(target) target->handle(this,FXSEL(SEL_COMMAND,message),(void*)(FXival)active);
+        }
       }
     return 1;
     }
@@ -808,25 +960,32 @@ long FXHeader::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
 // Button being moved
 long FXHeader::onMotion(FXObject*,FXSelector,void* ptr){
   FXEvent* ev=(FXEvent*)ptr;
-  FXint i,x,y,w,h,oldsplit,newsplit;
+  FXint oldsplit,newsplit,index;
   FXuint flg=flags;
 
   // Kill the tip
   flags&=~FLAG_TIP;
 
   // Kill the tip timer
-  if(timer) timer=getApp()->removeTimeout(timer);
+  getApp()->removeTimeout(this,ID_TIPTIMER);
 
   // Had the button pressed
   if(flags&FLAG_PRESSED){
 
+    // Dragging
+    if(flags&FLAG_TRYDRAG){
+      if(!(options&HEADER_TRACKING)) drawSplit(activepos+activesize);
+      flags&=~FLAG_TRYDRAG;
+      flags|=FLAG_DODRAG;
+      }
+
     // We were dragging a split
-    if(flags&FLAG_DODRAG){
+    else if(flags&FLAG_DODRAG){
       oldsplit=activepos+activesize;
       if(options&HEADER_VERTICAL)
-        activesize=ev->win_y-off-activepos;
+        activesize=ev->win_y-activeoffset-activepos;
       else
-        activesize=ev->win_x-off-activepos;
+        activesize=ev->win_x-activeoffset-activepos;
       if(activesize<0) activesize=0;
       newsplit=activepos+activesize;
       if(newsplit!=oldsplit){
@@ -836,7 +995,7 @@ long FXHeader::onMotion(FXObject*,FXSelector,void* ptr){
           }
         else{
           setItemSize(active,activesize);
-          if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)(FXival)active);
+          if(target) target->handle(this,FXSEL(SEL_CHANGED,message),(void*)(FXival)active);
           }
         }
       }
@@ -872,39 +1031,35 @@ long FXHeader::onMotion(FXObject*,FXSelector,void* ptr){
           }
         }
       }
-    return 1;
     }
 
-  // If over a split, show split cursor
-  if(isEnabled()){
+  // Just hovering over the widget
+  else{
+
+    // When hovering over a split, show the split cursor
     if(options&HEADER_VERTICAL){
-      for(i=0,y=0; i<nitems; i++){
-        h=items[i]->getHeight(this);
-        if(y+h-FUDGE<=ev->win_y && ev->win_y<y+h+FUDGE){
-          setDefaultCursor(getApp()->getDefaultCursor(DEF_VSPLIT_CURSOR));
-          return 1;
-          }
-        y+=h;
+      index=getItemAt(ev->win_y-FUDGE);
+      if(0<=index && index<nitems && items[index]->getPos()+items[index]->getSize()-FUDGE<ev->win_y){
+        setDefaultCursor(getApp()->getDefaultCursor(DEF_VSPLIT_CURSOR));
+        }
+      else{
+        setDefaultCursor(getApp()->getDefaultCursor(DEF_ARROW_CURSOR));
         }
       }
     else{
-      for(i=0,x=0; i<nitems; i++){
-        w=items[i]->getWidth(this);
-        if(x+w-FUDGE<=ev->win_x && ev->win_x<x+w+FUDGE){
-          setDefaultCursor(getApp()->getDefaultCursor(DEF_HSPLIT_CURSOR));
-          return 1;
-          }
-        x+=w;
+      index=getItemAt(ev->win_x-FUDGE);
+      if(0<=index && index<nitems && items[index]->getPos()+items[index]->getSize()-FUDGE<ev->win_x){
+        setDefaultCursor(getApp()->getDefaultCursor(DEF_HSPLIT_CURSOR));
+        }
+      else{
+        setDefaultCursor(getApp()->getDefaultCursor(DEF_ARROW_CURSOR));
         }
       }
+
+    // Reset tip timer if nothing's going on
+    getApp()->addTimeout(this,ID_TIPTIMER,getApp()->getMenuPause());
     }
 
-  // Not over a split, back to normal cursor
-  setDefaultCursor(getApp()->getDefaultCursor(DEF_ARROW_CURSOR));
-
-  // Reset tip timer if nothing's going on
-  timer=getApp()->addTimeout(getApp()->getMenuPause(),this,ID_TIPTIMER);
-
   // Force GUI update only when needed
   return (flg&FLAG_TIP);
   }
@@ -1007,9 +1162,9 @@ void FXHeader::load(FXStream& store){
 
 // Clean up
 FXHeader::~FXHeader(){
-  if(timer){getApp()->removeTimeout(timer);}
+  getApp()->removeTimeout(this,ID_TIPTIMER);
   clearItems();
-  items=(FXHeaderItem**)-1;
-  font=(FXFont*)-1;
-  timer=(FXTimer*)-1;
+  items=(FXHeaderItem**)-1L;
+  font=(FXFont*)-1L;
   }
+}
