@@ -3,7 +3,7 @@
 *              H o r i z o n t a l   C o n t a i n e r   O b j e c t            *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXHorizontalFrame.cpp,v 1.13.4.1 2003/09/18 13:54:29 fox Exp $            *
+* $Id: FXHorizontalFrame.cpp,v 1.18 2003/10/23 04:19:44 fox Exp $               *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -30,6 +30,7 @@
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXHorizontalFrame.h"
 
@@ -43,10 +44,12 @@
 */
 
 
-
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 // Map
 FXDEFMAP(FXHorizontalFrame) FXHorizontalFrameMap[]={
   FXMAPFUNC(SEL_FOCUS_PREV,0,FXHorizontalFrame::onFocusLeft),
@@ -234,4 +237,5 @@ void FXHorizontalFrame::layout(){
   flags&=~FLAG_DIRTY;
   }
 
+}
 
