@@ -3,7 +3,7 @@
 *                       M e n u    B u t t o n    O b j e c t                   *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXMenuButton.cpp,v 1.25.4.1 2002/07/08 15:16:58 fox Exp $                 *
+* $Id: FXMenuButton.cpp,v 1.35 2003/10/23 04:19:45 fox Exp $                    *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -32,6 +32,7 @@
 #include "FXRectangle.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXDCWindow.h"
 #include "FXFont.h"
@@ -39,6 +40,8 @@
 #include "FXMenuButton.h"
 #include "FXMenuPane.h"
 
+#include "FXMenuBar.h"
+
 /*
   Notes:
   - You can turn off arrows.
@@ -68,6 +71,7 @@
   - You can specify horizontal or vertical offset to position the pane away
     from the button a bit.
   - Should it grab first before doing the POST?
+  - GUI update disabled while menu is popped up.
 */
 
 #define MENUBUTTONARROW_WIDTH   11
@@ -77,9 +81,12 @@
 #define POPUP_MASK              (MENUBUTTON_UP|MENUBUTTON_LEFT)
 #define ATTACH_MASK             (MENUBUTTON_ATTACH_RIGHT|MENUBUTTON_ATTACH_CENTER)
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 // Map
 FXDEFMAP(FXMenuButton) FXMenuButtonMap[]={
   FXMAPFUNC(SEL_PAINT,0,FXMenuButton::onPaint),
@@ -107,7 +114,7 @@ FXIMPLEMENT(FXMenuButton,FXLabel,FXMenuButtonMap,ARRAYNUMBER(FXMenuButtonMap))
 
 // Deserialization
 FXMenuButton::FXMenuButton(){
-  pane=(FXPopup*)-1;
+  pane=(FXPopup*)-1L;
   offsetx=0;
   offsety=0;
   state=FALSE;
@@ -225,15 +232,13 @@ long FXMenuButton::onLeave(FXObject* sender,FXSelector sel,void* ptr){
 // Pressed left button
 long FXMenuButton::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   flags&=~FLAG_TIP;
-  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+  handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr);
   if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONPRESS,message),ptr)) return 1;
     if(state)
-      handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+      handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
     else
-      handle(this,MKUINT(ID_POST,SEL_COMMAND),NULL);
-    flags|=FLAG_PRESSED;
-    flags&=~FLAG_UPDATE;
+      handle(this,FXSEL(SEL_COMMAND,ID_POST),NULL);
     return 1;
     }
   return 0;
@@ -244,10 +249,8 @@ long FXMenuButton::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
 long FXMenuButton::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
   FXEvent* ev=(FXEvent*)ptr;
   if(isEnabled()){
-    flags|=FLAG_UPDATE;
-    flags&=~FLAG_PRESSED;
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
-    if(ev->moved){ handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL); }
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONRELEASE,message),ptr)) return 1;
+    if(ev->moved){ handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL); }
     return 1;
     }
   return 0;
@@ -276,9 +279,7 @@ long FXMenuButton::onMotion(FXObject*,FXSelector,void* ptr){
 // The widget lost the grab for some reason
 long FXMenuButton::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
   FXLabel::onUngrabbed(sender,sel,ptr);
-  handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
-  flags&=~FLAG_PRESSED;
-  flags|=FLAG_UPDATE;
+  handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
   return 1;
   }
 
@@ -289,12 +290,12 @@ long FXMenuButton::onKeyPress(FXObject*,FXSelector sel,void* ptr){
   flags&=~FLAG_TIP;
   if(pane && pane->shown() && pane->handle(pane,sel,ptr)) return 1;
   if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_KEYPRESS),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_KEYPRESS,message),ptr)) return 1;
     if(event->code==KEY_space || event->code==KEY_KP_Space){
       if(state)
-        handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+        handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
       else
-        handle(this,MKUINT(ID_POST,SEL_COMMAND),NULL);
+        handle(this,FXSEL(SEL_COMMAND,ID_POST),NULL);
       return 1;
       }
     }
@@ -307,7 +308,7 @@ long FXMenuButton::onKeyRelease(FXObject*,FXSelector sel,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   if(pane && pane->shown() && pane->handle(pane,sel,ptr)) return 1;
   if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_KEYRELEASE),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_KEYRELEASE,message),ptr)) return 1;
     if(event->code==KEY_space || event->code==KEY_KP_Space){
       return 1;
       }
@@ -320,12 +321,12 @@ long FXMenuButton::onKeyRelease(FXObject*,FXSelector sel,void* ptr){
 long FXMenuButton::onHotKeyPress(FXObject*,FXSelector,void* ptr){
   FXTRACE((200,"%s::onHotKeyPress %p\n",getClassName(),this));
   flags&=~FLAG_TIP;
-  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+  handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr);
   if(isEnabled()){
     if(state)
-      handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+      handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
     else
-      handle(this,MKUINT(ID_POST,SEL_COMMAND),NULL);
+      handle(this,FXSEL(SEL_COMMAND,ID_POST),NULL);
     }
   return 1;
   }
@@ -401,6 +402,7 @@ long FXMenuButton::onCmdPost(FXObject*,FXSelector,void*){
       pane->popup(this,x,y,w,h);
       if(!grabbed()) grab();
       }
+    flags&=~FLAG_UPDATE;
     state=TRUE;
     update();
     }
@@ -415,6 +417,7 @@ long FXMenuButton::onCmdUnpost(FXObject*,FXSelector,void*){
       pane->popdown();
       if(grabbed()) ungrab();
       }
+    flags|=FLAG_UPDATE;
     state=FALSE;
     update();
     }
@@ -596,7 +599,7 @@ long FXMenuButton::onPaint(FXObject*,FXSelector,void* ptr){
 
   // Draw text
   if(!label.empty()){
-    dc.setTextFont(font);
+    dc.setFont(font);
     if(isEnabled()){
       dc.setForeground(textColor);
       drawLabel(dc,label,hotoff,tx,ty,tw,th);
@@ -618,7 +621,7 @@ long FXMenuButton::onPaint(FXObject*,FXSelector,void* ptr){
 // Out of focus chain
 void FXMenuButton::killFocus(){
   FXLabel::killFocus();
-  handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+  handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
   }
 
 
@@ -697,5 +700,7 @@ void FXMenuButton::load(FXStream& store){
 
 // Delete it
 FXMenuButton::~FXMenuButton(){
-  pane=(FXPopup*)-1;
+  pane=(FXPopup*)-1L;
   }
+
+}
