@@ -3,7 +3,7 @@
 *                       M e n u   C a p t i o n   W i d g e t                   *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXMenuCaption.cpp,v 1.25 2002/01/18 22:43:01 jeroen Exp $                *
+* $Id: FXMenuCaption.cpp,v 1.40 2003/10/23 17:53:17 fox Exp $                   *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -31,6 +31,7 @@
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXDCWindow.h"
 #include "FXFont.h"
@@ -44,7 +45,6 @@
     accelerator key combination.
   - When menu label changes, hotkey might have to be adjusted.
   - Fix it so menu stays up when after Alt-F, you press Alt-E.
-  - Menu items should be derived from FXLabel.
   - FXMenuCascade should send ID_POST/ID_UNPOST to self.
   - Look into SEL_FOCUS_SELF some more:- menus should not
     get focus, or at least, return the focus to the original
@@ -55,17 +55,25 @@
 
 #define LEADSPACE   22
 #define TRAILSPACE  16
+#define MENU_MASK   (MENU_AUTOGRAY|MENU_AUTOHIDE)
+
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
 
 // Map
 FXDEFMAP(FXMenuCaption) FXMenuCaptionMap[]={
   FXMAPFUNC(SEL_PAINT,0,FXMenuCaption::onPaint),
   FXMAPFUNC(SEL_UPDATE,0,FXMenuCaption::onUpdate),
-  FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_HELP,FXMenuCaption::onQueryHelp),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETSTRINGVALUE,FXMenuCaption::onCmdSetStringValue),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_GETSTRINGVALUE,FXMenuCaption::onCmdGetStringValue),
+  FXMAPFUNC(SEL_UPDATE,FXMenuCaption::ID_QUERY_HELP,FXMenuCaption::onQueryHelp),
+  FXMAPFUNC(SEL_COMMAND,FXMenuCaption::ID_SETSTRINGVALUE,FXMenuCaption::onCmdSetStringValue),
+  FXMAPFUNC(SEL_COMMAND,FXMenuCaption::ID_GETSTRINGVALUE,FXMenuCaption::onCmdGetStringValue),
+  FXMAPFUNC(SEL_COMMAND,FXMenuCaption::ID_SETICONVALUE,FXMenuCaption::onCmdSetIconValue),
+  FXMAPFUNC(SEL_COMMAND,FXMenuCaption::ID_GETICONVALUE,FXMenuCaption::onCmdGetIconValue),
+  FXMAPFUNC(SEL_COMMAND,FXMenuCaption::ID_SETHELPSTRING,FXMenuCaption::onCmdSetHelp),
+  FXMAPFUNC(SEL_COMMAND,FXMenuCaption::ID_GETHELPSTRING,FXMenuCaption::onCmdGetHelp),
   };
 
 
@@ -82,13 +90,14 @@ FXMenuCaption::FXMenuCaption(){
 // Menu entry
 FXMenuCaption::FXMenuCaption(FXComposite* p,const FXString& text,FXIcon* ic,FXuint opts):
   FXWindow(p,opts,0,0,0,0){
+  FXString string=text.section('\t',0);
   flags|=FLAG_SHOWN;
-  label=text.extract(0,'\t','&');
-  help=text.extract(2,'\t');
+  label=fxstripHotKey(string);
+  help=text.section('\t',2);
   icon=ic;
   font=getApp()->getNormalFont();
-  hotkey=fxparsehotkey(text.text());
-  hotoff=fxfindhotkeyoffset(text.text());
+  hotkey=fxparseHotKey(string);
+  hotoff=fxfindHotKey(string);
   addHotKey(hotkey);
   textColor=getApp()->getForeColor();
   seltextColor=getApp()->getSelforeColor();
@@ -144,18 +153,32 @@ FXint FXMenuCaption::getDefaultWidth(){
 
 // Get default height
 FXint FXMenuCaption::getDefaultHeight(){
-  FXint th,ih,h;
-  th=ih=h=0;
+  FXint th,ih;
+  th=ih=0;
   if(!label.empty()) th=font->getFontHeight()+5;
   if(icon) ih=icon->getHeight()+5;
   return FXMAX(th,ih);
   }
 
 
+// Set help using a message
+long FXMenuCaption::onCmdSetHelp(FXObject*,FXSelector,void* ptr){
+  setHelpText(*((FXString*)ptr));
+  return 1;
+  }
+
+
+// Get help using a message
+long FXMenuCaption::onCmdGetHelp(FXObject*,FXSelector,void* ptr){
+  *((FXString*)ptr)=getHelpText();
+  return 1;
+  }
+
+
 // We were asked about status text
 long FXMenuCaption::onQueryHelp(FXObject* sender,FXSelector,void*){
   if(!help.empty() && (flags&FLAG_HELP)){
-    sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&help);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&help);
     return 1;
     }
   return 0;
@@ -185,7 +208,7 @@ long FXMenuCaption::onPaint(FXObject*,FXSelector,void* ptr){
     if(icon->getWidth()+5>xx) xx=icon->getWidth()+5;
     }
   if(!label.empty()){
-    dc.setTextFont(font);
+    dc.setFont(font);
     dc.setForeground(textColor);
     yy=font->getFontAscent()+(height-font->getFontHeight())/2;
     dc.drawText(xx,yy,label.text(),label.length());
@@ -199,7 +222,6 @@ long FXMenuCaption::onPaint(FXObject*,FXSelector,void* ptr){
 
 // Update value from a message
 long FXMenuCaption::onCmdSetStringValue(FXObject*,FXSelector,void* ptr){
-  if(ptr==NULL){ fxerror("%s::onCmdSetStringValue: NULL pointer.\n",getClassName()); }
   setText(*((FXString*)ptr));
   return 1;
   }
@@ -207,12 +229,25 @@ long FXMenuCaption::onCmdSetStringValue(FXObject*,FXSelector,void* ptr){
 
 // Obtain value from text field
 long FXMenuCaption::onCmdGetStringValue(FXObject*,FXSelector,void* ptr){
-  if(ptr==NULL){ fxerror("%s::onCmdGetStringValue: NULL pointer.\n",getClassName()); }
   *((FXString*)ptr)=getText();
   return 1;
   }
 
 
+// Update icon from a message
+long FXMenuCaption::onCmdSetIconValue(FXObject*,FXSelector,void* ptr){
+  setIcon(*((FXIcon**)ptr));
+  return 1;
+  }
+
+
+// Obtain icon from text field
+long FXMenuCaption::onCmdGetIconValue(FXObject*,FXSelector,void* ptr){
+  *((FXIcon**)ptr)=getIcon();
+  return 1;
+  }
+
+
 // Change help text
 void FXMenuCaption::setHelpText(const FXString& text){
   help=text;
@@ -221,13 +256,13 @@ void FXMenuCaption::setHelpText(const FXString& text){
 
 // Change text, and scan this text to replace accelerators
 void FXMenuCaption::setText(const FXString& text){
-  FXString str=text.extract(0,'\t','&');
-  if(label!=str){
+  FXString string=fxstripHotKey(text);
+  if(label!=string){
     remHotKey(hotkey);
-    hotkey=fxparsehotkey(text.text());
-    hotoff=fxfindhotkeyoffset(text.text());
+    hotkey=fxparseHotKey(text);
+    hotoff=fxfindHotKey(text);
     addHotKey(hotkey);
-    label=str;
+    label=string;
     recalc();
     update();
     }
@@ -255,6 +290,24 @@ void FXMenuCaption::setFont(FXFont* fnt){
   }
 
 
+// Set menu caption style
+void FXMenuCaption::setMenuStyle(FXuint style){
+  FXuint opts=(options&~MENU_MASK) | (style&MENU_MASK);
+  if(options!=opts){
+    options=opts;
+    recalc();
+    update();
+    }
+  }
+
+
+// Get menu caption style
+FXuint FXMenuCaption::getMenuStyle() const {
+  return (options&MENU_MASK);
+  }
+
+
+
 // Set text color
 void FXMenuCaption::setTextColor(FXColor clr){
   if(clr!=textColor){
@@ -338,8 +391,8 @@ void FXMenuCaption::load(FXStream& store){
 // Zap it
 FXMenuCaption::~FXMenuCaption(){
   remHotKey(hotkey);
-  font=(FXFont*)-1;
-  icon=(FXIcon*)-1;
+  font=(FXFont*)-1L;
+  icon=(FXIcon*)-1L;
   }
 
-
+}
