@@ -3,7 +3,7 @@
 *                    M e n u   S e p a r a t o r   W i d g e t                  *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXMenuSeparator.cpp,v 1.20 2002/01/18 22:43:01 jeroen Exp $              *
+* $Id: FXMenuSeparator.cpp,v 1.24 2003/10/23 04:19:45 fox Exp $                 *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -31,6 +31,7 @@
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXDCWindow.h"
 #include "FXMenuSeparator.h"
@@ -50,8 +51,12 @@
 #define LEADSPACE   22
 #define TRAILSPACE  16
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
+
 // Map
 FXDEFMAP(FXMenuSeparator) FXMenuSeparatorMap[]={
   FXMAPFUNC(SEL_PAINT,0,FXMenuSeparator::onPaint),
@@ -137,6 +142,5 @@ void FXMenuSeparator::load(FXStream& store){
   store >> shadowColor;
   }
 
-
-
+}
 
