@@ -3,7 +3,7 @@
 *                             O p t i o n   M e n u                             *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXOptionMenu.cpp,v 1.32.4.2 2003/06/20 19:02:07 fox Exp $                 *
+* $Id: FXOptionMenu.cpp,v 1.48 2003/10/23 04:19:45 fox Exp $                    *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -32,6 +32,7 @@
 #include "FXRectangle.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXDCWindow.h"
 #include "FXFont.h"
@@ -40,7 +41,7 @@
 #include "FXPopup.h"
 #include "FXButton.h"
 #include "FXMenuButton.h"
-#include "FXTooltip.h"
+#include "FXToolTip.h"
 #include "FXOptionMenu.h"
 
 /*
@@ -55,20 +56,22 @@
   - Right-click to cycle through options (as suggested by
     Stephane Ancelot <sancelot@crosswinds.net>) would be nice.
   - Hotkey's don't work in FXOption.
-  - Need to have "notify" in setCurrent.
-  - Need to send callback message from FXOptionMeny, and not from
-    FXOption.
   - Zecchini Mauro <mauro.zecchini@realtimesrl.com> donated code
     to add FXDataTarget get/set connectivity here also.
-
+  - Do we need API's other than those based on current item
+    as an index?
 */
 
 
 #define MENUGLYPH_WIDTH  10
 #define MENUGLYPH_HEIGHT 5
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
+
 // Map
 FXDEFMAP(FXOption) FXOptionMap[]={
   FXMAPFUNC(SEL_PAINT,0,FXOption::onPaint),
@@ -161,7 +164,7 @@ long FXOption::onPaint(FXObject*,FXSelector,void* ptr){
     drawDoubleRaisedRectangle(dc,ix,iy,MENUGLYPH_WIDTH,MENUGLYPH_HEIGHT);
     }
   if(!label.empty()){
-    dc.setTextFont(font);
+    dc.setFont(font);
     if(isEnabled()){
       dc.setForeground(textColor);
       drawLabel(dc,label,hotoff,tx,ty,tw,th);
@@ -197,9 +200,9 @@ long FXOption::onLeave(FXObject* sender,FXSelector sel,void* ptr){
 // Pressed left button; always unposts menu
 long FXOption::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
-    getParent()->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),this);
-    if(target) target->handle(this,MKUINT(message,SEL_COMMAND),ptr);
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONPRESS,message),ptr)) return 1;
+    getParent()->handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),this);
+    if(target) target->handle(this,FXSEL(SEL_COMMAND,message),ptr);
     return 1;
     }
   return 0;
@@ -209,10 +212,10 @@ long FXOption::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
 // Released left button; unpost menu if cursor has moved
 long FXOption::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
   if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONRELEASE,message),ptr)) return 1;
     if(((FXEvent*)ptr)->moved){
-      getParent()->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),this);
-      if(target) target->handle(this,MKUINT(message,SEL_COMMAND),ptr);
+      getParent()->handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),this);
+      if(target) target->handle(this,FXSEL(SEL_COMMAND,message),ptr);
       }
     return 1;
     }
@@ -224,7 +227,7 @@ long FXOption::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
 long FXOption::onKeyPress(FXObject*,FXSelector,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_KEYPRESS),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_KEYPRESS,message),ptr)) return 1;
     if(event->code==KEY_space || event->code==KEY_KP_Space){
       return 1;
       }
@@ -237,10 +240,10 @@ long FXOption::onKeyPress(FXObject*,FXSelector,void* ptr){
 long FXOption::onKeyRelease(FXObject*,FXSelector,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_KEYRELEASE),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_KEYRELEASE,message),ptr)) return 1;
     if(event->code==KEY_space || event->code==KEY_KP_Space){
-      getParent()->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),this);
-      if(target) target->handle(this,MKUINT(message,SEL_COMMAND),ptr);
+      getParent()->handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),this);
+      if(target) target->handle(this,FXSEL(SEL_COMMAND,message),ptr);
       return 1;
       }
     }
@@ -251,7 +254,7 @@ long FXOption::onKeyRelease(FXObject*,FXSelector,void* ptr){
 // Hot key combination pressed
 long FXOption::onHotKeyPress(FXObject*,FXSelector,void* ptr){
   flags&=~FLAG_TIP;
-  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+  handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr);
   FXTRACE((100,"FXOption::onHotKeyPress\n"));
   return 1;
   }
@@ -262,8 +265,8 @@ long FXOption::onHotKeyRelease(FXObject*,FXSelector,void* ptr){
   flags&=~FLAG_TIP;
   FXTRACE((100,"FXOption::onHotKeyRelease\n"));
   if(isEnabled()){
-    getParent()->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),this);
-    if(target) target->handle(this,MKUINT(message,SEL_COMMAND),ptr);
+    getParent()->handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),this);
+    if(target) target->handle(this,FXSEL(SEL_COMMAND,message),ptr);
     }
   return 1;
   }
@@ -428,12 +431,12 @@ long FXOptionMenu::onPaint(FXObject*,FXSelector,void* ptr){
 
   // Draw text
   if(!label.empty()){
-    dc.setTextFont(font);
+    dc.setFont(font);
     if(isEnabled()){
       dc.setForeground(textColor);
       drawLabel(dc,label,hotoff,tx,ty,tw,th);
       if(hasFocus()){
-        dc.drawFocusRectangle(border+2,border+2,width-2*border-4,height-2*border-4);
+        dc.drawFocusRectangle(border+1,border+1,width-2*border-2,height-2*border-2);
         }
       }
     else{
@@ -453,7 +456,7 @@ long FXOptionMenu::onKeyPress(FXObject*,FXSelector sel,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   flags&=~FLAG_TIP;
   if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_KEYPRESS),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_KEYPRESS,message),ptr)) return 1;
     if(pane && pane->shown() && pane->handle(pane,sel,ptr)) return 1;
     switch(event->code){
       case KEY_space:
@@ -469,17 +472,17 @@ long FXOptionMenu::onKeyPress(FXObject*,FXSelector sel,void* ptr){
 long FXOptionMenu::onKeyRelease(FXObject*,FXSelector sel,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_KEYRELEASE),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_KEYRELEASE,message),ptr)) return 1;
     if(pane && pane->shown() && pane->handle(pane,sel,ptr)) return 1;
     switch(event->code){
       case KEY_space:
       case KEY_KP_Space:
         if(pane){
           if(pane->shown()){
-            handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+            handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
             }
           else{
-            handle(this,MKUINT(ID_POST,SEL_COMMAND),NULL);
+            handle(this,FXSEL(SEL_COMMAND,ID_POST),NULL);
             }
           }
         return 1;
@@ -492,15 +495,15 @@ long FXOptionMenu::onKeyRelease(FXObject*,FXSelector sel,void* ptr){
 // Pressed left button
 long FXOptionMenu::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   flags&=~FLAG_TIP;
-  handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+  handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr);
   if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONPRESS,message),ptr)) return 1;
     if(pane){
       if(pane->shown()){
-        handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+        handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
         }
       else{
-        handle(this,MKUINT(ID_POST,SEL_COMMAND),NULL);
+        handle(this,FXSEL(SEL_COMMAND,ID_POST),NULL);
         }
       }
     return 1;
@@ -514,8 +517,8 @@ long FXOptionMenu::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
   FXEvent* ev=(FXEvent*)ptr;
   flags&=~FLAG_TIP;
   if(isEnabled()){
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
-    if(ev->moved && pane){ handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL); }
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONRELEASE,message),ptr)) return 1;
+    if(ev->moved && pane){ handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL); }
     return 1;
     }
   return 0;
@@ -564,7 +567,9 @@ long FXOptionMenu::onCmdUnpost(FXObject*,FXSelector,void* ptr){
     pane->popdown();
     if(grabbed()) ungrab();
     flags|=FLAG_UPDATE;
-    if(ptr) setCurrent((FXOption*)ptr);
+    if(ptr){
+      setCurrent((FXOption*)ptr,TRUE);
+      }
     }
   return 1;
   }
@@ -611,7 +616,7 @@ FXbool FXOptionMenu::contains(FXint parentx,FXint parenty) const {
 // Out of focus chain
 void FXOptionMenu::killFocus(){
   FXLabel::killFocus();
-  handle(current,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+  handle(current,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
   }
 
 
@@ -620,50 +625,42 @@ FXbool FXOptionMenu::canFocus() const { return 1; }
 
 
 // Set current selection
-void FXOptionMenu::setCurrent(FXOption *win){
-  if(win==NULL){ fxerror("%s::setCurrent: NULL window passed.\n",getClassName()); }
-  if(win->getParent()!=pane){ fxerror("%s::setCurrent: expected parent of window to be the pane.\n",getClassName()); }
+void FXOptionMenu::setCurrent(FXOption *win,FXbool notify){
   if(current!=win){
     current=win;
-    setText(current->getText());
-    setIcon(current->getIcon());
-    FXTRACE((200,"new option = %s\n",current->getText().text()));
+    if(win){
+      setText(current->getText());
+      setIcon(current->getIcon());
+      }
+    else{
+      setText(FXString::null);
+      setIcon(NULL);
+      }
+    if(notify && target){target->handle(this,FXSEL(SEL_COMMAND,message),(void*)(FXival)getCurrentNo());}
     }
   }
 
 
 // Set current option
-void FXOptionMenu::setCurrentNo(FXint no){
-  register FXint i=0;
-  if(pane){
-    // FXOption *win=(FXOption*)pane->childAtIndex(no);
-    FXOption *win=(FXOption*)pane->getFirst();
-    while(win && i!=no){
-      win=(FXOption*)win->getNext();
-      i++;
-      }
-    if(win) setCurrent(win);
-    }
+void FXOptionMenu::setCurrentNo(FXint no,FXbool notify){
+  setCurrent(pane ? (FXOption*)pane->childAtIndex(no) : NULL,notify);
   }
 
 
 // Get current option
 FXint FXOptionMenu::getCurrentNo() const {
-  register FXint i=0;
-  if(pane){
-    // return pane->indexOfChild(current);
-    FXOption *win=(FXOption*)pane->getFirst();
-    while(win && win!=current){
-      win=(FXOption*)win->getNext();
-      i++;
-      }
-    }
-  return i;
+  return pane ? pane->indexOfChild(current) : -1;
+  }
+
+
+// Get the number of options in this menu
+FXint FXOptionMenu::getNumOptions() const {
+  return pane ? pane->numChildren() : 0;
   }
 
 
 // Change popup
-void FXOptionMenu::setPopup(FXPopup *pup){
+void FXOptionMenu::setMenu(FXPopup *pup){
   FXOption *win;
   pane=pup;
   if(pane){
@@ -684,12 +681,12 @@ long FXOptionMenu::onQueryHelp(FXObject* sender,FXSelector,void*){
     if(current){
       FXString optionhelp=current->getHelpText();
       if(!optionhelp.empty()){
-        sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&optionhelp);
+        sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&optionhelp);
         return 1;
         }
       }
     if(!help.empty()){
-      sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&help);
+      sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&help);
       return 1;
       }
     }
@@ -704,12 +701,12 @@ long FXOptionMenu::onQueryTip(FXObject* sender,FXSelector,void*){
     if(current){
       FXString optiontip=current->getTipText();
       if(!optiontip.empty()){
-        sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&optiontip);
+        sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&optiontip);
         return 1;
         }
       }
     if(!tip.empty()){
-      sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&tip);
+      sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&tip);
       return 1;
       }
     }
@@ -741,7 +738,8 @@ void FXOptionMenu::load(FXStream& store){
 
 // Delete it
 FXOptionMenu::~FXOptionMenu(){
-  pane=(FXPopup*)-1;
-  current=(FXOption*)-1;
+  pane=(FXPopup*)-1L;
+  current=(FXOption*)-1L;
   }
 
+}
