@@ -3,7 +3,7 @@
 *                            P C X   I m a g e   O b j e c t                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2001,2002 by Janusz Ganczarski.   All Rights Reserved.          *
+* Copyright (C) 2001,2003 by Janusz Ganczarski.   All Rights Reserved.          *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,17 +19,19 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXPCXImage.cpp,v 1.3 2002/01/18 22:55:04 jeroen Exp $                    *
+* $Id: FXPCXImage.cpp,v 1.15 2003/10/23 04:19:45 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
+#include "FXMemoryStream.h"
 #include "FXString.h"
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXId.h"
 #include "FXDrawable.h"
@@ -43,9 +45,12 @@
   - Use corner color as transparency color, unless override.
 */
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 // Object implementation
 FXIMPLEMENT(FXPCXImage,FXImage,NULL,0)
 
@@ -53,12 +58,11 @@ FXIMPLEMENT(FXPCXImage,FXImage,NULL,0)
 
 // Initialize
 FXPCXImage::FXPCXImage(FXApp* a,const void *pix,FXuint opts,FXint w,FXint h):
-  FXImage(a,NULL,opts&~IMAGE_ALPHA,w,h){
+  FXImage(a,NULL,opts,w,h){
   if(pix){
     FXMemoryStream ms;
-    FXColor clearcolor;
-    ms.open((FXuchar*)pix,FXStreamLoad);
-    fxloadPCX(ms,data,clearcolor,width,height);
+    ms.open(FXStreamLoad,(FXuchar*)pix);
+    fxloadPCX(ms,data,width,height);
     options|=IMAGE_OWNED;
     ms.close();
     }
@@ -66,20 +70,18 @@ FXPCXImage::FXPCXImage(FXApp* a,const void *pix,FXuint opts,FXint w,FXint h):
 
 
 // Save pixel data only
-void FXPCXImage::savePixels(FXStream& store) const {
-  FXColor clearcolor=FXRGB(192,192,192);
-  FXASSERT(!(options&IMAGE_ALPHA));
-  fxsavePCX(store,data,clearcolor,width,height);
+FXbool FXPCXImage::savePixels(FXStream& store) const {
+  if(!fxsavePCX(store,data,width,height)) return FALSE;
+  return TRUE;
   }
 
 
 // Load pixel data only
-void FXPCXImage::loadPixels(FXStream& store){
-  FXColor clearcolor;
-  if(options&IMAGE_OWNED){FXFREE(&data);}
-  fxloadPCX(store,data,clearcolor,width,height);
-  options&=~IMAGE_ALPHA;
+FXbool FXPCXImage::loadPixels(FXStream& store){
+  if(options&IMAGE_OWNED){ FXFREE(&data); }
+  if(!fxloadPCX(store,data,width,height)) return FALSE;
   options|=IMAGE_OWNED;
+  return TRUE;
   }
 
 
@@ -87,5 +89,6 @@ void FXPCXImage::loadPixels(FXStream& store){
 FXPCXImage::~FXPCXImage(){
   }
 
+}
 
 
