@@ -3,7 +3,7 @@
 *                          P N G   I m a g e   O b j e c t                      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1999,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,17 +19,19 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXPNGImage.cpp,v 1.14 2002/01/18 22:43:01 jeroen Exp $                   *
+* $Id: FXPNGImage.cpp,v 1.26 2003/10/23 04:19:45 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
+#include "FXMemoryStream.h"
 #include "FXString.h"
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXPNGImage.h"
 
@@ -37,24 +39,26 @@
 
 /*
   Notes:
-  - FXPNGImage has an alpha channel
+  - FXPNGImage has an alpha channel.
 */
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 // Object implementation
 FXIMPLEMENT(FXPNGImage,FXImage,NULL,0)
 
 
 // Initialize
 FXPNGImage::FXPNGImage(FXApp* a,const void *pix,FXuint opts,FXint w,FXint h):
-  FXImage(a,NULL,opts|IMAGE_ALPHA,w,h){
+  FXImage(a,NULL,opts,w,h){
   if(pix){
     FXMemoryStream ms;
-    FXColor clearcolor;
-    ms.open((FXuchar*)pix,FXStreamLoad);
-    fxloadPNG(ms,data,clearcolor,width,height);
+    ms.open(FXStreamLoad,(FXuchar*)pix);
+    fxloadPNG(ms,data,width,height);
     options|=IMAGE_OWNED;
     ms.close();
     }
@@ -62,24 +66,24 @@ FXPNGImage::FXPNGImage(FXApp* a,const void *pix,FXuint opts,FXint w,FXint h):
 
 
 // Save the pixels only
-void FXPNGImage::savePixels(FXStream& store) const {
-  FXColor clearcolor=FXRGB(192,192,192);
-  FXASSERT(options&IMAGE_ALPHA);
-  fxsavePNG(store,data,clearcolor,width,height);
+FXbool FXPNGImage::savePixels(FXStream& store) const {
+  if(!fxsavePNG(store,data,width,height)) return FALSE;
+  return TRUE;
   }
 
 
 
 // Load pixels only
-void FXPNGImage::loadPixels(FXStream& store){
-  FXColor clearcolor;
-  if(options&IMAGE_OWNED){FXFREE(&data);}
-  fxloadPNG(store,data,clearcolor,width,height);
-  options|=IMAGE_ALPHA;
+FXbool FXPNGImage::loadPixels(FXStream& store){
+  if(options&IMAGE_OWNED){ FXFREE(&data); }
+  if(!fxloadPNG(store,data,width,height)) return FALSE;
   options|=IMAGE_OWNED;
+  return TRUE;
   }
 
 
 // Clean up
 FXPNGImage::~FXPNGImage(){
   }
+
+}
