@@ -3,7 +3,7 @@
 *                     P o p u p   W i n d o w   O b j e c t                     *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXPopup.cpp,v 1.49.4.2 2002/07/24 21:29:14 fox Exp $                      *
+* $Id: FXPopup.cpp,v 1.66 2003/10/23 04:19:45 fox Exp $                         *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -31,6 +31,7 @@
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXDCWindow.h"
 #include "FXPopup.h"
@@ -42,15 +43,24 @@
   - LAYOUT_FIX_xxx takes precedence over PACK_UNIFORM_xxx!!
   - Perhaps the grab owner could be equal to the owner?
   - Perhaps popup should resize when recalc() has been called!
-  - I believe that popup() should in fact enter a modal loop runModalWhileShown().
+  - Need to implement Mikael Aronssen's suggestion to make it easier to
+    run FXPopup in a modal loop.  This shouls also return a code, just
+    like FXDialogBox does.  Thus it becomes very straighforward to run
+    a popup menu just to get a single choice value back.
+  - Do not assume root window is at (0,0); multi-monitor machines may
+    have secondary monitor anywhere relative to primary display.
 */
 
 
 // Frame styles
 #define FRAME_MASK        (FRAME_SUNKEN|FRAME_RAISED|FRAME_THICK)
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
+
 
 // Map
 FXDEFMAP(FXPopup) FXPopupMap[]={
@@ -83,9 +93,14 @@ FXDEFMAP(FXPopup) FXPopupMap[]={
 FXIMPLEMENT(FXPopup,FXShell,FXPopupMap,ARRAYNUMBER(FXPopupMap))
 
 
+// Deserialization
+FXPopup::FXPopup():prevActive(NULL),nextActive(NULL){
+  }
+
+
 // Transient window used for popups
 FXPopup::FXPopup(FXWindow* owner,FXuint opts,FXint x,FXint y,FXint w,FXint h):
-  FXShell(owner,opts,x,y,w,h){
+  FXShell(owner,opts,x,y,w,h),prevActive(NULL),nextActive(NULL){
   defaultCursor=getApp()->getDefaultCursor(DEF_RARROW_CURSOR);
   dragCursor=getApp()->getDefaultCursor(DEF_RARROW_CURSOR);
   flags|=FLAG_ENABLED;
@@ -117,17 +132,24 @@ const char* FXPopup::GetClass() const { return "FXPopup"; }
 #endif
 
 
-// Popup can not get focus
+// Setting focus causes keyboard grab, because all keyboard
+// events need to be reported to the active popup.
 void FXPopup::setFocus(){
   FXShell::setFocus();
   //grabKeyboard();
   }
 
 
-// Popup can not get focus
+// Killing the focus should revert the keyboard grab to the
+// previously active popup, or release the keyboard grab
 void FXPopup::killFocus(){
   FXShell::killFocus();
-  //ungrabKeyboard();
+  //if(prevActive){
+  //  prevActive->setFocus();
+  //  }
+  //else{
+  //  ungrabKeyboard();
+  //  }
   }
 
 
@@ -446,10 +468,7 @@ long FXPopup::onFocusNext(FXObject*,FXSelector,void* ptr){
     child=getFocus()->getNext();
     while(child){
       if(child->shown()){
-        if(child->isEnabled() && child->canFocus()){
-          child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-          return 1;
-          }
+        if(child->handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr)) return 1;
         }
       child=child->getNext();
       }
@@ -457,10 +476,7 @@ long FXPopup::onFocusNext(FXObject*,FXSelector,void* ptr){
   child=getFirst();
   while(child){
     if(child->shown()){
-      if(child->isEnabled() && child->canFocus()){
-        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-        return 1;
-        }
+      if(child->handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr)) return 1;
       }
     child=child->getNext();
     }
@@ -475,10 +491,7 @@ long FXPopup::onFocusPrev(FXObject*,FXSelector,void* ptr){
     child=getFocus()->getPrev();
     while(child){
       if(child->shown()){
-        if(child->isEnabled() && child->canFocus()){
-          child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-          return 1;
-          }
+        if(child->handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr)) return 1;
         }
       child=child->getPrev();
       }
@@ -486,10 +499,7 @@ long FXPopup::onFocusPrev(FXObject*,FXSelector,void* ptr){
   child=getLast();
   while(child){
     if(child->shown()){
-      if(child->isEnabled() && child->canFocus()){
-        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-        return 1;
-        }
+      if(child->handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr)) return 1;
       }
     child=child->getPrev();
     }
@@ -558,7 +568,7 @@ long FXPopup::onMap(FXObject* sender,FXSelector sel,void* ptr){
 // Pressed button outside popup
 long FXPopup::onButtonPress(FXObject*,FXSelector,void*){
   FXTRACE((200,"%s::onButtonPress %p\n",getClassName(),this));
-  handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+  handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
   //popdown(0);
   return 1;
   }
@@ -568,7 +578,7 @@ long FXPopup::onButtonPress(FXObject*,FXSelector,void*){
 long FXPopup::onButtonRelease(FXObject*,FXSelector,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   FXTRACE((200,"%s::onButtonRelease %p\n",getClassName(),this));
-  if(event->moved){handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);}
+  if(event->moved){handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);}
   //popdown(0);
   return 1;
   }
@@ -577,7 +587,7 @@ long FXPopup::onButtonRelease(FXObject*,FXSelector,void* ptr){
 // The widget lost the grab for some reason; unpost the menu
 long FXPopup::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
   FXShell::onUngrabbed(sender,sel,ptr);
-  handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+  handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
   return 1;
   }
 
@@ -586,7 +596,7 @@ long FXPopup::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
 long FXPopup::onKeyPress(FXObject* sender,FXSelector sel,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   if(event->code==KEY_Escape || event->code==KEY_Cancel || event->code==KEY_Alt_L || event->code==KEY_Alt_R){
-    handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+    handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
     return 1;
     }
   return FXShell::onKeyPress(sender,sel,ptr);
@@ -597,7 +607,7 @@ long FXPopup::onKeyPress(FXObject* sender,FXSelector sel,void* ptr){
 long FXPopup::onKeyRelease(FXObject* sender,FXSelector sel,void* ptr){
   FXEvent* event=(FXEvent*)ptr;
   if(event->code==KEY_Escape || event->code==KEY_Cancel){
-    handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);
+    handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),NULL);
     return 1;
     }
   return FXShell::onKeyRelease(sender,sel,ptr);
@@ -609,7 +619,7 @@ long FXPopup::onKeyRelease(FXObject* sender,FXSelector sel,void* ptr){
 long FXPopup::onCmdUnpost(FXObject*,FXSelector,void* ptr){
   FXTRACE((150,"%s::onCmdUnpost %p\n",getClassName(),this));
   if(grabowner){
-    grabowner->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),ptr);
+    grabowner->handle(this,FXSEL(SEL_COMMAND,ID_UNPOST),ptr);
     }
   else{
     popdown();
@@ -619,18 +629,56 @@ long FXPopup::onCmdUnpost(FXObject*,FXSelector,void* ptr){
   }
 
 
+// Show popup and add to popup stack
+void FXPopup::show(){
+  if(!(flags&FLAG_SHOWN)){
+    FXShell::show();
+    prevActive=getApp()->popupWindow;
+    if(prevActive) prevActive->nextActive=this;
+    getApp()->popupWindow=this;
+    setFocus();
+    }
+  }
+
+
+// Hide popup and remove from popup stack
+void FXPopup::hide(){
+  if(flags&FLAG_SHOWN){
+    FXShell::hide();
+    if(getApp()->popupWindow==this) getApp()->popupWindow=prevActive;
+    if(prevActive) prevActive->nextActive=nextActive;
+    if(nextActive) nextActive->prevActive=prevActive;
+    nextActive=NULL;
+    prevActive=NULL;
+    killFocus();
+    }
+  // Focus back to popup under this one, iff this was top one
+  }
+
+
 // Popup the menu at some location
 void FXPopup::popup(FXWindow* grabto,FXint x,FXint y,FXint w,FXint h){
+#ifndef WIN32
+  FXint rx=getRoot()->getX();
+  FXint ry=getRoot()->getY();
   FXint rw=getRoot()->getWidth();
   FXint rh=getRoot()->getHeight();
+#else
+  RECT rect;
+  SystemParametersInfo(SPI_GETWORKAREA,sizeof(RECT),&rect,0);
+  FXint rx=rect.left;
+  FXint ry=rect.top;
+  FXint rw=rect.right-rect.left;
+  FXint rh=rect.bottom-rect.top;
+#endif
   FXTRACE((150,"%s::popup %p\n",getClassName(),this));
   grabowner=grabto;
   if((options&POPUP_SHRINKWRAP) || w<=1) w=getDefaultWidth();
   if((options&POPUP_SHRINKWRAP) || h<=1) h=getDefaultHeight();
   if(x+w>rw) x=rw-w;
   if(y+h>rh) y=rh-h;
-  if(x<0) x=0;
-  if(y<0) y=0;
+  if(x<rx) x=rx;
+  if(y<ry) y=ry;
   position(x,y,w,h);
   show();
   raise();
@@ -644,42 +692,40 @@ void FXPopup::popdown(){
   FXTRACE((150,"%s::popdown %p\n",getClassName(),this));
   if(!grabowner) ungrab();
   grabowner=NULL;
-  //if(getFocus()) getFocus()->killFocus();
   killFocus();
   hide();
   }
 
 
-// // Popup the menu and grab to the given owner
-// FXint FXPopup::popup(FXint x,FXint y,FXint w,FXint h){
-//   FXint rw,rh;
-//   create();
-//   if((options&POPUP_SHRINKWRAP) || w<=1) w=getDefaultWidth();
-//   if((options&POPUP_SHRINKWRAP) || h<=1) h=getDefaultHeight();
-//   rw=getRoot()->getWidth();
-//   rh=getRoot()->getHeight();
-//   if(x+w>rw) x=rw-w;
-//   if(y+h>rh) y=rh-h;
-//   if(x<0) x=0;
-//   if(y<0) y=0;
-//   position(x,y,w,h);
-//   show();
-//   raise();
-//   //setFocus();
-//   return getApp()->runPopup(this);
-//   }
-//
-//
-// // Pop down the menu
-// void FXPopup::popdown(FXint value){
-//   getApp()->stopModal(this,value);
-//   //killFocus();
-//   hide();
-//   }
+/*
+// Popup the menu and grab to the given owner
+FXint FXPopup::popup(FXint x,FXint y,FXint w,FXint h){
+  FXint rw,rh;
+  create();
+  if((options&POPUP_SHRINKWRAP) || w<=1) w=getDefaultWidth();
+  if((options&POPUP_SHRINKWRAP) || h<=1) h=getDefaultHeight();
+  rw=getRoot()->getWidth();
+  rh=getRoot()->getHeight();
+  if(x+w>rw) x=rw-w;
+  if(y+h>rh) y=rh-h;
+  if(x<0) x=0;
+  if(y<0) y=0;
+  position(x,y,w,h);
+  show();
+  raise();
+  return getApp()->runPopup(this);
+  }
 
 
+// Pop down the menu
+void FXPopup::popdown(FXint value){
+  getApp()->stopModal(this,value);
+  hide();
+  }
+*/
+
 // Close popup
-long FXPopup::onCmdChoice(FXObject*,FXSelector sel,void*){
+long FXPopup::onCmdChoice(FXObject*,FXSelector,void*){
   //popdown(SELID(sel)-ID_CHOICE);
   return 1;
   }
@@ -770,7 +816,14 @@ FXuint FXPopup::getFrameStyle() const {
   }
 
 
-// Zap
+// Unlink popup from active popup stack
 FXPopup::~FXPopup(){
-  grabowner=(FXWindow*)-1;
+  if(getApp()->popupWindow==this) getApp()->popupWindow=prevActive;
+  if(prevActive) prevActive->nextActive=nextActive;
+  if(nextActive) nextActive->prevActive=prevActive;
+  prevActive=(FXPopup*)-1L;
+  nextActive=(FXPopup*)-1L;
+  grabowner=(FXWindow*)-1L;
   }
+
+}
