@@ -3,7 +3,7 @@
 *                        P r i n t   J o b   D i a l o g                        *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1999,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXPrintDialog.cpp,v 1.31 2002/01/18 22:43:01 jeroen Exp $                *
+* $Id: FXPrintDialog.cpp,v 1.39 2003/10/23 04:19:45 fox Exp $                   *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -32,6 +32,7 @@
 #include "FXRectangle.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXDCPrint.h"
 #include "FXGIFIcon.h"
@@ -61,6 +62,7 @@
 #include <cups/cups.h>
 #include <cups/ppd.h>
 #endif
+#include "icons.h"
 
 /*
   Notes:
@@ -172,43 +174,12 @@
 
 */
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
 
-/* Generated by reswrap from file landscape.gif */
-const unsigned char landscape[]={
-  0x47,0x49,0x46,0x38,0x37,0x61,0x20,0x00,0x20,0x00,0xf1,0x00,0x00,0xb2,0xc0,0xdc,
-  0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0xff,0x2c,0x00,0x00,0x00,0x00,0x20,0x00,
-  0x20,0x00,0x00,0x02,0x7c,0x84,0x8f,0xa9,0xcb,0xed,0x0f,0x63,0x0a,0xb4,0xda,0x8b,
-  0xed,0x0a,0xa2,0xfb,0x0f,0x86,0x1d,0xa5,0x70,0xe2,0x29,0x72,0xc1,0x84,0xb6,0x9f,
-  0xb9,0x1e,0xa6,0xdb,0xce,0x31,0x30,0xd3,0xc2,0x10,0x62,0x46,0xee,0x1a,0xf0,0x6a,
-  0x3f,0xdd,0x4e,0xe8,0xb2,0xe8,0x84,0xc8,0x9a,0x80,0xb2,0x3c,0x26,0x9f,0x40,0xd1,
-  0xb0,0x89,0x52,0x55,0x41,0x43,0xa9,0x13,0xda,0x62,0x32,0xa7,0xe0,0x53,0xd7,0x9b,
-  0xa5,0xa2,0xce,0x47,0x36,0x48,0x6b,0xc6,0x8a,0xdd,0x1e,0xb8,0x51,0x67,0xbf,0x93,
-  0xb7,0xfa,0xb7,0xba,0xff,0xc5,0x07,0x38,0x42,0x95,0x61,0x78,0x78,0xf1,0x54,0x24,
-  0x01,0xa1,0x88,0x83,0x08,0x69,0xc8,0x38,0x49,0x59,0x69,0x79,0x89,0x99,0x09,0x51,
-  0x00,0x00,0x3b
-  };
-
-/* Generated by reswrap from file portrait.gif */
-const unsigned char portrait[]={
-  0x47,0x49,0x46,0x38,0x37,0x61,0x20,0x00,0x20,0x00,0xf1,0x00,0x00,0xb2,0xc0,0xdc,
-  0x00,0x00,0x00,0xff,0xff,0xff,0x00,0x00,0xff,0x2c,0x00,0x00,0x00,0x00,0x20,0x00,
-  0x20,0x00,0x00,0x02,0x90,0x84,0x11,0xa9,0xcb,0x79,0x0f,0x4f,0x10,0xb4,0xda,0xeb,
-  0xa2,0x9e,0xb7,0xd7,0x19,0x68,0x11,0xe7,0x75,0x5c,0x28,0x4a,0xa5,0x47,0xa2,0x29,
-  0xb9,0x7e,0x4c,0x8a,0xc4,0xb1,0x22,0xc2,0x36,0x26,0xb8,0x90,0xbe,0xfb,0xf4,0x72,
-  0xbb,0x01,0x6b,0xb8,0xb1,0x0d,0x8c,0x26,0xe4,0x28,0xb6,0x64,0xf2,0x7c,0x0f,0x60,
-  0x25,0x2a,0xb5,0x80,0x88,0x25,0xe3,0xf2,0x48,0x55,0x79,0x98,0xdf,0x66,0xd8,0x60,
-  0x95,0x96,0xa7,0xdc,0x0b,0x76,0xad,0x75,0xfe,0x3a,0x59,0xb8,0xf0,0x5c,0xb3,0x64,
-  0x05,0x51,0x73,0x9b,0xbf,0xf6,0xc6,0x96,0x14,0xe4,0x47,0x58,0x18,0x87,0x67,0x15,
-  0xb4,0x75,0x88,0x48,0xd1,0xf8,0xf4,0x78,0xf7,0xf7,0x18,0x39,0x37,0x09,0x29,0x57,
-  0x95,0xa9,0xa9,0xd8,0xd9,0xb3,0x29,0xd1,0x40,0x5a,0x2a,0xca,0xd9,0x89,0x87,0x56,
-  0xca,0xba,0xf0,0x50,0x00,0x00,0x3b
-  };
-
-
-/*******************************************************************************/
 
 // Map
 FXDEFMAP(FXPrintDialog) FXPrintDialogMap[]={
@@ -255,9 +226,10 @@ FXIMPLEMENT(FXPrintDialog,FXDialogBox,FXPrintDialogMap,ARRAYNUMBER(FXPrintDialog
 
 
 // Separator item
-FXPrintDialog::FXPrintDialog(FXWindow* owner,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h):
-  FXDialogBox(owner,name,opts|DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,x,y,w,h,0,0,0,0,4,4){
-  FXchar key[20],print[100]; const FXchar *paper;
+FXPrintDialog::FXPrintDialog(FXWindow* owner,const FXString& caption,FXuint opts,FXint x,FXint y,FXint w,FXint h):
+  FXDialogBox(owner,caption,opts|DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,x,y,w,h,0,0,0,0,4,4){
+  FXchar key[20],name[100];
+  //const FXchar *paper;
 
   FXVerticalFrame *contents=new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,10,10,10,10, 10,10);
 
@@ -358,11 +330,8 @@ FXPrintDialog::FXPrintDialog(FXWindow* owner,const FXString& name,FXuint opts,FX
   // List paper sizes
   for(int i=0; ; i++){
     sprintf(key,"%d",i);
-    paper=getApp()->reg().readStringEntry("PAPER",key,NULL);
-    if(!paper) break;
-    FXTRACE((100,"Paper %d=\"%s\"\n",i,paper));
-    sscanf(paper,"[%[^]]] %*f %*f %*f %*f %*f %*f",print);
-    media->appendItem(print);
+    if(getApp()->reg().readFormatEntry("PAPER",key,"[%[^]]] %*f %*f %*f %*f %*f %*f",name)!=1) break;
+    media->appendItem(name);
     }
 
   // These things are always reset
@@ -387,10 +356,10 @@ FXPrintDialog::FXPrintDialog(FXWindow* owner,const FXString& name,FXuint opts,FX
 
   // Obtain size
   sprintf(key,"%d",printer.mediasize);
-  paper=getApp()->reg().readStringEntry("PAPER",key,"[US Letter] 612 792 72 72 72 72");
 
   // Parse size
-  sscanf(paper,"[%[^]]] %lf %lf %lf %lf %lf %lf",print,&printer.mediawidth,&printer.mediaheight,&printer.leftmargin,&printer.rightmargin,&printer.topmargin,&printer.bottommargin);
+  getApp()->reg().readFormatEntry("PAPER",key,"[%[^]]] %lf %lf %lf %lf %lf %lf",name,&printer.mediawidth,&printer.mediaheight,&printer.leftmargin,&printer.rightmargin,&printer.topmargin,&printer.bottommargin);
+  FXTRACE((100,"Media=\"%s\" w=%g h=%g lm=%g rm=%g tm=%g bm=%g\n",name,printer.mediawidth,printer.mediaheight,printer.leftmargin,printer.rightmargin,printer.topmargin,printer.bottommargin));
 
   // Initial flags
   printer.flags=0;
@@ -605,10 +574,7 @@ long FXPrintDialog::onCmdToPrinter(FXObject*,FXSelector,void*){
 
 // Update send to printer
 long FXPrintDialog::onUpdToPrinter(FXObject* sender,FXSelector,void*){
-  if(printer.flags&PRINT_DEST_FILE)
-    sender->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
+  sender->handle(this,(printer.flags&PRINT_DEST_FILE)?FXSEL(SEL_COMMAND,ID_UNCHECK):FXSEL(SEL_COMMAND,ID_CHECK),NULL);
   return 1;
   }
 
@@ -625,10 +591,7 @@ long FXPrintDialog::onCmdToFile(FXObject*,FXSelector,void*){
 
 // Update send to file
 long FXPrintDialog::onUpdToFile(FXObject* sender,FXSelector,void*){
-  if(printer.flags&PRINT_DEST_FILE)
-    sender->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
+  sender->handle(this,(printer.flags&PRINT_DEST_FILE)?FXSEL(SEL_COMMAND,ID_CHECK):FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
   return 1;
   }
 
@@ -664,10 +627,7 @@ long FXPrintDialog::onCmdBrowse(FXObject*,FXSelector,void*){
 
 // Update browse output file
 long FXPrintDialog::onUpdBrowse(FXObject* sender,FXSelector,void*){
-  if(printer.flags&PRINT_DEST_FILE)
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+  sender->handle(this,(printer.flags&PRINT_DEST_FILE)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
   return 1;
   }
 
@@ -687,10 +647,7 @@ long FXPrintDialog::onCmdProps(FXObject*,FXSelector,void*){   // FIXME this need
 
 // Update printer properties panel
 long FXPrintDialog::onUpdProps(FXObject* sender,FXSelector,void*){
-  if(printer.flags&PRINT_DEST_FILE)
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
+  sender->handle(this,(printer.flags&PRINT_DEST_FILE)?FXSEL(SEL_COMMAND,ID_DISABLE):FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
   return 1;
   }
 
@@ -712,13 +669,13 @@ long FXPrintDialog::onUpdFileName(FXObject* sender,FXSelector,void*){
   FXString name;
   if(printer.flags&PRINT_DEST_FILE){
     name=printer.name;
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
     }
   else{
     name=getApp()->reg().readStringEntry("PRINTER","file","output.ps");
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
     }
-  sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&name);
+  sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&name);
   return 1;
   }
 
@@ -759,13 +716,13 @@ long FXPrintDialog::onUpdPrinterName(FXObject* sender,FXSelector,void*){
   FXString name;
   if(printer.flags&PRINT_DEST_FILE){
     name=getApp()->reg().readStringEntry("PRINTER","printer","");
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
     }
   else{
     name=printer.name;
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
     }
-  sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&name);
+  sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),(void*)&name);
   return 1;
   }
 
@@ -780,10 +737,7 @@ long FXPrintDialog::onCmdPortrait(FXObject*,FXSelector,void*){
 
 // Update portrait
 long FXPrintDialog::onUpdPortrait(FXObject* sender,FXSelector,void*){
-  if(printer.flags&PRINT_LANDSCAPE)
-    sender->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
+  sender->handle(this,(printer.flags&PRINT_LANDSCAPE)?FXSEL(SEL_COMMAND,ID_UNCHECK):FXSEL(SEL_COMMAND,ID_CHECK),NULL);
   return 1;
   }
 
@@ -798,17 +752,14 @@ long FXPrintDialog::onCmdLandscape(FXObject*,FXSelector,void*){
 
 // Update landscape
 long FXPrintDialog::onUpdLandscape(FXObject* sender,FXSelector,void*){
-  if(printer.flags&PRINT_LANDSCAPE)
-    sender->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
+  sender->handle(this,(printer.flags&PRINT_LANDSCAPE)?FXSEL(SEL_COMMAND,ID_CHECK):FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
   return 1;
   }
 
 
 // Pages
 long FXPrintDialog::onCmdPages(FXObject*,FXSelector sel,void*){
-  FXuint sid=SELID(sel);
+  FXuint sid=FXSELID(sel);
   switch(sid){
     case ID_PAGES_ALL:
       printer.flags&=~(PRINT_PAGES_EVEN|PRINT_PAGES_ODD|PRINT_PAGES_RANGE);
@@ -845,7 +796,7 @@ long FXPrintDialog::onCmdPages(FXObject*,FXSelector sel,void*){
 // Update Pages
 long FXPrintDialog::onUpdPages(FXObject* sender,FXSelector sel,void*){
   FXuint ch=ID_UNCHECK;
-  FXuint sid=SELID(sel);
+  FXuint sid=FXSELID(sel);
   switch(sid){
     case ID_PAGES_ALL:
       if(!(printer.flags&(PRINT_PAGES_EVEN|PRINT_PAGES_ODD|PRINT_PAGES_RANGE))) ch=ID_CHECK;
@@ -860,7 +811,7 @@ long FXPrintDialog::onUpdPages(FXObject* sender,FXSelector sel,void*){
       if(printer.flags&PRINT_PAGES_RANGE) ch=ID_CHECK;
       break;
     }
-  sender->handle(this,MKUINT(ch,SEL_COMMAND),NULL);
+  sender->handle(this,FXSEL(SEL_COMMAND,ch),NULL);
   return 1;
   }
 
@@ -875,10 +826,7 @@ long FXPrintDialog::onCmdColor(FXObject*,FXSelector,void*){
 
 // Update color
 long FXPrintDialog::onUpdColor(FXObject* sender,FXSelector,void*){
-  if(printer.flags&PRINT_COLOR)
-    sender->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
+  sender->handle(this,(printer.flags&PRINT_COLOR)?FXSEL(SEL_COMMAND,ID_CHECK):FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
   return 1;
   }
 
@@ -893,24 +841,21 @@ long FXPrintDialog::onCmdGray(FXObject*,FXSelector,void*){
 
 // Update gray
 long FXPrintDialog::onUpdGray(FXObject* sender,FXSelector,void*){
-  if(printer.flags&PRINT_COLOR)
-    sender->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
+  sender->handle(this,(printer.flags&PRINT_COLOR)?FXSEL(SEL_COMMAND,ID_UNCHECK):FXSEL(SEL_COMMAND,ID_CHECK),NULL);
   return 1;
   }
 
 
 // Number of copies
 long FXPrintDialog::onCmdNumCopies(FXObject* sender,FXSelector,void*){
-  sender->handle(this,MKUINT(ID_GETINTVALUE,SEL_COMMAND),(void*)&printer.numcopies);
+  sender->handle(this,FXSEL(SEL_COMMAND,ID_GETINTVALUE),(void*)&printer.numcopies);
   return 1;
   }
 
 
 // Update number of copies
 long FXPrintDialog::onUpdNumCopies(FXObject* sender,FXSelector,void*){
-  sender->handle(this,MKUINT(ID_SETINTVALUE,SEL_COMMAND),(void*)&printer.numcopies);
+  sender->handle(this,FXSEL(SEL_COMMAND,ID_SETINTVALUE),(void*)&printer.numcopies);
   return 1;
   }
 
@@ -918,7 +863,7 @@ long FXPrintDialog::onUpdNumCopies(FXObject* sender,FXSelector,void*){
 // First page
 long FXPrintDialog::onCmdFirstPage(FXObject* sender,FXSelector,void*){
   FXint pg;
-  sender->handle(this,MKUINT(ID_GETINTVALUE,SEL_COMMAND),(void*)&pg);
+  sender->handle(this,FXSEL(SEL_COMMAND,ID_GETINTVALUE),(void*)&pg);
   FXASSERT(pg>0);
   if((FXuint)pg<printer.firstpage) pg=printer.firstpage;
   if((FXuint)pg>printer.lastpage) pg=printer.lastpage;
@@ -929,11 +874,8 @@ long FXPrintDialog::onCmdFirstPage(FXObject* sender,FXSelector,void*){
 
 // Update first page
 long FXPrintDialog::onUpdFirstPage(FXObject* sender,FXSelector,void*){
-  sender->handle(this,MKUINT(ID_SETINTVALUE,SEL_COMMAND),(void*)&printer.frompage);
-  if(printer.flags&PRINT_PAGES_RANGE)
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+  sender->handle(this,FXSEL(SEL_COMMAND,ID_SETINTVALUE),(void*)&printer.frompage);
+  sender->handle(this,(printer.flags&PRINT_PAGES_RANGE)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
   return 1;
   }
 
@@ -941,7 +883,7 @@ long FXPrintDialog::onUpdFirstPage(FXObject* sender,FXSelector,void*){
 // Last page
 long FXPrintDialog::onCmdLastPage(FXObject* sender,FXSelector,void*){
   FXint pg;
-  sender->handle(this,MKUINT(ID_GETINTVALUE,SEL_COMMAND),(void*)&pg);
+  sender->handle(this,FXSEL(SEL_COMMAND,ID_GETINTVALUE),(void*)&pg);
   FXASSERT(pg>0);
   if((FXuint)pg<printer.firstpage) pg=printer.firstpage;
   if((FXuint)pg>printer.lastpage) pg=printer.lastpage;
@@ -952,11 +894,8 @@ long FXPrintDialog::onCmdLastPage(FXObject* sender,FXSelector,void*){
 
 // Update last page
 long FXPrintDialog::onUpdLastPage(FXObject* sender,FXSelector,void*){
-  sender->handle(this,MKUINT(ID_SETINTVALUE,SEL_COMMAND),(void*)&printer.topage);
-  if(printer.flags&PRINT_PAGES_RANGE)
-    sender->handle(this,MKUINT(ID_ENABLE,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_DISABLE,SEL_COMMAND),NULL);
+  sender->handle(this,FXSEL(SEL_COMMAND,ID_SETINTVALUE),(void*)&printer.topage);
+  sender->handle(this,(printer.flags&PRINT_PAGES_RANGE)?FXSEL(SEL_COMMAND,ID_ENABLE):FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
   return 1;
   }
 
@@ -970,10 +909,7 @@ long FXPrintDialog::onCmdCollateNormal(FXObject*,FXSelector,void*){
 
 // Update Collate Normal
 long FXPrintDialog::onUpdCollateNormal(FXObject* sender,FXSelector,void*){
-  if(printer.flags&PRINT_COLLATE_REVERSED)
-    sender->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
+  sender->handle(this,(printer.flags&PRINT_COLLATE_REVERSED)?FXSEL(SEL_COMMAND,ID_UNCHECK):FXSEL(SEL_COMMAND,ID_CHECK),NULL);
   return 1;
   }
 
@@ -987,10 +923,7 @@ long FXPrintDialog::onCmdCollateReversed(FXObject*,FXSelector,void*){
 
 // Update Collate Reversed
 long FXPrintDialog::onUpdCollateReversed(FXObject* sender,FXSelector,void*){
-  if(printer.flags&PRINT_COLLATE_REVERSED)
-    sender->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
-  else
-    sender->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
+  sender->handle(this,(printer.flags&PRINT_COLLATE_REVERSED)?FXSEL(SEL_COMMAND,ID_CHECK):FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
   return 1;
   }
 
@@ -998,12 +931,10 @@ long FXPrintDialog::onUpdCollateReversed(FXObject* sender,FXSelector,void*){
 // Standard paper dimensions
 long FXPrintDialog::onCmdMedia(FXObject*,FXSelector,void*){
   FXchar key[20],name[100];
-  const FXchar *paper;
   printer.mediasize=media->getCurrentItem();
   getApp()->reg().writeIntEntry("PRINTER","media",printer.mediasize);
   sprintf(key,"%d",printer.mediasize);
-  paper=getApp()->reg().readStringEntry("PAPER",key,"[US Letter] 612 792 72 72 72 72");
-  sscanf(paper,"[%[^]]] %lf %lf %lf %lf %lf %lf",name,&printer.mediawidth,&printer.mediaheight,&printer.leftmargin,&printer.rightmargin,&printer.topmargin,&printer.bottommargin);
+  getApp()->reg().readFormatEntry("PAPER",key,"[%[^]]] %lf %lf %lf %lf %lf %lf",name,&printer.mediawidth,&printer.mediaheight,&printer.leftmargin,&printer.rightmargin,&printer.topmargin,&printer.bottommargin);
   FXTRACE((100,"Media=\"%s\" w=%g h=%g lm=%g rm=%g tm=%g bm=%g\n",name,printer.mediawidth,printer.mediaheight,printer.leftmargin,printer.rightmargin,printer.topmargin,printer.bottommargin));
   return 1;
   }
@@ -1011,7 +942,7 @@ long FXPrintDialog::onCmdMedia(FXObject*,FXSelector,void*){
 
 // Update Standard paper dimensions
 long FXPrintDialog::onUpdMedia(FXObject*,FXSelector,void*){
-  if(printer.mediasize<=media->getNumItems()) media->setCurrentItem(printer.mediasize);
+  if((FXint)printer.mediasize<=media->getNumItems()) media->setCurrentItem(printer.mediasize);
   return 1;
   }
 
@@ -1044,8 +975,10 @@ void FXPrintDialog::load(FXStream& store){
 FXPrintDialog::~FXPrintDialog(){
   delete landscapeIcon;
   delete portraitIcon;
-  landscapeIcon=(FXIcon*)-1;
-  portraitIcon=(FXIcon*)-1;
+  landscapeIcon=(FXIcon*)-1L;
+  portraitIcon=(FXIcon*)-1L;
   }
 
+}
+
 
