@@ -3,7 +3,7 @@
 *                          R e c t a n g l e    C l a s s                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1994,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1994,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXRectangle.cpp,v 1.6 2002/01/18 22:43:02 jeroen Exp $                   *
+* $Id: FXRectangle.cpp,v 1.10 2003/04/05 14:22:37 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -30,6 +30,11 @@
 #include "FXPoint.h"
 #include "FXRectangle.h"
 
+using namespace FX;
+
+/*******************************************************************************/
+
+namespace FX {
 
 // Fast inlines
 static inline FXshort _max(FXshort a,FXshort b){ return a>b?a:b; }
@@ -141,3 +146,5 @@ FXStream& operator>>(FXStream& store,FXRectangle& r){
   store >> r.x >> r.y >> r.w >> r.h;
   return store;
   }
+
+}
