@@ -3,7 +3,7 @@
 *                      C l i p p i n g   R e g i o n                            *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2000,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXRegion.cpp,v 1.10 2002/01/18 22:43:02 jeroen Exp $                      *
+* $Id: FXRegion.cpp,v 1.16 2003/04/05 14:22:37 fox Exp $                        *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -30,6 +30,11 @@
 #include "FXRectangle.h"
 #include "FXRegion.h"
 
+using namespace FX;
+
+/*******************************************************************************/
+
+namespace FX {
 
 // Construct new empty region
 FXRegion::FXRegion(){
@@ -41,7 +46,7 @@ FXRegion::FXRegion(){
   }
 
 
-// Construct new region set to given rectangle
+// Construct rectangle region
 FXRegion::FXRegion(FXint x,FXint y,FXint w,FXint h){
 #ifndef WIN32
   XRectangle r;
@@ -54,6 +59,22 @@ FXRegion::FXRegion(FXint x,FXint y,FXint w,FXint h){
   }
 
 
+// Construct polygon region
+FXRegion::FXRegion(const FXPoint* points,FXuint npoints,FXbool winding){
+#ifndef WIN32
+  region=XPolygonRegion((XPoint*)points,npoints,winding?WindingRule:EvenOddRule);
+#else
+  register FXuint i;
+  POINT pts[1024];
+  for(i=0; i<npoints; i++){
+    pts[i].x=points[i].x;
+    pts[i].y=points[i].y;
+    }
+  region=(void*)CreatePolygonRgn(pts,npoints,winding?WINDING:ALTERNATE);
+#endif
+  }
+
+
 // Construct new region copied from region r
 FXRegion::FXRegion(const FXRegion& r){
 #ifndef WIN32
@@ -275,3 +296,5 @@ FXRegion::~FXRegion(){
 #endif
   }
 
+}
+
