@@ -3,7 +3,7 @@
 *                           R e g i s t r y   C l a s s                         *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXRegistry.cpp,v 1.32.4.1 2003/05/16 10:54:30 fox Exp $                   *
+* $Id: FXRegistry.cpp,v 1.37 2003/05/12 09:00:01 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -107,8 +107,12 @@
 
 #define DESKTOP        "Desktop"
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
+
 // Object implementation
 FXIMPLEMENT(FXRegistry,FXSettings,NULL,0)
 
@@ -543,13 +547,14 @@ FXbool FXRegistry::writeToRegistry(void* hRootKey){
 // Write to registry group
 FXbool FXRegistry::writeToRegistryGroup(void* org,const char* groupname){
   FXchar section[MAXNAME];
-  DWORD sectionsize,sectionindex,disp,s,e;
+  DWORD sectionsize,sectionindex,disp;
   HKEY groupkey,sectionkey;
+  FXint s,e;
   FILETIME writetime;
   FXStringDict *group;
   if(RegCreateKeyEx((HKEY)org,groupname,0,REG_NONE,REG_OPTION_NON_VOLATILE,KEY_WRITE|KEY_READ,NULL,&groupkey,&disp)==ERROR_SUCCESS){
 
-/*
+/*    
     // First, purge all existing sections
     sectionindex=0;
     sectionsize=MAXNAME;
@@ -558,7 +563,7 @@ FXbool FXRegistry::writeToRegistryGroup(void* org,const char* groupname){
       sectionsize=MAXNAME;
       sectionindex++;
       }
-*/
+*/      
     // First, purge all existing sections
     while(1){
       sectionindex=0;
@@ -600,3 +605,4 @@ x:    s=next(s);
 
 #endif
 
+}
