@@ -3,7 +3,7 @@
 *                       T e x t   S e a r c h   D i a l o g                     *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2000,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXSearchDialog.cpp,v 1.21 2002/01/18 22:43:04 jeroen Exp $               *
+* $Id: FXSearchDialog.cpp,v 1.25 2003/10/23 04:19:45 fox Exp $                  *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -31,6 +31,7 @@
 #include "FXRectangle.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXFont.h"
 #include "FXFrame.h"
@@ -51,9 +52,11 @@
     hidden.
 */
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
 
 
 // Object implementation
@@ -75,4 +78,5 @@ FXSearchDialog::FXSearchDialog(FXWindow* owner,const FXString& caption,FXIcon* i
 FXSearchDialog::~FXSearchDialog(){
   }
 
+}
 
