@@ -3,7 +3,7 @@
 *                           S e t t i n g s   C l a s s                         *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXSettings.cpp,v 1.16.4.1 2002/11/13 15:14:07 fox Exp $                      *
+* $Id: FXSettings.cpp,v 1.26 2003/07/28 14:50:34 fox Exp $                      *
 ********************************************************************************/
 #ifdef HAVE_VSSCANF
 #ifndef _GNU_SOURCE
@@ -50,15 +50,20 @@
 
   - FXSectionDict should go; FXSettings should simply derive from FXDict.
 
+  - Escape sequences now allow octal (\377) as well as hex (\xff) codes.
+
 */
 
 #define MAXBUFFER 2000
 #define MAXNAME   200
 #define MAXVALUE  2000
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 // Object implementation
 FXIMPLEMENT(FXSettings,FXDict,NULL,0)
 
@@ -188,7 +193,7 @@ next: lineno++;
 FXbool FXSettings::parseValue(FXchar* value,const FXchar* buffer){
   register const FXchar *ptr=buffer;
   register FXchar *out=value;
-  unsigned int v1,v2,h,l;
+  register FXuint v,c;
 
   // Was quoted string; copy verbatim
   if(*ptr=='"'){
@@ -225,16 +230,37 @@ FXbool FXSettings::parseValue(FXchar* value,const FXchar* buffer){
             case '"':
               *out++='"';
               break;
-            case 'x':
-              ptr++;
-              v1=*ptr++;
-              if(!v1) return FALSE;
-              v2=*ptr;
-              if(!v2) return FALSE;
-              h=v1<='9'?v1-'0':toupper(v1)-'A'+10;
-              l=v2<='9'?v2-'0':toupper(v2)-'A'+10;
-              *out++=(h<<4)+l;
+            case '\'':
+              *out++='\'';
               break;
+            case 'x':
+              v='x';
+              if(isxdigit((FXuchar)*(ptr+1))){
+                c=*++ptr;
+                v=('a'<=c)?(c-'a'+10):('A'<=c)?(c-'A'+10):(c-'0');
+                if(isxdigit((FXuchar)*(ptr+1))){
+                  c=*++ptr;
+                  v=(v<<4)+(('a'<=c)?(c-'a'+10):('A'<=c)?(c-'A'+10):(c-'0'));
+                  }
+                }
+              *out++=v;
+              break;
+            case '0':
+            case '1':
+            case '2':
+            case '3':
+            case '4':
+            case '5':
+            case '6':
+            case '7':
+              v=*ptr-'0';
+              if('0'<=*(ptr+1) && *(ptr+1)<='7'){
+                v=v*8+*++ptr-'0';
+                if('0'<=*(ptr+1) && *(ptr+1)<='7'){
+                  v=v*8+*++ptr-'0';
+                  }
+                }
+              *out++=v;
             default:
               *out++=*ptr;
               break;
@@ -338,7 +364,6 @@ FXbool FXSettings::unparseFile(const FXString& filename){
 
 // Unparse value by quoting strings; return TRUE if quote needed
 FXbool FXSettings::unparseValue(FXchar* buffer,const FXchar* value){
-  const FXchar hex[]="0123456789ABCDEF";
   register FXchar *ptr=buffer;
   register FXbool mustquote=FALSE;
   register FXuint v;
@@ -390,17 +415,22 @@ FXbool FXSettings::unparseValue(FXchar* buffer,const FXchar* value){
         *ptr++='"';
         mustquote=TRUE;
         break;
+      case '\'':
+        *ptr++='\\';
+        *ptr++='\'';
+        mustquote=TRUE;
+        break;
       case ' ':
         *ptr++=' ';
         mustquote=TRUE;
         break;
       default:
-        v=*value;
+        v=(FXuchar)*value;
         if(v<0x20 || 0x7f<v){
           *ptr++='\\';
           *ptr++='x';
-          *ptr++=hex[((v>>4)&15)];
-          *ptr++=hex[v&15];
+          *ptr++=FXString::HEX[v>>4];
+          *ptr++=FXString::HEX[v&15];
           mustquote=TRUE;
           }
         else{
@@ -424,9 +454,9 @@ extern "C" int vsscanf(const char* str, const char* format, va_list arg_ptr);
 
 // Read a formatted registry entry
 FXint FXSettings::readFormatEntry(const FXchar *section,const FXchar *key,const FXchar *fmt,...){
-  if(!section){ fxerror("FXSettings::readFormatEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::readFormatEntry: NULL key argument.\n"); }
-  if(!fmt){ fxerror("FXSettings::readFormatEntry: NULL fmt argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::readFormatEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::readFormatEntry: bad key argument.\n"); }
+  if(!fmt){ fxerror("FXSettings::readFormatEntry: bad fmt argument.\n"); }
   FXStringDict *group=find(section);
   va_list args;
   va_start(args,fmt);
@@ -444,8 +474,8 @@ FXint FXSettings::readFormatEntry(const FXchar *section,const FXchar *key,const
 
 // Read a string-valued registry entry
 const FXchar *FXSettings::readStringEntry(const FXchar *section,const FXchar *key,const FXchar *def){
-  if(!section){ fxerror("FXSettings::readStringEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::readStringEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::readStringEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::readStringEntry: bad key argument.\n"); }
   FXStringDict *group=find(section);
   if(group){
     const char *value=group->find(key);
@@ -457,8 +487,8 @@ const FXchar *FXSettings::readStringEntry(const FXchar *section,const FXchar *ke
 
 // Read a int-valued registry entry
 FXint FXSettings::readIntEntry(const FXchar *section,const FXchar *key,FXint def){
-  if(!section){ fxerror("FXSettings::readIntEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::readIntEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::readIntEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::readIntEntry: bad key argument.\n"); }
   FXStringDict *group=find(section);
   if(group){
     const char *value=group->find(key);
@@ -478,8 +508,8 @@ FXint FXSettings::readIntEntry(const FXchar *section,const FXchar *key,FXint def
 
 // Read a unsigned int-valued registry entry
 FXuint FXSettings::readUnsignedEntry(const FXchar *section,const FXchar *key,FXuint def){
-  if(!section){ fxerror("FXSettings::readUnsignedEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::readUnsignedEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::readUnsignedEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::readUnsignedEntry: bad key argument.\n"); }
   FXStringDict *group=find(section);
   if(group){
     const char *value=group->find(key);
@@ -499,8 +529,8 @@ FXuint FXSettings::readUnsignedEntry(const FXchar *section,const FXchar *key,FXu
 
 // Read a double-valued registry entry
 FXdouble FXSettings::readRealEntry(const FXchar *section,const FXchar *key,FXdouble def){
-  if(!section){ fxerror("FXSettings::readRealEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::readRealEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::readRealEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::readRealEntry: bad key argument.\n"); }
   FXStringDict *group=find(section);
   if(group){
     const char *value=group->find(key);
@@ -515,8 +545,8 @@ FXdouble FXSettings::readRealEntry(const FXchar *section,const FXchar *key,FXdou
 
 // Read a color registry entry
 FXColor FXSettings::readColorEntry(const FXchar *section,const FXchar *key,FXColor def){
-  if(!section){ fxerror("FXSettings::readColorEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::readColorEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::readColorEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::readColorEntry: bad key argument.\n"); }
   FXStringDict *group=find(section);
   if(group){
     const char *value=group->find(key);
@@ -530,9 +560,9 @@ FXColor FXSettings::readColorEntry(const FXchar *section,const FXchar *key,FXCol
 
 // Write a formatted registry entry
 FXint FXSettings::writeFormatEntry(const FXchar *section,const FXchar *key,const FXchar *fmt,...){
-  if(!section){ fxerror("FXSettings::writeFormatEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::writeFormatEntry: NULL key argument.\n"); }
-  if(!fmt){ fxerror("FXSettings::writeFormatEntry: NULL fmt argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::writeFormatEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::writeFormatEntry: bad key argument.\n"); }
+  if(!fmt){ fxerror("FXSettings::writeFormatEntry: bad fmt argument.\n"); }
   FXStringDict *group=insert(section);
   va_list args;
   va_start(args,fmt);
@@ -554,8 +584,8 @@ FXint FXSettings::writeFormatEntry(const FXchar *section,const FXchar *key,const
 
 // Write a string-valued registry entry
 FXbool FXSettings::writeStringEntry(const FXchar *section,const FXchar *key,const FXchar *val){
-  if(!section){ fxerror("FXSettings::writeStringEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::writeStringEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::writeStringEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::writeStringEntry: bad key argument.\n"); }
   FXStringDict *group=insert(section);
   if(group){
     group->replace(key,val,TRUE);
@@ -568,8 +598,8 @@ FXbool FXSettings::writeStringEntry(const FXchar *section,const FXchar *key,cons
 
 // Write a int-valued registry entry
 FXbool FXSettings::writeIntEntry(const FXchar *section,const FXchar *key,FXint val){
-  if(!section){ fxerror("FXSettings::writeIntEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::writeIntEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::writeIntEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::writeIntEntry: bad key argument.\n"); }
   FXStringDict *group=insert(section);
   if(group){
     FXchar buffer[10];
@@ -584,8 +614,8 @@ FXbool FXSettings::writeIntEntry(const FXchar *section,const FXchar *key,FXint v
 
 // Write a unsigned int-valued registry entry
 FXbool FXSettings::writeUnsignedEntry(const FXchar *section,const FXchar *key,FXuint val){
-  if(!section){ fxerror("FXSettings::writeUnsignedEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::writeUnsignedEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::writeUnsignedEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::writeUnsignedEntry: bad key argument.\n"); }
   FXStringDict *group=insert(section);
   if(group){
     FXchar buffer[10];
@@ -600,8 +630,8 @@ FXbool FXSettings::writeUnsignedEntry(const FXchar *section,const FXchar *key,FX
 
 // Write a double-valued registry entry
 FXbool FXSettings::writeRealEntry(const FXchar *section,const FXchar *key,FXdouble val){
-  if(!section){ fxerror("FXSettings::writeRealEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::writeRealEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::writeRealEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::writeRealEntry: bad key argument.\n"); }
   FXStringDict *group=insert(section);
   if(group){
     FXchar buffer[60];
@@ -616,8 +646,8 @@ FXbool FXSettings::writeRealEntry(const FXchar *section,const FXchar *key,FXdoub
 
 // Write a color registry entry
 FXbool FXSettings::writeColorEntry(const FXchar *section,const FXchar *key,FXColor val){
-  if(!section){ fxerror("FXSettings::writeColorEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::writeColorEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::writeColorEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::writeColorEntry: bad key argument.\n"); }
   FXStringDict *group=insert(section);
   if(group){
     FXchar buffer[60];
@@ -631,8 +661,8 @@ FXbool FXSettings::writeColorEntry(const FXchar *section,const FXchar *key,FXCol
 
 // Delete a registry entry
 FXbool FXSettings::deleteEntry(const FXchar *section,const FXchar *key){
-  if(!section){ fxerror("FXSettings::deleteEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::deleteEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::deleteEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::deleteEntry: bad key argument.\n"); }
   FXStringDict *group=insert(section);
   if(group){
     group->remove(key);
@@ -645,7 +675,7 @@ FXbool FXSettings::deleteEntry(const FXchar *section,const FXchar *key){
 
 // Delete section
 FXbool FXSettings::deleteSection(const FXchar *section){
-  if(!section){ fxerror("FXSettings::deleteSection: NULL section argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::deleteSection: bad section argument.\n"); }
   remove(section);
   modified=TRUE;
   return TRUE;
@@ -662,15 +692,15 @@ FXbool FXSettings::clear(){
 
 // See if section exists
 FXbool FXSettings::existingSection(const FXchar *section){
-  if(!section){ fxerror("FXSettings::existingSection: NULL section argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::existingSection: bad section argument.\n"); }
   return find(section)!=NULL;
   }
 
 
 // See if entry exists
 FXbool FXSettings::existingEntry(const FXchar *section,const FXchar *key){
-  if(!section){ fxerror("FXSettings::existingEntry: NULL section argument.\n"); }
-  if(!key){ fxerror("FXSettings::existingEntry: NULL key argument.\n"); }
+  if(!section || !section[0]){ fxerror("FXSettings::existingEntry: bad section argument.\n"); }
+  if(!key || !key[0]){ fxerror("FXSettings::existingEntry: bad key argument.\n"); }
   FXStringDict *group=find(section);
   return group && group->find(key)!=NULL;
   }
@@ -680,3 +710,5 @@ FXbool FXSettings::existingEntry(const FXchar *section,const FXchar *key){
 FXSettings::~FXSettings(){
   clear();
   }
+
+}
