@@ -3,7 +3,7 @@
 *                               S i z e    C l a s s                            *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1994,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1994,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXSize.cpp,v 1.3 2002/01/18 22:43:04 jeroen Exp $                        *
+* $Id: FXSize.cpp,v 1.6 2003/04/05 14:22:37 fox Exp $                           *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -28,7 +28,11 @@
 #include "FXStream.h"
 #include "FXSize.h"
 
+using namespace FX;
 
+/*******************************************************************************/
+
+namespace FX {
 
 // Save object to a stream
 FXStream& operator<<(FXStream& store,const FXSize& s){
@@ -42,3 +46,4 @@ FXStream& operator>>(FXStream& store,FXSize& s){
   return store;
   }
 
+}
