@@ -3,7 +3,7 @@
 *                             S l i d e r   O b j e c t s                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXSlider.cpp,v 1.33.4.2 2003/06/20 19:02:07 fox Exp $                     *
+* $Id: FXSlider.cpp,v 1.46 2003/10/23 17:53:17 fox Exp $                        *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -30,6 +30,7 @@
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXDCWindow.h"
 #include "FXSlider.h"
@@ -61,8 +62,11 @@
 
 #define SLIDER_MASK (SLIDER_VERTICAL|SLIDER_ARROW_UP|SLIDER_ARROW_DOWN|SLIDER_INSIDE_BAR|SLIDER_TICKS_TOP|SLIDER_TICKS_BOTTOM)
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
 
 // Map
 FXDEFMAP(FXSlider) FXSliderMap[]={
@@ -73,19 +77,22 @@ FXDEFMAP(FXSlider) FXSliderMap[]={
   FXMAPFUNC(SEL_MIDDLEBUTTONPRESS,0,FXSlider::onMiddleBtnPress),
   FXMAPFUNC(SEL_MIDDLEBUTTONRELEASE,0,FXSlider::onMiddleBtnRelease),
   FXMAPFUNC(SEL_UNGRABBED,0,FXSlider::onUngrabbed),
-  FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_TIP,FXSlider::onQueryTip),
-  FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_HELP,FXSlider::onQueryHelp),
-  FXMAPFUNC(SEL_TIMEOUT,FXSlider::ID_AUTOINC,FXSlider::onTimeInc),
-  FXMAPFUNC(SEL_TIMEOUT,FXSlider::ID_AUTODEC,FXSlider::onTimeDec),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETVALUE,FXSlider::onCmdSetValue),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETINTVALUE,FXSlider::onCmdSetIntValue),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETREALVALUE,FXSlider::onCmdSetRealValue),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_GETINTVALUE,FXSlider::onCmdGetIntValue),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_GETREALVALUE,FXSlider::onCmdGetRealValue),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETINTRANGE,FXSlider::onCmdSetIntRange),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_GETINTRANGE,FXSlider::onCmdGetIntRange),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETREALRANGE,FXSlider::onCmdSetRealRange),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_GETREALRANGE,FXSlider::onCmdGetRealRange),
+  FXMAPFUNC(SEL_UPDATE,FXSlider::ID_QUERY_TIP,FXSlider::onQueryTip),
+  FXMAPFUNC(SEL_UPDATE,FXSlider::ID_QUERY_HELP,FXSlider::onQueryHelp),
+  FXMAPFUNC(SEL_TIMEOUT,FXSlider::ID_AUTOSLIDE,FXSlider::onAutoSlide),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_SETVALUE,FXSlider::onCmdSetValue),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_SETINTVALUE,FXSlider::onCmdSetIntValue),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_SETREALVALUE,FXSlider::onCmdSetRealValue),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_GETINTVALUE,FXSlider::onCmdGetIntValue),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_GETREALVALUE,FXSlider::onCmdGetRealValue),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_SETINTRANGE,FXSlider::onCmdSetIntRange),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_GETINTRANGE,FXSlider::onCmdGetIntRange),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_SETREALRANGE,FXSlider::onCmdSetRealRange),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_GETREALRANGE,FXSlider::onCmdGetRealRange),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_SETHELPSTRING,FXSlider::onCmdSetHelp),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_GETHELPSTRING,FXSlider::onCmdGetHelp),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_SETTIPSTRING,FXSlider::onCmdSetTip),
+  FXMAPFUNC(SEL_COMMAND,FXSlider::ID_GETTIPSTRING,FXSlider::onCmdGetTip),
   };
 
 
@@ -96,7 +103,6 @@ FXIMPLEMENT(FXSlider,FXFrame,FXSliderMap,ARRAYNUMBER(FXSliderMap))
 // Make a slider
 FXSlider::FXSlider(){
   flags|=FLAG_ENABLED|FLAG_SHOWN;
-  timer=NULL;
   headpos=0;
   dragpoint=0;
   }
@@ -108,7 +114,6 @@ FXSlider::FXSlider(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXint
   flags|=FLAG_ENABLED|FLAG_SHOWN;
   target=tgt;
   message=sel;
-  timer=NULL;
   range[0]=0;
   range[1]=100;
   pos=50;
@@ -225,10 +230,38 @@ void FXSlider::layout(){
   }
 
 
+// Set help using a message
+long FXSlider::onCmdSetHelp(FXObject*,FXSelector,void* ptr){
+  setHelpText(*((FXString*)ptr));
+  return 1;
+  }
+
+
+// Get help using a message
+long FXSlider::onCmdGetHelp(FXObject*,FXSelector,void* ptr){
+  *((FXString*)ptr)=getHelpText();
+  return 1;
+  }
+
+
+// Set tip using a message
+long FXSlider::onCmdSetTip(FXObject*,FXSelector,void* ptr){
+  setTipText(*((FXString*)ptr));
+  return 1;
+  }
+
+
+// Get tip using a message
+long FXSlider::onCmdGetTip(FXObject*,FXSelector,void* ptr){
+  *((FXString*)ptr)=getTipText();
+  return 1;
+  }
+
+
 // We were asked about status text
 long FXSlider::onQueryHelp(FXObject* sender,FXSelector,void*){
   if(!help.empty() && (flags&FLAG_HELP)){
-    sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),&help);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),&help);
     return 1;
     }
   return 0;
@@ -238,7 +271,7 @@ long FXSlider::onQueryHelp(FXObject* sender,FXSelector,void*){
 // We were asked about tip text
 long FXSlider::onQueryTip(FXObject* sender,FXSelector,void*){
   if(!tip.empty() && (flags&FLAG_TIP)){
-    sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),&tip);
+    sender->handle(this,FXSEL(SEL_COMMAND,ID_SETSTRINGVALUE),&tip);
     return 1;
     }
   return 0;
@@ -316,15 +349,15 @@ long FXSlider::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   flags&=~FLAG_TIP;
   if(isEnabled()){
     grab();
-    if(timer) timer=getApp()->removeTimeout(timer);
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
+    getApp()->removeTimeout(this,ID_AUTOSLIDE);
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONPRESS,message),ptr)) return 1;
     if(options&SLIDER_VERTICAL){
       if(event->win_y<headpos){
-        timer=getApp()->addTimeout(getApp()->getScrollDelay(),this,ID_AUTOINC);
+        getApp()->addTimeout(this,ID_AUTOSLIDE,getApp()->getScrollDelay(),(void*)(FXival)incr);
         p=pos+incr;
         }
       else if(event->win_y>(headpos+headsize)){
-        timer=getApp()->addTimeout(getApp()->getScrollDelay(),this,ID_AUTODEC);
+        getApp()->addTimeout(this,ID_AUTOSLIDE,getApp()->getScrollDelay(),(void*)(FXival)-incr);
         p=pos-incr;
         }
       else{
@@ -334,11 +367,11 @@ long FXSlider::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
       }
     else{
       if(event->win_x<headpos){
-        timer=getApp()->addTimeout(getApp()->getScrollDelay(),this,ID_AUTODEC);
+        getApp()->addTimeout(this,ID_AUTOSLIDE,getApp()->getScrollDelay(),(void*)(FXival)-incr);
         p=pos-incr;
         }
       else if(event->win_x>(headpos+headsize)){
-        timer=getApp()->addTimeout(getApp()->getScrollDelay(),this,ID_AUTOINC);
+        getApp()->addTimeout(this,ID_AUTOSLIDE,getApp()->getScrollDelay(),(void*)(FXival)incr);
         p=pos+incr;
         }
       else{
@@ -350,7 +383,7 @@ long FXSlider::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
     if(p>range[1]) p=range[1];
     if(p!=pos){
       setValue(p);
-      if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)(FXival)pos);
+      if(target) target->handle(this,FXSEL(SEL_CHANGED,message),(void*)(FXival)pos);
       flags|=FLAG_CHANGED;
       }
     flags&=~FLAG_UPDATE;
@@ -365,14 +398,14 @@ long FXSlider::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
   FXuint flgs=flags;
   if(isEnabled()){
     ungrab();
-    if(timer) timer=getApp()->removeTimeout(timer);
+    getApp()->removeTimeout(this,ID_AUTOSLIDE);
     setValue(pos);                                                 // Hop to exact position
     flags&=~FLAG_PRESSED;
     flags&=~FLAG_CHANGED;
     flags|=FLAG_UPDATE;
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONRELEASE,message),ptr)) return 1;
     if(flgs&FLAG_CHANGED){
-      if(target) target->handle(this,MKUINT(message,SEL_COMMAND),(void*)(FXival)pos);
+      if(target) target->handle(this,FXSEL(SEL_COMMAND,message),(void*)(FXival)pos);
       }
     return 1;
     }
@@ -424,7 +457,7 @@ long FXSlider::onMotion(FXObject*,FXSelector,void* ptr){
     if(p>range[1]) p=range[1];
     if(pos!=p){
       pos=p;
-      if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)(FXival)pos);
+      if(target) target->handle(this,FXSEL(SEL_CHANGED,message),(void*)(FXival)pos);
       flags|=FLAG_CHANGED;
       }
     return 1;
@@ -440,7 +473,7 @@ long FXSlider::onMiddleBtnPress(FXObject*,FXSelector,void* ptr){
   flags&=~FLAG_TIP;
   if(isEnabled()){
     grab();
-    if(target && target->handle(this,MKUINT(message,SEL_MIDDLEBUTTONPRESS),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_MIDDLEBUTTONPRESS,message),ptr)) return 1;
     dragpoint=headsize/2;
     yy=border+padtop+2;
     xx=border+padleft+2;
@@ -480,7 +513,7 @@ long FXSlider::onMiddleBtnPress(FXObject*,FXSelector,void* ptr){
     if(p>range[1]) p=range[1];
     if(p!=pos){
       pos=p;
-      if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)(FXival)pos);
+      if(target) target->handle(this,FXSEL(SEL_CHANGED,message),(void*)(FXival)pos);
       flags|=FLAG_CHANGED;
       }
     flags|=FLAG_PRESSED;
@@ -496,14 +529,14 @@ long FXSlider::onMiddleBtnRelease(FXObject*,FXSelector,void* ptr){
   FXuint flgs=flags;
   if(isEnabled()){
     ungrab();
-    if(timer) timer=getApp()->removeTimeout(timer);
+    getApp()->removeTimeout(this,ID_AUTOSLIDE);
     flags&=~FLAG_PRESSED;
     flags&=~FLAG_CHANGED;
     flags|=FLAG_UPDATE;
     setValue(pos);                                                 // Hop to exact position
-    if(target && target->handle(this,MKUINT(message,SEL_MIDDLEBUTTONRELEASE),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_MIDDLEBUTTONRELEASE,message),ptr)) return 1;
     if(flgs&FLAG_CHANGED){
-      if(target) target->handle(this,MKUINT(message,SEL_COMMAND),(void*)(FXival)pos);
+      if(target) target->handle(this,FXSEL(SEL_COMMAND,message),(void*)(FXival)pos);
       }
     return 1;
     }
@@ -514,7 +547,7 @@ long FXSlider::onMiddleBtnRelease(FXObject*,FXSelector,void* ptr){
 // The widget lost the grab for some reason
 long FXSlider::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
   FXFrame::onUngrabbed(sender,sel,ptr);
-  if(timer) timer=getApp()->removeTimeout(timer);
+  getApp()->removeTimeout(this,ID_AUTOSLIDE);
   flags&=~FLAG_PRESSED;
   flags&=~FLAG_CHANGED;
   flags|=FLAG_UPDATE;
@@ -522,39 +555,22 @@ long FXSlider::onUngrabbed(FXObject* sender,FXSelector sel,void* ptr){
   }
 
 
-// Increment line timeout
-long FXSlider::onTimeInc(FXObject*,FXSelector,void*){
-  FXint p=pos+incr;
-  if(p>=range[1]){
-    p=range[1];
-    timer=NULL;
-    }
-  else{
-    timer=getApp()->addTimeout(getApp()->getScrollSpeed(),this,ID_AUTOINC);
-    }
-  if(p!=pos){
-    setValue(p);
-    if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)(FXival)pos);
-    flags|=FLAG_CHANGED;
-    return 1;
-    }
-  return 0;
-  }
-
-
-// Decrement line timeout
-long FXSlider::onTimeDec(FXObject*,FXSelector,void*){
-  FXint p=pos-incr;
+// Automatically move slider while holding down mouse
+long FXSlider::onAutoSlide(FXObject*,FXSelector,void* ptr){
+  register FXint inc=(FXint)(FXival)ptr;
+  register FXint p=pos+inc;
   if(p<=range[0]){
     p=range[0];
-    timer=NULL;
+    }
+  else if(p>=range[1]){
+    p=range[1];
     }
   else{
-    timer=getApp()->addTimeout(getApp()->getScrollSpeed(),this,ID_AUTODEC);
+    getApp()->addTimeout(this,ID_AUTOSLIDE,getApp()->getScrollSpeed(),(void*)(FXival)inc);
     }
   if(p!=pos){
     setValue(p);
-    if(target) target->handle(this,MKUINT(message,SEL_CHANGED),(void*)(FXival)pos);
+    if(target) target->handle(this,FXSEL(SEL_CHANGED,message),(void*)(FXival)pos);
     flags|=FLAG_CHANGED;
     return 1;
     }
@@ -595,7 +611,7 @@ void FXSlider::drawVertTicks(FXDCWindow& dc,FXint x,FXint,FXint,FXint){
 // Draw slider head
 void FXSlider::drawSliderHead(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
   FXint m;
-  dc.setForeground(backColor);
+  dc.setForeground(baseColor);
   dc.fillRectangle(x,y,w,h);
   if(options&SLIDER_VERTICAL){
     m=(h>>1);
@@ -687,10 +703,14 @@ void FXSlider::drawSliderHead(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h){
 
 // Handle repaint
 long FXSlider::onPaint(FXObject*,FXSelector,void* ptr){
-  FXEvent *ev=(FXEvent*)ptr;
+  FXEvent *event=(FXEvent*)ptr;
   FXint tx,ty,hhs=headsize/2;
   FXint xx,yy,ww,hh;
-  FXDCWindow dc(this,ev);
+  FXDCWindow dc(this,event);
+
+  // Paint background
+  dc.setForeground(backColor);
+  dc.fillRectangle(0,0,width,height);
 
   // Repaint border
   drawFrame(dc,0,0,width,height);
@@ -702,9 +722,6 @@ long FXSlider::onPaint(FXObject*,FXSelector,void* ptr){
   hh=height-(border<<1)-padtop-padbottom;
   FXASSERT(range[0]<=pos && pos<=range[1]);
 
-  dc.setForeground(backColor);
-  dc.fillRectangle(xx,yy,ww,hh);
-
   // Draw the slot
   if(options&SLIDER_VERTICAL){
 
@@ -714,11 +731,6 @@ long FXSlider::onPaint(FXObject*,FXSelector,void* ptr){
 
     // Draw slider
     if(options&SLIDER_INSIDE_BAR){
-//       dc.setForeground(backColor);
-//       dc.fillRectangle(border,border,width-(border<<1),padtop);
-//       dc.fillRectangle(border,height-border-padbottom,width-(border<<1),padbottom);
-//       dc.fillRectangle(border,border+padtop,padleft,hh);
-//       dc.fillRectangle(width-border-padright,border+padtop,padright,hh);
       drawDoubleSunkenRectangle(dc,xx,yy,ww,hh);
       dc.setStipple(STIPPLE_GRAY);
       dc.setForeground(slotColor);
@@ -734,11 +746,6 @@ long FXSlider::onPaint(FXObject*,FXSelector,void* ptr){
       if(options&SLIDER_ARROW_LEFT) tx=xx+hhs+(ww-slotsize-hhs)/2;
       else if(options&SLIDER_ARROW_RIGHT) tx=xx+(ww-slotsize-hhs)/2;
       else tx=xx+(ww-slotsize)/2;
-//       dc.setForeground(backColor);
-//       dc.fillRectangle(border,border,width-(border<<1),padtop);
-//       dc.fillRectangle(border,height-border-padbottom,width-(border<<1),padbottom);
-//       dc.fillRectangle(border,border+padtop,tx-border,hh);
-//       dc.fillRectangle(tx+slotsize,border+padtop,width-border-tx-slotsize,hh);
       drawDoubleSunkenRectangle(dc,tx,yy,slotsize,hh);
       dc.setForeground(slotColor);
       dc.fillRectangle(tx+2,yy+2,slotsize-4,hh-4);
@@ -755,11 +762,6 @@ long FXSlider::onPaint(FXObject*,FXSelector,void* ptr){
 
     // Draw slider
     if(options&SLIDER_INSIDE_BAR){
-//       dc.setForeground(backColor);
-//       dc.fillRectangle(border,border,width-(border<<1),padtop);
-//       dc.fillRectangle(border,height-border-padbottom,width-(border<<1),padbottom);
-//       dc.fillRectangle(border,border+padtop,padleft,hh);
-//       dc.fillRectangle(width-border-padright,border+padtop,padright,hh);
       drawDoubleSunkenRectangle(dc,xx,yy,ww,hh);
       dc.setForeground(slotColor);
       dc.setStipple(STIPPLE_GRAY);
@@ -776,11 +778,6 @@ long FXSlider::onPaint(FXObject*,FXSelector,void* ptr){
       if(options&SLIDER_ARROW_UP) ty=yy+hhs+(hh-slotsize-hhs)/2;
       else if(options&SLIDER_ARROW_DOWN) ty=yy+(hh-slotsize-hhs)/2;
       else ty=yy+(hh-slotsize)/2;
-//       dc.setForeground(backColor);
-//       dc.fillRectangle(border,border,width-(border<<1),ty-border);
-//       dc.fillRectangle(border,ty+slotsize,width-(border<<1),height-border-ty-slotsize);
-//       dc.fillRectangle(border,border+padtop,padleft,hh);
-//       dc.fillRectangle(width-border-padright,border+padtop,padright,hh);
       drawDoubleSunkenRectangle(dc,xx,ty,ww,slotsize);
       dc.setForeground(slotColor);
       dc.fillRectangle(xx+2,ty+2,ww-4,slotsize-4);
@@ -794,8 +791,8 @@ long FXSlider::onPaint(FXObject*,FXSelector,void* ptr){
 
 
 // Set slider range; this also revalidates the position,
-// and possibly moves the head [even if the position was still
-// OK, the head might still have to be moved to the exact position].
+// and possibly moves the head [even if the position was still OK,
+// the head might still have to be moved to the exact position].
 void FXSlider::setRange(FXint lo,FXint hi){
   if(lo>hi){ fxerror("%s::setRange: trying to set negative range.\n",getClassName()); }
   if(range[0]!=lo || range[1]!=hi){
@@ -952,8 +949,7 @@ void FXSlider::load(FXStream& store){
 
 // Delete
 FXSlider::~FXSlider(){
-  if(timer){getApp()->removeTimeout(timer);}
-  timer=(FXTimer*)-1;
+  getApp()->removeTimeout(this,ID_AUTOSLIDE);
   }
 
-
+}
