@@ -3,7 +3,7 @@
 *                S p l i t t e r   W i n d o w   O b j e c t                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXSplitter.cpp,v 1.26.4.1 2003/06/06 19:32:03 fox Exp $                   *
+* $Id: FXSplitter.cpp,v 1.38 2003/10/23 04:19:45 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -30,6 +30,7 @@
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXDCWindow.h"
 #include "FXSplitter.h"
@@ -54,15 +55,18 @@
     Note that this rule must have one exception: the last (or first if the option
     SPLITTER_REVERSED has been passed) must necessarily be allowed to be
     any size.
-
+  - If we're just re-sizing a split, do we need to incur a GUI-Update?
 */
 
 // Splitter styles
 #define SPLITTER_MASK     (SPLITTER_REVERSED|SPLITTER_VERTICAL|SPLITTER_TRACKING)
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 // Map
 FXDEFMAP(FXSplitter) FXSplitterMap[]={
   FXMAPFUNC(SEL_MOTION,0,FXSplitter::onMotion),
@@ -434,7 +438,7 @@ long FXSplitter::onLeftBtnPress(FXObject*,FXSelector,void* ptr){
   FXEvent* ev=(FXEvent*)ptr;
   if(isEnabled()){
     grab();
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONPRESS),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONPRESS,message),ptr)) return 1;
     if(options&SPLITTER_VERTICAL){
       window=findVSplit(ev->win_y);
       if(window){
@@ -479,7 +483,7 @@ long FXSplitter::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
     flags|=FLAG_UPDATE;
     flags&=~FLAG_CHANGED;
     flags&=~FLAG_PRESSED;
-    if(target && target->handle(this,MKUINT(message,SEL_LEFTBUTTONRELEASE),ptr)) return 1;
+    if(target && target->handle(this,FXSEL(SEL_LEFTBUTTONRELEASE,message),ptr)) return 1;
     if(flgs&FLAG_PRESSED){
       if(!(options&SPLITTER_TRACKING)){
         if(options&SPLITTER_VERTICAL){
@@ -491,11 +495,11 @@ long FXSplitter::onLeftBtnRelease(FXObject*,FXSelector,void* ptr){
           adjustHLayout();
           }
         if(flgs&FLAG_CHANGED){
-          if(target) target->handle(this,MKUINT(message,SEL_CHANGED),NULL);
+          if(target) target->handle(this,FXSEL(SEL_CHANGED,message),NULL);
           }
         }
       if(flgs&FLAG_CHANGED){
-        if(target) target->handle(this,MKUINT(message,SEL_COMMAND),NULL);
+        if(target) target->handle(this,FXSEL(SEL_COMMAND,message),NULL);
         }
       }
     return 1;
@@ -519,7 +523,7 @@ long FXSplitter::onMotion(FXObject*,FXSelector,void* ptr){
           }
         else{
           adjustVLayout();
-          if(target) target->handle(this,MKUINT(message,SEL_CHANGED),NULL);
+          if(target) target->handle(this,FXSEL(SEL_CHANGED,message),NULL);
           }
         flags|=FLAG_CHANGED;
         }
@@ -533,7 +537,7 @@ long FXSplitter::onMotion(FXObject*,FXSelector,void* ptr){
           }
         else{
           adjustHLayout();
-          if(target) target->handle(this,MKUINT(message,SEL_CHANGED),NULL);
+          if(target) target->handle(this,FXSEL(SEL_CHANGED,message),NULL);
           }
         flags|=FLAG_CHANGED;
         }
@@ -557,7 +561,7 @@ long FXSplitter::onFocusPrev(FXObject* sender,FXSelector sel,void* ptr){
 
 
 // Focus moved up
-long FXSplitter::onFocusUp(FXObject*,FXSelector sel,void* ptr){
+long FXSplitter::onFocusUp(FXObject*,FXSelector,void* ptr){
   FXWindow *child;
   if(options&SPLITTER_VERTICAL){
     if(getFocus())
@@ -565,11 +569,8 @@ long FXSplitter::onFocusUp(FXObject*,FXSelector sel,void* ptr){
     else
       child=getLast();
     while(child){
-      if(child->isEnabled() && child->canFocus()){
-        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-        return 1;
-        }
-      if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+      if(child->handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr)) return 1;
+      if(child->handle(this,FXSEL(SEL_FOCUS_UP,0),ptr)) return 1;
       child=child->getPrev();
       }
     }
@@ -578,7 +579,7 @@ long FXSplitter::onFocusUp(FXObject*,FXSelector sel,void* ptr){
 
 
 // Focus moved down
-long FXSplitter::onFocusDown(FXObject*,FXSelector sel,void* ptr){
+long FXSplitter::onFocusDown(FXObject*,FXSelector,void* ptr){
   FXWindow *child;
   if(options&SPLITTER_VERTICAL){
     if(getFocus())
@@ -586,11 +587,8 @@ long FXSplitter::onFocusDown(FXObject*,FXSelector sel,void* ptr){
     else
       child=getFirst();
     while(child){
-      if(child->isEnabled() && child->canFocus()){
-        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-        return 1;
-        }
-      if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+      if(child->handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr)) return 1;
+      if(child->handle(this,FXSEL(SEL_FOCUS_DOWN,0),ptr)) return 1;
       child=child->getNext();
       }
     }
@@ -599,7 +597,7 @@ long FXSplitter::onFocusDown(FXObject*,FXSelector sel,void* ptr){
 
 
 // Focus moved to left
-long FXSplitter::onFocusLeft(FXObject*,FXSelector sel,void* ptr){
+long FXSplitter::onFocusLeft(FXObject*,FXSelector,void* ptr){
   FXWindow *child;
   if(!(options&SPLITTER_VERTICAL)){
     if(getFocus())
@@ -607,11 +605,8 @@ long FXSplitter::onFocusLeft(FXObject*,FXSelector sel,void* ptr){
     else
       child=getLast();
     while(child){
-      if(child->isEnabled() && child->canFocus()){
-        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-        return 1;
-        }
-      if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+      if(child->handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr)) return 1;
+      if(child->handle(this,FXSEL(SEL_FOCUS_LEFT,0),ptr)) return 1;
       child=child->getPrev();
       }
     }
@@ -620,7 +615,7 @@ long FXSplitter::onFocusLeft(FXObject*,FXSelector sel,void* ptr){
 
 
 // Focus moved to right
-long FXSplitter::onFocusRight(FXObject*,FXSelector sel,void* ptr){
+long FXSplitter::onFocusRight(FXObject*,FXSelector,void* ptr){
   FXWindow *child;
   if(!(options&SPLITTER_VERTICAL)){
     if(getFocus())
@@ -628,11 +623,8 @@ long FXSplitter::onFocusRight(FXObject*,FXSelector sel,void* ptr){
     else
       child=getFirst();
     while(child){
-      if(child->isEnabled() && child->canFocus()){
-        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
-        return 1;
-        }
-      if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+      if(child->handle(this,FXSEL(SEL_FOCUS_SELF,0),ptr)) return 1;
+      if(child->handle(this,FXSEL(SEL_FOCUS_RIGHT,0),ptr)) return 1;
       child=child->getNext();
       }
     }
@@ -658,6 +650,36 @@ void FXSplitter::drawVSplit(FXint pos){
   }
 
 
+// Return size of the panel at index
+FXint FXSplitter::getSplit(FXint index) const {
+  FXWindow *window=childAtIndex(index);
+  if(window){
+    if(options&SPLITTER_VERTICAL){
+      return window->getHeight();
+      }
+    else{
+      return window->getWidth();
+      }
+    }
+  return 0;
+  }
+
+
+// Change the size of panel at the given index
+void FXSplitter::setSplit(FXint index,FXint size){
+  FXWindow *window=childAtIndex(index);
+  if(window){
+    if(options&SPLITTER_VERTICAL){
+      window->setHeight(size);
+      }
+    else{
+      window->setWidth(size);
+      }
+    window->recalc();
+    }
+  }
+
+
 // Return splitter style
 FXuint FXSplitter::getSplitterStyle() const {
   return (options&SPLITTER_MASK);
@@ -722,5 +744,7 @@ void FXSplitter::load(FXStream& store){
 
 // Zap it
 FXSplitter::~FXSplitter(){
-  window=(FXWindow*)-1;
+  window=(FXWindow*)-1L;
   }
+
+}
