@@ -3,7 +3,7 @@
 *                          T I F F  I c o n   O b j e c t                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2001,2002 Eric Gillet.   All Rights Reserved.                   *
+* Copyright (C) 2001,2003 Eric Gillet.   All Rights Reserved.                   *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,17 +19,19 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXTIFIcon.cpp,v 1.6 2002/01/18 22:55:04 jeroen Exp $                     *
+* $Id: FXTIFIcon.cpp,v 1.17 2003/10/23 04:19:45 fox Exp $                       *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
+#include "FXMemoryStream.h"
 #include "FXString.h"
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXTIFIcon.h"
 
@@ -39,19 +41,22 @@
   - FXTIFIcon has an alpha channel.
 */
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 FXIMPLEMENT(FXTIFIcon,FXIcon,NULL,0)
 
 
 // Initialize
 FXTIFIcon::FXTIFIcon(FXApp* a,const void *pix,FXColor clr,FXuint opts,FXint w,FXint h):
-  FXIcon(a,NULL,clr,opts|IMAGE_ALPHA,w,h){
+  FXIcon(a,NULL,clr,opts,w,h){
   codec=0;
   if(pix){
     FXMemoryStream ms;
-    ms.open((FXuchar *)pix,FXStreamLoad);
+    ms.open(FXStreamLoad,(FXuchar*)pix);
     loadPixels(ms);
     ms.close();
     }
@@ -59,24 +64,24 @@ FXTIFIcon::FXTIFIcon(FXApp* a,const void *pix,FXColor clr,FXuint opts,FXint w,FX
 
 
 // Save pixels only
-void FXTIFIcon::savePixels(FXStream& store) const {
-  FXASSERT(options&IMAGE_ALPHA);
-  fxsaveTIF(store,data,transp,width,height,codec);
+FXbool FXTIFIcon::savePixels(FXStream& store) const {
+  if(!fxsaveTIF(store,data,width,height,codec)) return FALSE;
+  return TRUE;
   }
 
 
 // Load pixels only
-void FXTIFIcon::loadPixels(FXStream& store){
-  FXColor clearcolor=0;
-  if(options&IMAGE_OWNED){FXFREE(&data);}
-  fxloadTIF(store,data,clearcolor,width,height,codec);
-  if(!(options&IMAGE_ALPHACOLOR)) transp=clearcolor;
+FXbool FXTIFIcon::loadPixels(FXStream& store){
+  if(options&IMAGE_OWNED){ FXFREE(&data); }
+  if(!fxloadTIF(store,data,width,height,codec)) return FALSE;
   if(options&IMAGE_ALPHAGUESS) transp=guesstransp();
-  options|=IMAGE_ALPHA;
   options|=IMAGE_OWNED;
+  return TRUE;
   }
 
 
 // Clean up
 FXTIFIcon::~FXTIFIcon(){
   }
+
+}
