@@ -3,7 +3,7 @@
 *                          T I F F  I m a g e   O b j e c t                     *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2001,2002 Eric Gillet.   All Rights Reserved.                   *
+* Copyright (C) 2001,2003 Eric Gillet.   All Rights Reserved.                   *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,17 +19,19 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXTIFImage.cpp,v 1.6 2002/01/18 22:55:04 jeroen Exp $                    *
+* $Id: FXTIFImage.cpp,v 1.17 2003/10/23 04:19:45 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
+#include "FXMemoryStream.h"
 #include "FXString.h"
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXRegistry.h"
+#include "FXHash.h"
 #include "FXApp.h"
 #include "FXTIFImage.h"
 
@@ -40,46 +42,48 @@
   - FXTIFImage has an alpha channel.
 */
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
 // Object implementation
 FXIMPLEMENT(FXTIFImage,FXImage,NULL,0)
 
 
 // Initialize
 FXTIFImage::FXTIFImage(FXApp* a,const void *pix,FXuint opts,FXint w,FXint h):
-  FXImage(a,NULL,opts|IMAGE_ALPHA,w,h){
+  FXImage(a,NULL,opts,w,h){
   codec=0;
   if(pix){
     FXMemoryStream ms;
-    FXColor clearcolor;
-    ms.open((FXuchar*)pix,FXStreamLoad);
-    fxloadTIF(ms,data,clearcolor,width,height,codec);
+    ms.open(FXStreamLoad,(FXuchar*)pix);
+    fxloadTIF(ms,data,width,height,codec);
     options|=IMAGE_OWNED;
     ms.close();
     }
   }
 
 // Save the pixels only
-void FXTIFImage::savePixels(FXStream& store) const {
-  FXColor clearcolor=FXRGB(192,192,192);
-  FXASSERT(options&IMAGE_ALPHA);
-  fxsaveTIF(store,data,clearcolor,width,height,codec);
+FXbool FXTIFImage::savePixels(FXStream& store) const {
+  if(!fxsaveTIF(store,data,width,height,codec)) return FALSE;
+  return TRUE;
   }
 
 
 
 // Load pixels only
-void FXTIFImage::loadPixels(FXStream& store){
-  FXColor clearcolor;
-  if(options&IMAGE_OWNED){FXFREE(&data);}
-  fxloadTIF(store,data,clearcolor,width,height,codec);
-  options|=IMAGE_ALPHA;
+FXbool FXTIFImage::loadPixels(FXStream& store){
+  if(options&IMAGE_OWNED){ FXFREE(&data); }
+  if(!fxloadTIF(store,data,width,height,codec)) return FALSE;
   options|=IMAGE_OWNED;
+  return TRUE;
   }
 
 
 // Clean up
 FXTIFImage::~FXTIFImage(){
   }
+
+}
