@@ -3,7 +3,7 @@
 *                   W i l d c a r d   M a t c h   F u n c t i o n               *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2000,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fxfilematch.cpp,v 1.6 2002/01/18 22:43:07 jeroen Exp $                   *
+* $Id: fxfilematch.cpp,v 1.9 2003/04/05 14:22:37 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -72,8 +72,11 @@
 // If folding case, make lower case
 #define FOLD(c)          ((flags&FILEMATCH_CASEFOLD)?tolower(c):(c))
 
+using namespace FX;
+
 /*******************************************************************************/
 
+namespace FX {
 
 
 // Perform match
@@ -194,3 +197,4 @@ nxt:if(domatch(p,q,flags)) return 1;
   return 0;
   }
 
+}
