@@ -3,7 +3,7 @@
 *                      J P E G    I n p u t / O u t p u t                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2002 by David Tyree.   All Rights Reserved.                *
+* Copyright (C) 2000,2003 by David Tyree.   All Rights Reserved.                *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,21 +19,38 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fxjpegio.cpp,v 1.25.4.1 2003/07/27 01:15:11 fox Exp $                     *
+* $Id: fxjpegio.cpp,v 1.40 2003/08/20 13:09:55 fox Exp $                        *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
 #include "fxdefs.h"
 #include "FXStream.h"
+#ifdef HAVE_JPEG_H
+extern "C" {
+/* Theo Veenker <Theo.Veenker@let.uu.nl> says this is needed for CYGWIN */
+#if (defined(__CYGWIN__) || defined(__MINGW32__) || defined(_MSC_VER)) && !defined(XMD_H)
+#define XMD_H
+typedef short INT16;
+typedef int INT32;
+#include "jpeglib.h"
+#undef XMD_H
+#elif defined __WINE__
+#define XMD_H
+#include "jpeglib.h"
+#else
+#include "jpeglib.h"
+#endif
+}
+#endif
+
+#include <setjmp.h>
 
 
 /*
   To Do:
-  - Add more options for fast jpeg loading
-  - Write a more detailed class that offers more options
-  - Add the ability to load jpegs in the background
-  - When loading JPEG, we should NOT load an entire buffer but only as much
-    as needed; otherwise, we'll be reading data that does not belong to us.
+  - Add more options for fast jpeg loading.
+  - Write a more detailed class that offers more options.
+  - Add the ability to load jpegs in the background.
   - We should NOT assume that we can reposition the current stream position;
     for example, with bzip2 or gzip streams this is not possible.
   - References:
@@ -45,36 +62,16 @@
 
 #define JPEG_BUFFER_SIZE 4096
 
-/*******************************************************************************/
-
-/// Load a jpeg from a stream
-extern FXAPI FXbool fxloadJPG(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXint& height,FXint& quality);
-
-
-/// Save a jpeg to a stream
-extern FXAPI FXbool fxsaveJPG(FXStream& store,const FXuchar* data,FXColor transp,FXint width,FXint height,FXint quality);
 
+using namespace FX;
 
 /*******************************************************************************/
 
+namespace FX {
+
+
 #ifdef HAVE_JPEG_H
 
-extern "C" {
-
-/* Theo Veenker <Theo.Veenker@let.uu.nl> says this is needed for CYGWIN */
-#if (defined(__CYGWIN__) || defined(__MINGW32__) || defined(_MSC_VER)) && !defined(XMD_H)
-#define XMD_H
-typedef short INT16;
-typedef int INT32;
-#include "jpeglib.h"
-#undef XMD_H
-#else
-#include "jpeglib.h"
-#endif
-
-}
-
-
 // Source Manager for libjpeg
 struct FOX_jpeg_source_mgr {
   struct jpeg_source_mgr pub;
@@ -91,17 +88,22 @@ struct FOX_jpeg_dest_mgr {
   };
 
 
-// Fatal error use FOX's way of reporing errors
-static void fatal_error(j_common_ptr cinfo){
-  FXchar message[JMSG_LENGTH_MAX];
-  cinfo->err->format_message(cinfo,message);
-  fxerror("FXJPEG: %s\n",message);
-  }
+// For error handler
+struct FOX_jpeg_error_mgr {
+  struct jpeg_error_mgr error_mgr;
+  jmp_buf jmpbuf;
+  };
 
 
 /*******************************************************************************/
 
 
+// Fatal error use FOX's way of reporing errors
+static void fatal_error(j_common_ptr cinfo){
+  longjmp(((FOX_jpeg_error_mgr*)cinfo->err)->jmpbuf,1);
+  }
+
+
 // A no-op in our case
 static void init_source(j_decompress_ptr){
   }
@@ -112,8 +114,13 @@ static void init_source(j_decompress_ptr){
 // data belonging to the objects following this JPEG remain in the stream!
 static boolean fill_input_buffer(j_decompress_ptr cinfo){
   FOX_jpeg_source_mgr *src=(FOX_jpeg_source_mgr*)cinfo->src;
-  src->stream->load(src->buffer,1);
-  if(src->stream->status()!=FXStreamOK){    // Insert a fake EOI marker
+/*
+src->stream->load(src->buffer,JPEG_BUFFER_SIZE);
+src->pub.next_input_byte=src->buffer;
+src->pub.bytes_in_buffer=JPEG_BUFFER_SIZE;
+*/
+  *src->stream >> src->buffer[0];
+  if(src->stream->eof()){    // Insert a fake EOI marker
     src->buffer[0]=0xff;
     src->buffer[1]=JPEG_EOI;
     src->pub.next_input_byte=src->buffer;
@@ -148,22 +155,35 @@ static void term_source(j_decompress_ptr){
 
 
 // Load a JPEG image
-FXbool fxloadJPG(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXint& height,FXint&){
-  struct jpeg_decompress_struct srcinfo;
-  struct jpeg_error_mgr jerr;
+FXbool fxloadJPG(FXStream& store,FXColor*& data,FXint& width,FXint& height,FXint&){
+  jpeg_decompress_struct srcinfo;
+  FOX_jpeg_error_mgr jerr;
   FOX_jpeg_source_mgr src;
-  JSAMPARRAY buffer;
+  JSAMPLE *buffer[1];
+  register FXColor *pp;
+  register JSAMPLE *qq;
   int row_stride;
 
+  // Null out
+  data=NULL;
+  width=0;
+  height=0;
+
+  // No sample buffer
+  buffer[0]=NULL;
+
   // initialize the jpeg data structure;
   jpeg_create_decompress(&srcinfo);
 
   // setup the error handler
-  srcinfo.err = jpeg_std_error(&jerr);
-  jerr.error_exit=fatal_error;
+  srcinfo.err=jpeg_std_error(&jerr.error_mgr);
+  jerr.error_mgr.error_exit=fatal_error;
 
-  // set our src manager
-  srcinfo.src=&src.pub;
+  // Set error handling
+  if(setjmp(jerr.jmpbuf)){
+    jpeg_destroy_decompress(&srcinfo);
+    return FALSE;
+    }
 
   // setup our src manager
   src.pub.init_source=init_source;
@@ -175,6 +195,9 @@ FXbool fxloadJPG(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXi
   src.pub.next_input_byte=NULL;
   src.stream=&store;
 
+  // set our src manager
+  srcinfo.src=&src.pub;
+
   // read the header from the jpg;
   jpeg_read_header(&srcinfo,TRUE);
 
@@ -184,28 +207,40 @@ FXbool fxloadJPG(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXi
   jpeg_start_decompress(&srcinfo);
 
   row_stride=srcinfo.output_width*srcinfo.output_components;
-  height=srcinfo.image_height;
-  width=srcinfo.image_width;
-  transp=0;
 
-  // Buffer to receive
-  FXMALLOC(&data,FXuchar,3*height*width);
-  if(!data){
+  // Data to receive
+  if(!FXMALLOC(&data,FXColor,srcinfo.image_height*srcinfo.image_width)){
     jpeg_destroy_decompress(&srcinfo);
     return FALSE;
     }
 
-  buffer=(*srcinfo.mem->alloc_sarray) ((j_common_ptr)&srcinfo,JPOOL_IMAGE,row_stride,1);
-  FXuchar *temp_data=data;
-  while(srcinfo.output_scanline<srcinfo.output_height){
-    jpeg_read_scanlines(&srcinfo,buffer,1);
-    memcpy(temp_data,*buffer,srcinfo.output_width*srcinfo.output_components);
-    temp_data=temp_data+(srcinfo.output_width*srcinfo.output_components);
+  height=srcinfo.image_height;
+  width=srcinfo.image_width;
+
+  // Sample buffer
+  if(!FXMALLOC(&buffer[0],JSAMPLE,row_stride)){
+    FXFREE(&data);
+    jpeg_destroy_decompress(&srcinfo);
+    return FALSE;
     }
 
-  // wrap up
+  // Read the jpeg data
+  pp=data;
+  while(srcinfo.output_scanline<srcinfo.output_height){
+    jpeg_read_scanlines(&srcinfo,buffer,1);
+    qq=buffer[0];
+    for(FXint i=0; i<width; i++,pp++){
+      ((FXuchar*)pp)[0]=*qq++;
+      ((FXuchar*)pp)[1]=*qq++;
+      ((FXuchar*)pp)[2]=*qq++;
+      ((FXuchar*)pp)[3]=255;
+      }
+    }
+
+  // Clean up
   jpeg_finish_decompress(&srcinfo);
   jpeg_destroy_decompress(&srcinfo);
+  FXFREE(&buffer[0]);
   return TRUE;
   }
 
@@ -239,24 +274,35 @@ static void term_destination(j_compress_ptr cinfo){
 
 
 // Save a JPEG image
-FXbool fxsaveJPG(FXStream& store,const FXuchar* data,FXColor,FXint width,FXint height,FXint quality){
-  struct jpeg_compress_struct dstinfo;
-  struct jpeg_error_mgr jerr;
+FXbool fxsaveJPG(FXStream& store,const FXColor* data,FXint width,FXint height,FXint quality){
+  jpeg_compress_struct dstinfo;
+  FOX_jpeg_error_mgr jerr;
   FOX_jpeg_dest_mgr dst;
-  JSAMPROW row_pointer[1];
-  int row_stride=width*3;
+  JSAMPLE *buffer[1];
+  register const FXColor *pp;
+  register JSAMPLE *qq;
 
-  FXASSERT(data);
-  FXASSERT(0<quality && quality<=100);
+  // Must make sense
+  if(!data || width<=0 || height<=0 || quality<=0 || 100<quality) return FALSE;
 
-  // specify the error manager
-  dstinfo.err=jpeg_std_error(&jerr);
-  jerr.error_exit=fatal_error;
+  // Row buffer
+  if(!FXMALLOC(&buffer,JSAMPLE,width*3)) return FALSE;
+
+  // Specify the error manager
+  dstinfo.err=jpeg_std_error(&jerr.error_mgr);
+  jerr.error_mgr.error_exit=fatal_error;
+
+  // Set error handling
+  if(setjmp(jerr.jmpbuf)){
+    FXFREE(&buffer[0]);
+    jpeg_destroy_compress(&dstinfo);
+    return FALSE;
+    }
 
   // initialize the structure
   jpeg_create_compress(&dstinfo);
 
-  // specify the use of our destination manager
+  // Specify the use of our destination manager
   dst.pub.init_destination=init_destination;
   dst.pub.empty_output_buffer=empty_output_buffer;
   dst.pub.term_destination=term_destination;
@@ -264,7 +310,7 @@ FXbool fxsaveJPG(FXStream& store,const FXuchar* data,FXColor,FXint width,FXint h
   dst.pub.next_output_byte=NULL;
   dst.stream=&store;
 
-  // set up the input paramaters for the file
+  // Set up the input parameters for the file
   dstinfo.image_width=width;
   dstinfo.image_height=height;
   dstinfo.input_components=3;
@@ -275,15 +321,22 @@ FXbool fxsaveJPG(FXStream& store,const FXuchar* data,FXColor,FXint width,FXint h
   jpeg_set_quality(&dstinfo,quality,TRUE);
   jpeg_start_compress(&dstinfo,TRUE);
 
-  // write the jpeg data
+  // Write the jpeg data
+  pp=data;
   while(dstinfo.next_scanline<dstinfo.image_height){
-    row_pointer[0]=(JSAMPROW)&data[dstinfo.next_scanline*row_stride];
-    jpeg_write_scanlines(&dstinfo,row_pointer,1);
+    qq=buffer[0];
+    for(FXint i=0; i<width; i++,pp++){
+      *qq++=((FXuchar*)pp)[0];
+      *qq++=((FXuchar*)pp)[1];
+      *qq++=((FXuchar*)pp)[2];
+      }
+    jpeg_write_scanlines(&dstinfo,buffer,1);
     }
 
-  // wrap up
+  // Clean up
   jpeg_finish_compress(&dstinfo);
   jpeg_destroy_compress(&dstinfo);
+  FXFREE(&buffer[0]);
   return TRUE;
   }
 
@@ -295,7 +348,7 @@ FXbool fxsaveJPG(FXStream& store,const FXuchar* data,FXColor,FXint width,FXint h
 
 
 // Stub routine
-FXbool fxloadJPG(FXStream&,FXuchar*& data,FXColor& transp,FXint& width,FXint& height,FXint& quality){
+FXbool fxloadJPG(FXStream&,FXColor*& data,FXint& width,FXint& height,FXint& quality){
   static const FXuchar jpeg_bits[] = {
    0xff, 0xff, 0xff, 0xff, 0x01, 0x00, 0x00, 0x80, 0xfd, 0xff, 0xff, 0xbf,
    0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0,
@@ -308,22 +361,23 @@ FXbool fxloadJPG(FXStream&,FXuchar*& data,FXColor& transp,FXint& width,FXint& he
    0xe5, 0x04, 0x9f, 0xa3, 0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0,
    0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0, 0xfd, 0xff, 0xff, 0xbf,
    0x01, 0x00, 0x00, 0x80, 0xff, 0xff, 0xff, 0xff};
-  register FXint p,q;
-  FXCALLOC(&data,FXuchar,32*32*3);
-  for(p=q=0; p<32*32*3; p+=3,q+=1){
-    if(!(jpeg_bits[q>>3]&(1<<(q&7)))) data[p+0]=data[p+1]=data[p+2]=255;
+  register FXint p;
+  FXMALLOC(&data,FXColor,32*32);
+  for(p=0; p<32*32; p++){
+    data[p]=(jpeg_bits[p>>3]&(1<<(p&7))) ? FXRGB(0,0,0) : FXRGB(255,255,255);
     }
-  transp=0;
   width=32;
   height=32;
-  return FALSE;
+  return TRUE;
   }
 
 
 // Stub routine
-FXbool fxsaveJPG(FXStream&,const FXuchar*,FXColor,FXint,FXint,FXint){
+FXbool fxsaveJPG(FXStream&,const FXColor*,FXint,FXint,FXint){
   return FALSE;
   }
 
 
 #endif
+
+}
