@@ -3,7 +3,7 @@
 *                          X P M   I n p u t / O u t p u t                      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2000,2003 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fxxpmio.cpp,v 1.17.4.1 2003/05/12 11:04:35 fox Exp $                      *
+* $Id: fxxpmio.cpp,v 1.31 2003/09/05 06:06:42 fox Exp $                         *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -35,53 +35,81 @@
   - If the image is rendered opaque, the transparent is close to white.
   - References:
       http://www-sop.inria.fr/koala/lehors/xpm.html
+  - XPM reader/writer is tweaked so that an XPM written to FXStream
+    can be read back in and read exactly as many bytes as were written.
+  - There may be other comment blocks in the file
 */
 
-#define MAXPRINTABLE  92
-#define MAXVALUE      96
-#define ALPHA_COLOR   FXRGB(252,253,254)
-//#define ALPHA_COLOR   FXRGB(178,192,220)
+#define MAXPRINTABLE    92
+#define MAXVALUE        96
+#define HASH1(x,n)      (((unsigned int)(x)*13)%(n))            // Number [0..n-1]
+#define HASH2(x,n)      (1|(((unsigned int)(x)*17)%((n)-1)))    // Number [1..n-2]
 
-
-#define HASH1(x,n) (((unsigned int)(x)*13)%(n))           // Number [0..n-1]
-#define HASH2(x,n) (1|(((unsigned int)(x)*17)%((n)-1)))   // Number [1..n-1]
+using namespace FX;
 
 
 /*******************************************************************************/
 
-/// Load an X Pixmap from array of strings
-extern FXAPI FXbool fxloadXPM(const FXchar **pix,FXuchar*& data,FXColor& transp,FXint& width,FXint& height);
+namespace FX {
 
 
-/// Load an X Pixmap file from a stream
-extern FXAPI FXbool fxloadXPM(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXint& height);
-
-
-/// Save an X Pixmap file to a stream
-extern FXAPI FXbool fxsaveXPM(FXStream& store,const FXuchar *data,FXColor transp,FXint width,FXint height);
-
-
-/*******************************************************************************/
+// Little helper
+static void readbuffer(FXStream& store,FXchar* buffer,FXuint size){
+  register FXuint i=0;
+  FXchar ch;
+  while(!store.eof()){
+    store >> ch;
+    if(ch=='"') break;
+    if(ch!='/') continue;
+    store >> ch;
+    if(ch!='*') continue;
+    while(!store.eof()){
+      store >> ch;
+      if(ch!='*') continue;
+      store >> ch;
+      if(ch=='/') break;
+      }
+    }
+  while(!store.eof() && i<size){
+    store >> ch;
+    if(ch=='"') break;
+    buffer[i++]=ch;
+    }
+  while(!store.eof()){
+    store >> ch;
+    if(ch=='\n') break;
+    }
+  buffer[i]=0;
+  }
 
 
 // Load image from array of strings
-FXbool fxloadXPM(const FXchar **pixels,FXuchar*& data,FXColor& transp,FXint& width,FXint& height){
+FXbool fxloadXPM(const FXchar **pixels,FXColor*& data,FXint& width,FXint& height){
   FXuint    ncolortable,index,ncolors,cpp,c;
   FXColor  *colortable=NULL;
   FXint     ww,hh,i,j;
   FXchar    name[100];
   FXchar    type[10];
   const FXchar *ptr;
-  FXuchar  *pix;
+  FXColor  *pix;
   FXColor   color;
 
+  // Null out
   data=NULL;
+  width=0;
+  height=0;
+
+  // NULL pointer passed in
+  if(!pixels) return FALSE;
+
+  // Read pointer
   ptr=*pixels++;
 
-  // Parse values
-  sscanf(ptr,"%d %d %u %u",&ww,&hh,&ncolors,&cpp);
+  // No size description line
+  if(!ptr) return FALSE;
 
-  //FXTRACE((150,"fxloadXPM: width=%d height=%d ncolors=%d cpp=%d\n",ww,hh,ncolors,cpp));
+  // Parse size description
+  sscanf(ptr,"%d %d %u %u",&ww,&hh,&ncolors,&cpp);
 
   // Check size
   if(ww<1 || hh<1 || ww>16384 || hh>16384) return FALSE;
@@ -96,64 +124,56 @@ FXbool fxloadXPM(const FXchar **pixels,FXuchar*& data,FXColor& transp,FXint& wid
   if(ncolortable<ncolors) return FALSE;
 
   // Make color table
-  FXMALLOC(&colortable,FXColor,ncolortable);
-  if(!colortable){return FALSE;}
+  if(!FXMALLOC(&colortable,FXColor,ncolortable)){
+    return FALSE;
+    }
 
   // Read the color table
   for(c=0; c<ncolors; c++){
     ptr=*pixels++;
-    if(!ptr){FXFREE(&colortable);return FALSE;}
     index=*ptr++ - ' ';
     if(cpp==2) index=index+MAXVALUE*(*ptr++ - ' ');
-    if(index>ncolortable){FXFREE(&colortable);return FALSE;}
+    if(index>ncolortable){
+      FXFREE(&colortable);
+      return FALSE;
+      }
     sscanf(ptr,"%s %s",type,name);
     if(type[0]!='c') sscanf(ptr,"%*s %*s %s %s",type,name);
     color=fxcolorfromname(name);
-    if(color==0) color=transp=ALPHA_COLOR;
     colortable[index]=color;
-    //FXTRACE((150,"fxloadXPM: color %d: index=%d type=%s name=%s color=#%08x\n",c,index,type,name,color));
     }
 
-  // Allocate pixels
-  FXMALLOC(&data,FXuchar,3*ww*hh);
-  if(!data){FXFREE(&colortable);return FALSE;}
+  // Try allocate pixels
+  if(!FXMALLOC(&data,FXColor,ww*hh)){
+    FXFREE(&colortable);
+    return FALSE;
+    }
 
   // Read the pixels
   for(i=0,pix=data; i<hh; i++){
     ptr=*pixels++;
-    if(!ptr){FXFREE(&colortable);FXFREE(&data);return FALSE;}
     for(j=0; j<ww; j++){
       index=*ptr++ - ' ';
       if(cpp==2) index=index+MAXVALUE*(*ptr++ - ' ');
-      if(index>ncolortable){FXFREE(&colortable);FXFREE(&data);return FALSE;}
-      color=colortable[index];
-      *pix++=FXREDVAL(color);
-      *pix++=FXGREENVAL(color);
-      *pix++=FXBLUEVAL(color);
+      if(index>ncolortable){
+        FXFREE(&colortable);
+        FXFREE(&data);
+        return FALSE;
+        }
+      *pix++=colortable[index];
       }
     }
   FXFREE(&colortable);
   width=ww;
   height=hh;
-  //FXTRACE((150,"fxloadXPM: done\n"));
   return TRUE;
   }
 
 
 /*******************************************************************************/
 
-
-static void readbuffer(FXStream& store,FXchar* buffer,FXuint size){
-  FXchar ch;
-  while((store.status()!=FXStreamEnd) && (store>>ch,ch!='"'));
-  while((store.status()!=FXStreamEnd) && (store>>ch,ch!='"') && size--) *buffer++=ch;
-  while((store.status()!=FXStreamEnd) && (store>>ch,ch!='\n'));
-  *buffer=0;
-  }
-
-
 // Load image from stream
-FXbool fxloadXPM(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXint& height){
+FXbool fxloadXPM(FXStream& store,FXColor*& data,FXint& width,FXint& height){
   FXuint    ncolortable,index,ncolors,cpp,c;
   FXColor  *colortable=NULL;
   FXint     ww,hh,i,j;
@@ -162,19 +182,20 @@ FXbool fxloadXPM(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXi
   FXchar    type[10];
   FXchar    ch;
   FXchar   *ptr;
-  FXuchar  *pix;
+  FXColor  *pix;
   FXColor   color;
 
+  // Null out
   data=NULL;
+  width=0;
+  height=0;
 
   readbuffer(store,header,sizeof(header));
-  if(store.status()!=FXStreamOK) return FALSE;
+  if(store.eof()) return FALSE;
 
-  // Parse values
+  // Parse size description
   sscanf(header,"%d %d %u %u",&ww,&hh,&ncolors,&cpp);
 
-  //FXTRACE((150,"fxloadXPM: width=%d height=%d ncolors=%d cpp=%d\n",ww,hh,ncolors,cpp));
-
   // Check size
   if(ww<1 || hh<1 || ww>16384 || hh>16384) return FALSE;
 
@@ -184,53 +205,59 @@ FXbool fxloadXPM(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXi
   // Color lookup table
   ncolortable = (cpp==1) ? MAXVALUE : MAXVALUE*MAXVALUE;
 
+  FXTRACE((50,"fxloadXPM: width=%d height=%d ncolors=%d cpp=%d\n",width,height,ncolors,cpp));
+
   // Check if characters/color is consistent with number of colors
   if(ncolortable<ncolors) return FALSE;
 
   // Make color table
-  FXMALLOC(&colortable,FXColor,ncolortable);
-  if(!colortable){return FALSE;}
+  if(!FXMALLOC(&colortable,FXColor,ncolortable)){
+    return FALSE;
+    }
 
   // Read the color table
   for(c=0; c<ncolors; c++){
     readbuffer(store,header,sizeof(header));
-    if(store.status()!=FXStreamOK) return FALSE;
+    if(store.eof()) return FALSE;
     ptr=header;
     index=*ptr++ - ' ';
     if(cpp==2) index=index+MAXVALUE*(*ptr++ - ' ');
-    if(index>ncolortable){FXFREE(&colortable);return FALSE;}
+    if(index>ncolortable){
+      FXFREE(&colortable);
+      return FALSE;
+      }
     sscanf(ptr,"%s %s",type,name);
     if(type[0]!='c') sscanf(ptr,"%*s %*s %s %s",type,name);
     color=fxcolorfromname(name);
-    if(color==0) color=transp=ALPHA_COLOR;
     colortable[index]=color;
-    //FXTRACE((150,"fxloadXPM: color %d: index=%d type=%s name=%s color=#%08x\n",c,index,type,name,color));
     }
 
-  // Allocate pixels
-  FXMALLOC(&data,FXuchar,3*ww*hh);
-  if(!data){FXFREE(&colortable);return FALSE;}
+  // Try allocate pixels
+  if(!FXMALLOC(&data,FXColor,ww*hh)){
+    FXFREE(&colortable);
+    return FALSE;
+    }
 
   // Read the pixels
   for(i=0,pix=data; i<hh; i++){
-    while((store.status()!=FXStreamEnd) && (store>>ch,ch!='"'));
+    while(!store.eof() && (store>>ch,ch!='"'));
     for(j=0; j<ww; j++){
       store >> ch;
       index=ch-' ';
       if(cpp==2){ store >> ch; index=index+MAXVALUE*(ch-' '); }
-      if(index>ncolortable){FXFREE(&colortable);FXFREE(&data);return FALSE;}
-      color=colortable[index];
-      *pix++=FXREDVAL(color);
-      *pix++=FXGREENVAL(color);
-      *pix++=FXBLUEVAL(color);
+      if(index>ncolortable){
+        FXFREE(&colortable);
+        FXFREE(&data);
+        return FALSE;
+        }
+      *pix++=colortable[index];
       }
-    while((store.status()!=FXStreamEnd) && (store>>ch,ch!='\n'));
-    if(store.status()!=FXStreamOK) return FALSE;
+    while(!store.eof() && (store>>ch,ch!='\n'));
+    if(store.eof()) return FALSE;
     }
   FXFREE(&colortable);
   width=ww;
   height=hh;
-  //FXTRACE((150,"fxloadXPM: done\n"));
   return TRUE;
   }
 
@@ -239,31 +266,32 @@ FXbool fxloadXPM(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXi
 
 
 // Save image to a stream
-FXbool fxsaveXPM(FXStream& store,const FXuchar *data,FXColor transp,FXint width,FXint height){
+FXbool fxsaveXPM(FXStream& store,const FXColor *data,FXint width,FXint height){
   const FXchar printable[]=" .XoO+@#$%&*=-;:>,<1234567890qwertyuipasdfghjklzxcvbnmMNBVCZASDFGHJKLPIUYTREWQ!~^/()_`'][{}|";
   const FXchar quote='"';
   const FXchar comma=',';
   const FXchar newline='\n';
-  FXuchar   rmap[256];      // Red colormap
-  FXuchar   gmap[256];      // Green colormap
-  FXuchar   bmap[256];      // Blue colormap
+  FXColor   colormap[256];
   FXint     numpixels=width*height;
   FXint     ncolors,cpp,len,i,j,c1,c2,ok;
-  FXchar    header[200];
+  FXchar    buffer[200];
   FXColor   color;
   FXuchar  *pixels,*ptr,pix;
 
+  // Must make sense
+  if(!data || width<=0 || height<=0) return FALSE;
+
   // Allocate temp buffer for pixels
   if(!FXMALLOC(&pixels,FXuchar,numpixels)) return FALSE;
 
   // First, try EZ quantization, because it is exact; a previously
   // loaded XPM will be re-saved with exactly the same colors.
-  ok=fxezquantize(pixels,data,rmap,gmap,bmap,ncolors,width,height,256);
+  ok=fxezquantize(pixels,data,colormap,ncolors,width,height,256);
 
   if(!ok){
 
-    // Floyd-Steinberg quantize full 24 bpp to 256 colors, organized as 3:3:2
-    fxfsquantize(pixels,data,rmap,gmap,bmap,ncolors,width,height,256);
+    // Floyd-Steinberg quantize full 32 bpp to 256 colors, organized as 3:3:2
+    fxfsquantize(pixels,data,colormap,ncolors,width,height,256);
     }
 
   FXASSERT(ncolors<=256);
@@ -271,27 +299,25 @@ FXbool fxsaveXPM(FXStream& store,const FXuchar *data,FXColor transp,FXint width,
   // How many characters needed to represent one pixel, characters per line
   cpp=(ncolors>MAXPRINTABLE)?2:1;
 
-  //FXTRACE((150,"fxsaveXPM: width=%d height=%d ncolors=%d cpp=%d\n",width,height,ncolors,cpp));
-
   // Save header
   store.save("/* XPM */\nstatic char * image[] = {\n",36);
 
   // Save values
-  len=sprintf(header,"\"%d %d %d %d\",\n",width,height,ncolors,cpp);
-  store.save(header,len);
+  len=sprintf(buffer,"\"%d %d %d %d\",\n",width,height,ncolors,cpp);
+  store.save(buffer,len);
 
   // Save the colors
   for(i=0; i<ncolors; i++){
-    color=FXRGB(rmap[i],gmap[i],bmap[i]);
+    color=colormap[i];
     c1=printable[i%MAXPRINTABLE];
     c2=printable[i/MAXPRINTABLE];
-    if(color!=transp){
-      len=sprintf(header,"\"%c%c c #%02x%02x%02x\",\n",c1,c2,rmap[i],gmap[i],bmap[i]);
-      store.save(header,len);
+    if(FXALPHAVAL(color)){
+      len=sprintf(buffer,"\"%c%c c #%02x%02x%02x\",\n",c1,c2,FXREDVAL(color),FXGREENVAL(color),FXBLUEVAL(color));
+      store.save(buffer,len);
       }
     else{
-      len=sprintf(header,"\"%c%c c None\",\n",c1,c2);
-      store.save(header,len);
+      len=sprintf(buffer,"\"%c%c c None\",\n",c1,c2);
+      store.save(buffer,len);
       }
     }
 
@@ -314,9 +340,8 @@ FXbool fxsaveXPM(FXStream& store,const FXuchar *data,FXColor transp,FXint width,
     }
   store.save("};\n",3);
   FXFREE(&pixels);
-  //FXTRACE((150,"fxsaveXPM: done\n"));
   return TRUE;
   }
 
-
+}
 
